#nullable enable
using Canon.Core.Abstractions;
using Canon.Core.GrammarParser;
using Canon.Core.Enums;
namespace Canon.Server.Services;

public class GeneratedTransformer : ITransformer
{
    private IDictionary<TerminatorBase, string> _shiftPointers;

    public string Name { get; }

    public IDictionary<Terminator, ReduceInformation> ReduceTable { get; }

    public IDictionary<TerminatorBase, ITransformer> ShiftTable { get; }

    public GeneratedTransformer(Dictionary<TerminatorBase, string> shiftTable,
        Dictionary<Terminator, ReduceInformation> reduceTable, string name)
    {
        ReduceTable = reduceTable;
        ShiftTable = new Dictionary<TerminatorBase, ITransformer>();
        _shiftPointers = shiftTable;
        Name = name;
    }

    public GeneratedTransformer()
    {
        ReduceTable = new Dictionary<Terminator, ReduceInformation>();
        ShiftTable = new Dictionary<TerminatorBase, ITransformer>();
        _shiftPointers = new Dictionary<TerminatorBase, string>();
        Name = Guid.NewGuid().ToString();
    }

    public void ConstructShiftTable(Dictionary<string, GeneratedTransformer> transformers)
    {
        foreach (KeyValuePair<TerminatorBase,string> pair in _shiftPointers)
        {
            ShiftTable.Add(pair.Key, transformers[pair.Value]);
        }
    }

    public override bool Equals(object? obj)
    {
        if (obj is not GeneratedTransformer other)
        {
            return false;
        }

        return Name == other.Name;
    }

    public override int GetHashCode() => Name.GetHashCode();
}
public class GeneratedGrammarParser : IGrammarParser
{
	private static readonly Dictionary<string, GeneratedTransformer> s_transformers = new()
	{
		{ "1943566a-342f-47b2-b2fe-f9cd93e7301b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ProgramStruct), "11fadf53-074e-4c41-beec-025f2a388a34"}, { new NonTerminator(NonTerminatorType.ProgramHead), "25849629-192d-438f-82d4-e2e1ce303914"}, { new Terminator(KeywordType.Program), "4e826861-3f30-40d6-a8d6-6398d4458586"},}, new Dictionary<Terminator, ReduceInformation>{ }, "1943566a-342f-47b2-b2fe-f9cd93e7301b") },
		{ "11fadf53-074e-4c41-beec-025f2a388a34", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { Terminator.EndTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.StartNonTerminator))}, }, "11fadf53-074e-4c41-beec-025f2a388a34") },
		{ "25849629-192d-438f-82d4-e2e1ce303914", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.Semicolon), "a45cd9db-a8fe-4996-992f-ce0d13ac64e3"},}, new Dictionary<Terminator, ReduceInformation>{ }, "25849629-192d-438f-82d4-e2e1ce303914") },
		{ "4e826861-3f30-40d6-a8d6-6398d4458586", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { Terminator.IdentifierTerminator, "889ec2c2-fddc-4d73-b806-3d4115be6fba"},}, new Dictionary<Terminator, ReduceInformation>{ }, "4e826861-3f30-40d6-a8d6-6398d4458586") },
		{ "a45cd9db-a8fe-4996-992f-ce0d13ac64e3", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ProgramBody), "dffb56e4-406e-43cc-92e1-659066f72c83"}, { new NonTerminator(NonTerminatorType.ConstDeclarations), "36b870f8-7003-4f15-8b17-0e8d8ba4e7ca"}, { new Terminator(KeywordType.Const), "122c667d-07d6-4739-bd55-c481822af1ca"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Begin), new ReduceInformation(0, new NonTerminator(NonTerminatorType.ConstDeclarations))}, { new Terminator(KeywordType.Procedure), new ReduceInformation(0, new NonTerminator(NonTerminatorType.ConstDeclarations))}, { new Terminator(KeywordType.Function), new ReduceInformation(0, new NonTerminator(NonTerminatorType.ConstDeclarations))}, { new Terminator(KeywordType.Var), new ReduceInformation(0, new NonTerminator(NonTerminatorType.ConstDeclarations))}, }, "a45cd9db-a8fe-4996-992f-ce0d13ac64e3") },
		{ "889ec2c2-fddc-4d73-b806-3d4115be6fba", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.LeftParenthesis), "af228c00-74f1-4a53-81d5-9ebc9ced2a4b"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.ProgramHead))}, }, "889ec2c2-fddc-4d73-b806-3d4115be6fba") },
		{ "dffb56e4-406e-43cc-92e1-659066f72c83", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.Period), "24fc7ea4-6686-4153-98b6-6ba2db1dab19"},}, new Dictionary<Terminator, ReduceInformation>{ }, "dffb56e4-406e-43cc-92e1-659066f72c83") },
		{ "36b870f8-7003-4f15-8b17-0e8d8ba4e7ca", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.VarDeclarations), "80f05294-9904-40db-93e4-70fa1cacdc9e"}, { new Terminator(KeywordType.Var), "429ae25c-2b70-4bd9-91dd-bab1dd774e45"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Begin), new ReduceInformation(0, new NonTerminator(NonTerminatorType.VarDeclarations))}, { new Terminator(KeywordType.Procedure), new ReduceInformation(0, new NonTerminator(NonTerminatorType.VarDeclarations))}, { new Terminator(KeywordType.Function), new ReduceInformation(0, new NonTerminator(NonTerminatorType.VarDeclarations))}, }, "36b870f8-7003-4f15-8b17-0e8d8ba4e7ca") },
		{ "122c667d-07d6-4739-bd55-c481822af1ca", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ConstDeclaration), "9c1b0c72-789a-402f-9a91-a49624093890"}, { Terminator.IdentifierTerminator, "8ed7d885-b402-4805-b7c7-a25113fab7a4"},}, new Dictionary<Terminator, ReduceInformation>{ }, "122c667d-07d6-4739-bd55-c481822af1ca") },
		{ "af228c00-74f1-4a53-81d5-9ebc9ced2a4b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.IdentifierList), "dbb1afc4-407f-4911-ae7b-30ec34fe6671"}, { Terminator.IdentifierTerminator, "917c8a8e-7c76-440b-b939-40362b1c4425"},}, new Dictionary<Terminator, ReduceInformation>{ }, "af228c00-74f1-4a53-81d5-9ebc9ced2a4b") },
		{ "24fc7ea4-6686-4153-98b6-6ba2db1dab19", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { Terminator.EndTerminator, new ReduceInformation(4, new NonTerminator(NonTerminatorType.ProgramStruct))}, }, "24fc7ea4-6686-4153-98b6-6ba2db1dab19") },
		{ "80f05294-9904-40db-93e4-70fa1cacdc9e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.SubprogramDeclarations), "00db10a9-57c4-48e0-9a01-42c4c6f9dc85"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Begin), new ReduceInformation(0, new NonTerminator(NonTerminatorType.SubprogramDeclarations))}, { new Terminator(KeywordType.Procedure), new ReduceInformation(0, new NonTerminator(NonTerminatorType.SubprogramDeclarations))}, { new Terminator(KeywordType.Function), new ReduceInformation(0, new NonTerminator(NonTerminatorType.SubprogramDeclarations))}, }, "80f05294-9904-40db-93e4-70fa1cacdc9e") },
		{ "429ae25c-2b70-4bd9-91dd-bab1dd774e45", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.VarDeclaration), "fd277ecb-f0dc-4dc9-b201-4736ff33e16f"}, { new NonTerminator(NonTerminatorType.IdentifierList), "98e439b2-6456-497d-be3a-2d44c50a720f"}, { Terminator.IdentifierTerminator, "3ed14f62-8e9f-4c1a-8e1e-5d522192093b"},}, new Dictionary<Terminator, ReduceInformation>{ }, "429ae25c-2b70-4bd9-91dd-bab1dd774e45") },
		{ "9c1b0c72-789a-402f-9a91-a49624093890", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.Semicolon), "1a2df7a9-ea91-4a45-bdca-02310abea165"},}, new Dictionary<Terminator, ReduceInformation>{ }, "9c1b0c72-789a-402f-9a91-a49624093890") },
		{ "8ed7d885-b402-4805-b7c7-a25113fab7a4", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(OperatorType.Equal), "a8ab64d5-f5c1-4e41-83d6-70c056c00d9f"},}, new Dictionary<Terminator, ReduceInformation>{ }, "8ed7d885-b402-4805-b7c7-a25113fab7a4") },
		{ "dbb1afc4-407f-4911-ae7b-30ec34fe6671", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "326e33e5-9da7-47c1-9fe8-c3f6830658eb"}, { new Terminator(DelimiterType.Comma), "2162f3f5-b2e2-405a-a246-a76e411ec64a"},}, new Dictionary<Terminator, ReduceInformation>{ }, "dbb1afc4-407f-4911-ae7b-30ec34fe6671") },
		{ "917c8a8e-7c76-440b-b939-40362b1c4425", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.IdentifierList))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.IdentifierList))}, }, "917c8a8e-7c76-440b-b939-40362b1c4425") },
		{ "00db10a9-57c4-48e0-9a01-42c4c6f9dc85", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.CompoundStatement), "3c842ecd-161e-4219-bcbe-2fc862062d4d"}, { new Terminator(KeywordType.Begin), "3c8b05b9-b1a1-4990-bf24-ed7c4f6cf797"}, { new NonTerminator(NonTerminatorType.Subprogram), "f5e86f47-fa7f-4fb1-8376-b0e71e1036de"}, { new NonTerminator(NonTerminatorType.SubprogramHead), "05968b27-77ba-4385-b192-3e4431fa3771"}, { new Terminator(KeywordType.Procedure), "73db2413-f84a-4a8c-8a4d-2128d40caa83"}, { new Terminator(KeywordType.Function), "32772270-a047-4624-9131-2ddd9bdac40d"},}, new Dictionary<Terminator, ReduceInformation>{ }, "00db10a9-57c4-48e0-9a01-42c4c6f9dc85") },
		{ "fd277ecb-f0dc-4dc9-b201-4736ff33e16f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.Semicolon), "ba4a64d0-cc18-4be1-a919-35b613b7f3b8"},}, new Dictionary<Terminator, ReduceInformation>{ }, "fd277ecb-f0dc-4dc9-b201-4736ff33e16f") },
		{ "98e439b2-6456-497d-be3a-2d44c50a720f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.Colon), "6af85cd8-db8e-4f69-b339-118aceb9d58e"}, { new Terminator(DelimiterType.Comma), "19b90e6e-1e3b-4063-b92b-d3f3ad667ca2"},}, new Dictionary<Terminator, ReduceInformation>{ }, "98e439b2-6456-497d-be3a-2d44c50a720f") },
		{ "3ed14f62-8e9f-4c1a-8e1e-5d522192093b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Colon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.IdentifierList))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.IdentifierList))}, }, "3ed14f62-8e9f-4c1a-8e1e-5d522192093b") },
		{ "1a2df7a9-ea91-4a45-bdca-02310abea165", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { Terminator.IdentifierTerminator, "96728733-8b22-4516-b05e-bdff14c767c6"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Begin), new ReduceInformation(3, new NonTerminator(NonTerminatorType.ConstDeclarations))}, { new Terminator(KeywordType.Procedure), new ReduceInformation(3, new NonTerminator(NonTerminatorType.ConstDeclarations))}, { new Terminator(KeywordType.Function), new ReduceInformation(3, new NonTerminator(NonTerminatorType.ConstDeclarations))}, { new Terminator(KeywordType.Var), new ReduceInformation(3, new NonTerminator(NonTerminatorType.ConstDeclarations))}, }, "1a2df7a9-ea91-4a45-bdca-02310abea165") },
		{ "a8ab64d5-f5c1-4e41-83d6-70c056c00d9f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ConstValue), "013ac741-2d38-4f10-beba-f853a835c8e7"}, { new Terminator(OperatorType.Plus), "6fdf7012-c004-46a8-b259-c2939925031c"}, { new Terminator(OperatorType.Minus), "c654570d-4062-4758-8e9c-14fe4cc2a227"}, { Terminator.NumberTerminator, "2be558a7-1c18-499f-8ec3-9cf689fe0193"}, { Terminator.CharacterTerminator, "085e4679-3deb-40b2-a35e-81d6eac5e1ae"},}, new Dictionary<Terminator, ReduceInformation>{ }, "a8ab64d5-f5c1-4e41-83d6-70c056c00d9f") },
		{ "326e33e5-9da7-47c1-9fe8-c3f6830658eb", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(5, new NonTerminator(NonTerminatorType.ProgramHead))}, }, "326e33e5-9da7-47c1-9fe8-c3f6830658eb") },
		{ "2162f3f5-b2e2-405a-a246-a76e411ec64a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { Terminator.IdentifierTerminator, "55d12708-13a6-4c61-bf69-8c848ac4d979"},}, new Dictionary<Terminator, ReduceInformation>{ }, "2162f3f5-b2e2-405a-a246-a76e411ec64a") },
		{ "3c842ecd-161e-4219-bcbe-2fc862062d4d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Period), new ReduceInformation(4, new NonTerminator(NonTerminatorType.ProgramBody))}, }, "3c842ecd-161e-4219-bcbe-2fc862062d4d") },
		{ "3c8b05b9-b1a1-4990-bf24-ed7c4f6cf797", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.StatementList), "c4430166-109c-4e04-921e-80af6f94a55f"}, { new NonTerminator(NonTerminatorType.Statement), "382f5704-ba25-4f9d-a658-af06ba9c8ecc"}, { new NonTerminator(NonTerminatorType.Variable), "ccb9ddb8-4ca9-4498-906e-81b654135662"}, { Terminator.IdentifierTerminator, "764301e0-1e52-4941-80fc-f768fabb848a"}, { new NonTerminator(NonTerminatorType.ProcedureCall), "d19e5b10-7ef3-4981-8c0b-deca4bfff866"}, { new NonTerminator(NonTerminatorType.CompoundStatement), "67b6f07f-8024-4a8f-8f06-3e29ee8141d8"}, { new Terminator(KeywordType.If), "1139d083-a185-4a24-a86e-2dfa3d0514e2"}, { new Terminator(KeywordType.For), "8337e624-bbce-4b9a-8231-b6b264eefd7f"}, { new Terminator(KeywordType.Begin), "5c23381d-aee0-4773-ab73-20aa2a344931"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, }, "3c8b05b9-b1a1-4990-bf24-ed7c4f6cf797") },
		{ "f5e86f47-fa7f-4fb1-8376-b0e71e1036de", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.Semicolon), "957827df-f48f-442b-bec7-ab2080e32435"},}, new Dictionary<Terminator, ReduceInformation>{ }, "f5e86f47-fa7f-4fb1-8376-b0e71e1036de") },
		{ "05968b27-77ba-4385-b192-3e4431fa3771", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.Semicolon), "57f035be-fdbb-44b5-9108-07251e89d821"},}, new Dictionary<Terminator, ReduceInformation>{ }, "05968b27-77ba-4385-b192-3e4431fa3771") },
		{ "73db2413-f84a-4a8c-8a4d-2128d40caa83", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { Terminator.IdentifierTerminator, "ea565722-ec3d-4b33-ac27-ff9d72acfddc"},}, new Dictionary<Terminator, ReduceInformation>{ }, "73db2413-f84a-4a8c-8a4d-2128d40caa83") },
		{ "32772270-a047-4624-9131-2ddd9bdac40d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { Terminator.IdentifierTerminator, "2d90f8ce-a1fd-425e-95eb-21775de2734e"},}, new Dictionary<Terminator, ReduceInformation>{ }, "32772270-a047-4624-9131-2ddd9bdac40d") },
		{ "ba4a64d0-cc18-4be1-a919-35b613b7f3b8", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.IdentifierList), "7b79d969-3543-466c-8c4f-2403ae51406c"}, { Terminator.IdentifierTerminator, "3ed14f62-8e9f-4c1a-8e1e-5d522192093b"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Begin), new ReduceInformation(3, new NonTerminator(NonTerminatorType.VarDeclarations))}, { new Terminator(KeywordType.Procedure), new ReduceInformation(3, new NonTerminator(NonTerminatorType.VarDeclarations))}, { new Terminator(KeywordType.Function), new ReduceInformation(3, new NonTerminator(NonTerminatorType.VarDeclarations))}, }, "ba4a64d0-cc18-4be1-a919-35b613b7f3b8") },
		{ "6af85cd8-db8e-4f69-b339-118aceb9d58e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Type), "76ce47d7-0021-46d0-901b-57e7acf77051"}, { new NonTerminator(NonTerminatorType.BasicType), "03fa10c0-f221-4476-8ac3-a84cb91df27a"}, { new Terminator(KeywordType.Array), "6208cee5-0a9f-4d9e-9149-5efa773fbcf9"}, { new Terminator(KeywordType.Integer), "e3c9fab2-9cd8-44ad-a33c-54482b5323d5"}, { new Terminator(KeywordType.Real), "3bb38fcd-bf5c-401a-a2e4-f6f4d91bc650"}, { new Terminator(KeywordType.Boolean), "01a76185-0283-464f-b7d6-9f5bb8517edd"}, { new Terminator(KeywordType.Character), "ee488773-f67c-418c-a1a5-ddd934d4253b"},}, new Dictionary<Terminator, ReduceInformation>{ }, "6af85cd8-db8e-4f69-b339-118aceb9d58e") },
		{ "19b90e6e-1e3b-4063-b92b-d3f3ad667ca2", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { Terminator.IdentifierTerminator, "d522700d-0e6e-4a72-ad24-e765c60c6311"},}, new Dictionary<Terminator, ReduceInformation>{ }, "19b90e6e-1e3b-4063-b92b-d3f3ad667ca2") },
		{ "96728733-8b22-4516-b05e-bdff14c767c6", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(OperatorType.Equal), "e72d1c7d-c459-4b11-bc70-edd87cb2e149"},}, new Dictionary<Terminator, ReduceInformation>{ }, "96728733-8b22-4516-b05e-bdff14c767c6") },
		{ "013ac741-2d38-4f10-beba-f853a835c8e7", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.ConstDeclaration))}, }, "013ac741-2d38-4f10-beba-f853a835c8e7") },
		{ "6fdf7012-c004-46a8-b259-c2939925031c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { Terminator.NumberTerminator, "bb2dba84-2cec-4600-a009-6cd6f2e762c4"},}, new Dictionary<Terminator, ReduceInformation>{ }, "6fdf7012-c004-46a8-b259-c2939925031c") },
		{ "c654570d-4062-4758-8e9c-14fe4cc2a227", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { Terminator.NumberTerminator, "71ebc44a-d6ca-42cb-a8ae-2efee62c7f0a"},}, new Dictionary<Terminator, ReduceInformation>{ }, "c654570d-4062-4758-8e9c-14fe4cc2a227") },
		{ "2be558a7-1c18-499f-8ec3-9cf689fe0193", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.ConstValue))}, }, "2be558a7-1c18-499f-8ec3-9cf689fe0193") },
		{ "085e4679-3deb-40b2-a35e-81d6eac5e1ae", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.ConstValue))}, }, "085e4679-3deb-40b2-a35e-81d6eac5e1ae") },
		{ "55d12708-13a6-4c61-bf69-8c848ac4d979", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdentifierList))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdentifierList))}, }, "55d12708-13a6-4c61-bf69-8c848ac4d979") },
		{ "c4430166-109c-4e04-921e-80af6f94a55f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(KeywordType.End), "4758be8f-9722-47a6-9e0c-369b492f3f81"}, { new Terminator(DelimiterType.Semicolon), "f83f97d0-e0c9-462f-b6fa-0bc1226d63fe"},}, new Dictionary<Terminator, ReduceInformation>{ }, "c4430166-109c-4e04-921e-80af6f94a55f") },
		{ "382f5704-ba25-4f9d-a658-af06ba9c8ecc", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.StatementList))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.StatementList))}, }, "382f5704-ba25-4f9d-a658-af06ba9c8ecc") },
		{ "ccb9ddb8-4ca9-4498-906e-81b654135662", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(OperatorType.Assign), "289b2a05-55d5-4771-a2ce-3999e7cfead1"},}, new Dictionary<Terminator, ReduceInformation>{ }, "ccb9ddb8-4ca9-4498-906e-81b654135662") },
		{ "764301e0-1e52-4941-80fc-f768fabb848a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(OperatorType.Assign), "f3630dbe-c2d7-43ee-a30b-055d5d71520d"}, { new NonTerminator(NonTerminatorType.IdVarPart), "48402e50-8efc-4ee1-b0ac-b318b198cdc6"}, { new Terminator(DelimiterType.LeftSquareBracket), "ad1a1d2d-24d0-4c6e-8654-b1025dbada2f"}, { new Terminator(DelimiterType.LeftParenthesis), "abf8b682-4ac1-45bc-9b28-eb89f4ae0216"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(OperatorType.Assign), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.ProcedureCall))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.ProcedureCall))}, }, "764301e0-1e52-4941-80fc-f768fabb848a") },
		{ "d19e5b10-7ef3-4981-8c0b-deca4bfff866", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Statement))}, }, "d19e5b10-7ef3-4981-8c0b-deca4bfff866") },
		{ "67b6f07f-8024-4a8f-8f06-3e29ee8141d8", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Statement))}, }, "67b6f07f-8024-4a8f-8f06-3e29ee8141d8") },
		{ "1139d083-a185-4a24-a86e-2dfa3d0514e2", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "8d60fecf-8eba-4d5c-adcd-5b92e16c3bc4"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "9372190c-03b2-4dd8-b6d0-ed9a16c071cc"}, { new NonTerminator(NonTerminatorType.Term), "4101b78d-0b68-4d0a-819e-ec5221d8ad24"}, { new NonTerminator(NonTerminatorType.Factor), "87546012-c006-4323-84d2-09d75a8399c5"}, { Terminator.NumberTerminator, "fa97211a-80b7-4975-9eee-548312d9b847"}, { new NonTerminator(NonTerminatorType.Variable), "18fb7818-280d-424d-acd7-c30c11393bbf"}, { new Terminator(DelimiterType.LeftParenthesis), "b9e87be5-24b1-455e-8377-274fab33e19a"}, { Terminator.IdentifierTerminator, "2568b015-15cf-42e5-bacf-64d5d56c3b0e"}, { new Terminator(KeywordType.Not), "bb6682cd-c5f2-49f5-a560-001ce41b69a5"}, { new Terminator(OperatorType.Minus), "0ebd6126-1360-4d5b-ba9c-0b49e8d814fa"},}, new Dictionary<Terminator, ReduceInformation>{ }, "1139d083-a185-4a24-a86e-2dfa3d0514e2") },
		{ "8337e624-bbce-4b9a-8231-b6b264eefd7f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { Terminator.IdentifierTerminator, "9b162179-6984-440a-a586-81893a049329"},}, new Dictionary<Terminator, ReduceInformation>{ }, "8337e624-bbce-4b9a-8231-b6b264eefd7f") },
		{ "5c23381d-aee0-4773-ab73-20aa2a344931", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.StatementList), "38ae7a30-a988-4208-b44d-5ed83fb9e099"}, { new NonTerminator(NonTerminatorType.Statement), "382f5704-ba25-4f9d-a658-af06ba9c8ecc"}, { new NonTerminator(NonTerminatorType.Variable), "ccb9ddb8-4ca9-4498-906e-81b654135662"}, { Terminator.IdentifierTerminator, "764301e0-1e52-4941-80fc-f768fabb848a"}, { new NonTerminator(NonTerminatorType.ProcedureCall), "d19e5b10-7ef3-4981-8c0b-deca4bfff866"}, { new NonTerminator(NonTerminatorType.CompoundStatement), "67b6f07f-8024-4a8f-8f06-3e29ee8141d8"}, { new Terminator(KeywordType.If), "1139d083-a185-4a24-a86e-2dfa3d0514e2"}, { new Terminator(KeywordType.For), "8337e624-bbce-4b9a-8231-b6b264eefd7f"}, { new Terminator(KeywordType.Begin), "5c23381d-aee0-4773-ab73-20aa2a344931"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, }, "5c23381d-aee0-4773-ab73-20aa2a344931") },
		{ "957827df-f48f-442b-bec7-ab2080e32435", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Begin), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SubprogramDeclarations))}, { new Terminator(KeywordType.Procedure), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SubprogramDeclarations))}, { new Terminator(KeywordType.Function), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SubprogramDeclarations))}, }, "957827df-f48f-442b-bec7-ab2080e32435") },
		{ "57f035be-fdbb-44b5-9108-07251e89d821", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.SubprogramBody), "8fbeee48-0771-43a8-8814-000495800718"}, { new NonTerminator(NonTerminatorType.ConstDeclarations), "338d85a3-9e41-41a3-b097-d5ea8e3b9cd0"}, { new Terminator(KeywordType.Const), "33c4b030-faaf-4171-8f65-a12f1985bc86"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Begin), new ReduceInformation(0, new NonTerminator(NonTerminatorType.ConstDeclarations))}, { new Terminator(KeywordType.Var), new ReduceInformation(0, new NonTerminator(NonTerminatorType.ConstDeclarations))}, }, "57f035be-fdbb-44b5-9108-07251e89d821") },
		{ "ea565722-ec3d-4b33-ac27-ff9d72acfddc", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.FormalParameter), "eb54ac6f-8637-4218-8863-2d4022241177"}, { new Terminator(DelimiterType.LeftParenthesis), "cd8dafe8-35e0-4b3d-a6e0-af382fe9aa6a"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(0, new NonTerminator(NonTerminatorType.FormalParameter))}, }, "ea565722-ec3d-4b33-ac27-ff9d72acfddc") },
		{ "2d90f8ce-a1fd-425e-95eb-21775de2734e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.FormalParameter), "1253842a-e217-450f-8418-1c91176b8005"}, { new Terminator(DelimiterType.LeftParenthesis), "bc3d5dcb-a209-4fe0-ad56-92e056a92e4a"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Colon), new ReduceInformation(0, new NonTerminator(NonTerminatorType.FormalParameter))}, }, "2d90f8ce-a1fd-425e-95eb-21775de2734e") },
		{ "7b79d969-3543-466c-8c4f-2403ae51406c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.Colon), "28be05cb-9b3f-4084-8071-4a5f478ba86f"}, { new Terminator(DelimiterType.Comma), "19b90e6e-1e3b-4063-b92b-d3f3ad667ca2"},}, new Dictionary<Terminator, ReduceInformation>{ }, "7b79d969-3543-466c-8c4f-2403ae51406c") },
		{ "76ce47d7-0021-46d0-901b-57e7acf77051", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.VarDeclaration))}, }, "76ce47d7-0021-46d0-901b-57e7acf77051") },
		{ "03fa10c0-f221-4476-8ac3-a84cb91df27a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Type))}, }, "03fa10c0-f221-4476-8ac3-a84cb91df27a") },
		{ "6208cee5-0a9f-4d9e-9149-5efa773fbcf9", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.LeftSquareBracket), "00749ffb-5285-4597-aaa9-01e9ebe5ff52"},}, new Dictionary<Terminator, ReduceInformation>{ }, "6208cee5-0a9f-4d9e-9149-5efa773fbcf9") },
		{ "e3c9fab2-9cd8-44ad-a33c-54482b5323d5", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.BasicType))}, }, "e3c9fab2-9cd8-44ad-a33c-54482b5323d5") },
		{ "3bb38fcd-bf5c-401a-a2e4-f6f4d91bc650", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.BasicType))}, }, "3bb38fcd-bf5c-401a-a2e4-f6f4d91bc650") },
		{ "01a76185-0283-464f-b7d6-9f5bb8517edd", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.BasicType))}, }, "01a76185-0283-464f-b7d6-9f5bb8517edd") },
		{ "ee488773-f67c-418c-a1a5-ddd934d4253b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.BasicType))}, }, "ee488773-f67c-418c-a1a5-ddd934d4253b") },
		{ "d522700d-0e6e-4a72-ad24-e765c60c6311", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Colon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdentifierList))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdentifierList))}, }, "d522700d-0e6e-4a72-ad24-e765c60c6311") },
		{ "e72d1c7d-c459-4b11-bc70-edd87cb2e149", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ConstValue), "606154c3-6112-4b9f-a20b-297158b92fe9"}, { new Terminator(OperatorType.Plus), "6fdf7012-c004-46a8-b259-c2939925031c"}, { new Terminator(OperatorType.Minus), "c654570d-4062-4758-8e9c-14fe4cc2a227"}, { Terminator.NumberTerminator, "2be558a7-1c18-499f-8ec3-9cf689fe0193"}, { Terminator.CharacterTerminator, "085e4679-3deb-40b2-a35e-81d6eac5e1ae"},}, new Dictionary<Terminator, ReduceInformation>{ }, "e72d1c7d-c459-4b11-bc70-edd87cb2e149") },
		{ "bb2dba84-2cec-4600-a009-6cd6f2e762c4", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.ConstValue))}, }, "bb2dba84-2cec-4600-a009-6cd6f2e762c4") },
		{ "71ebc44a-d6ca-42cb-a8ae-2efee62c7f0a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.ConstValue))}, }, "71ebc44a-d6ca-42cb-a8ae-2efee62c7f0a") },
		{ "4758be8f-9722-47a6-9e0c-369b492f3f81", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Period), new ReduceInformation(3, new NonTerminator(NonTerminatorType.CompoundStatement))}, }, "4758be8f-9722-47a6-9e0c-369b492f3f81") },
		{ "f83f97d0-e0c9-462f-b6fa-0bc1226d63fe", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Statement), "f5884ee6-969a-4b78-9680-485689fa42e3"}, { new NonTerminator(NonTerminatorType.Variable), "ccb9ddb8-4ca9-4498-906e-81b654135662"}, { Terminator.IdentifierTerminator, "764301e0-1e52-4941-80fc-f768fabb848a"}, { new NonTerminator(NonTerminatorType.ProcedureCall), "d19e5b10-7ef3-4981-8c0b-deca4bfff866"}, { new NonTerminator(NonTerminatorType.CompoundStatement), "67b6f07f-8024-4a8f-8f06-3e29ee8141d8"}, { new Terminator(KeywordType.If), "1139d083-a185-4a24-a86e-2dfa3d0514e2"}, { new Terminator(KeywordType.For), "8337e624-bbce-4b9a-8231-b6b264eefd7f"}, { new Terminator(KeywordType.Begin), "5c23381d-aee0-4773-ab73-20aa2a344931"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, }, "f83f97d0-e0c9-462f-b6fa-0bc1226d63fe") },
		{ "289b2a05-55d5-4771-a2ce-3999e7cfead1", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "3880860f-586b-4713-b98c-2f753b13f897"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "bd8e2acc-96f6-473c-8fd4-7f806bc7970a"}, { new NonTerminator(NonTerminatorType.Term), "597008e4-1243-4ae2-ba90-329e4dc5ebd9"}, { new NonTerminator(NonTerminatorType.Factor), "ed166eeb-11fc-4025-a5ff-df1fb55129bf"}, { Terminator.NumberTerminator, "da7ed270-1de2-409d-8992-b6e78325e8ec"}, { new NonTerminator(NonTerminatorType.Variable), "16141db6-7be2-4636-b42e-2eb50645928b"}, { new Terminator(DelimiterType.LeftParenthesis), "39c7ef5b-d272-43f7-83ac-975e6142661c"}, { Terminator.IdentifierTerminator, "97c62080-8f8c-4d05-90bb-f8e531abdc6e"}, { new Terminator(KeywordType.Not), "753e7f13-2ad6-442a-b578-5d4205731d5d"}, { new Terminator(OperatorType.Minus), "5771e46b-f864-4859-a24b-ed54f3875040"},}, new Dictionary<Terminator, ReduceInformation>{ }, "289b2a05-55d5-4771-a2ce-3999e7cfead1") },
		{ "f3630dbe-c2d7-43ee-a30b-055d5d71520d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "f2b7134e-98b0-4231-8f9d-7d7f0ab65912"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "bd8e2acc-96f6-473c-8fd4-7f806bc7970a"}, { new NonTerminator(NonTerminatorType.Term), "597008e4-1243-4ae2-ba90-329e4dc5ebd9"}, { new NonTerminator(NonTerminatorType.Factor), "ed166eeb-11fc-4025-a5ff-df1fb55129bf"}, { Terminator.NumberTerminator, "da7ed270-1de2-409d-8992-b6e78325e8ec"}, { new NonTerminator(NonTerminatorType.Variable), "16141db6-7be2-4636-b42e-2eb50645928b"}, { new Terminator(DelimiterType.LeftParenthesis), "39c7ef5b-d272-43f7-83ac-975e6142661c"}, { Terminator.IdentifierTerminator, "97c62080-8f8c-4d05-90bb-f8e531abdc6e"}, { new Terminator(KeywordType.Not), "753e7f13-2ad6-442a-b578-5d4205731d5d"}, { new Terminator(OperatorType.Minus), "5771e46b-f864-4859-a24b-ed54f3875040"},}, new Dictionary<Terminator, ReduceInformation>{ }, "f3630dbe-c2d7-43ee-a30b-055d5d71520d") },
		{ "48402e50-8efc-4ee1-b0ac-b318b198cdc6", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(OperatorType.Assign), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, }, "48402e50-8efc-4ee1-b0ac-b318b198cdc6") },
		{ "ad1a1d2d-24d0-4c6e-8654-b1025dbada2f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "47f09497-f8cd-42be-8eac-22f162dfc5b4"}, { new NonTerminator(NonTerminatorType.Expression), "10204ac0-c523-4b6c-890d-d508a8e83f48"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "72d6e812-1720-408a-ac1c-5ac20fa267d1"}, { new NonTerminator(NonTerminatorType.Term), "0b3b44d2-8458-48d1-b7c4-2329d479b56c"}, { new NonTerminator(NonTerminatorType.Factor), "2f595e0e-c29e-494d-a5bb-a570db3254fd"}, { Terminator.NumberTerminator, "c86f637c-4e17-4d3d-bfd7-f1fc5abdf7f4"}, { new NonTerminator(NonTerminatorType.Variable), "fa555a17-89d0-4cf7-989a-0b0775ade6d4"}, { new Terminator(DelimiterType.LeftParenthesis), "8fab798a-03a8-45b7-8380-7b7eaee651fa"}, { Terminator.IdentifierTerminator, "513b9c84-fc68-4334-976d-7547b1ec521f"}, { new Terminator(KeywordType.Not), "928a980c-85a8-4cf5-a5d1-117516baef4e"}, { new Terminator(OperatorType.Minus), "e2c085de-e99b-4f3a-b869-c3ca5f5dcf75"},}, new Dictionary<Terminator, ReduceInformation>{ }, "ad1a1d2d-24d0-4c6e-8654-b1025dbada2f") },
		{ "abf8b682-4ac1-45bc-9b28-eb89f4ae0216", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "30c7c97e-db74-4795-8faa-8d8ac2ff7627"}, { new NonTerminator(NonTerminatorType.Expression), "a582ab7f-9aa1-4130-9df3-2ba9dd0fe6f3"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "c3098ddd-d7d7-4d8b-b9ab-1ebedac5467f"}, { new NonTerminator(NonTerminatorType.Term), "c83de3f5-4130-4b2e-8715-e72b2e35275e"}, { new NonTerminator(NonTerminatorType.Factor), "9c184da0-a0b4-4ae3-9b1e-93b0fe09fbf1"}, { Terminator.NumberTerminator, "eaf4d875-7318-4f6e-b25b-829d30a888b8"}, { new NonTerminator(NonTerminatorType.Variable), "eb827526-1aa2-47ab-8b01-b1b0792e4297"}, { new Terminator(DelimiterType.LeftParenthesis), "2894d0f9-3faf-4be5-b2b6-c499611fc58d"}, { Terminator.IdentifierTerminator, "8efce527-7ba0-435f-a58f-b048a7a0e505"}, { new Terminator(KeywordType.Not), "dd22d779-6929-400f-832a-6d4e7eda69e6"}, { new Terminator(OperatorType.Minus), "28fb603c-e1e9-4bfa-ab9b-f8887e1996c0"},}, new Dictionary<Terminator, ReduceInformation>{ }, "abf8b682-4ac1-45bc-9b28-eb89f4ae0216") },
		{ "8d60fecf-8eba-4d5c-adcd-5b92e16c3bc4", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(KeywordType.Then), "15d68093-a965-4e25-928e-73f8efefffb8"},}, new Dictionary<Terminator, ReduceInformation>{ }, "8d60fecf-8eba-4d5c-adcd-5b92e16c3bc4") },
		{ "9372190c-03b2-4dd8-b6d0-ed9a16c071cc", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.RelationOperator), "e110f401-b0ba-4e86-aede-81893526197f"}, { new Terminator(OperatorType.Equal), "6112dd54-ce34-4dd2-be4c-b06dd725bbd8"}, { new Terminator(OperatorType.NotEqual), "1338afe1-da9e-4380-8a28-7f2d82ef2d68"}, { new Terminator(OperatorType.Less), "fc9b446b-c4b5-4720-bc46-49344508202f"}, { new Terminator(OperatorType.LessEqual), "39ac36f9-efe7-4a1b-ba37-a9729fbf341a"}, { new Terminator(OperatorType.Greater), "fc85ac1b-5a0f-405f-a8f0-a1e20d44351d"}, { new Terminator(OperatorType.GreaterEqual), "f0598e66-3db3-44bd-8b9d-cd6c038df056"}, { new NonTerminator(NonTerminatorType.AddOperator), "a380c80b-dee6-45ff-a9a1-8c1f5e13207a"}, { new Terminator(OperatorType.Plus), "493d6d76-ed65-4308-aee5-65c5fe502546"}, { new Terminator(OperatorType.Minus), "c41b579b-0a72-411a-b9ed-f3ca30f5a615"}, { new Terminator(KeywordType.Or), "22878828-eed7-470f-a07f-595bb2eb7e2a"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Expression))}, }, "9372190c-03b2-4dd8-b6d0-ed9a16c071cc") },
		{ "4101b78d-0b68-4d0a-819e-ec5221d8ad24", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "abc7c9b1-5843-493b-a7fc-ebb63e19a9b5"}, { new Terminator(OperatorType.Multiply), "fd20e81d-52b7-4920-adab-e2a1e49a6b6a"}, { new Terminator(OperatorType.Divide), "ef7280a1-85cf-4590-ab87-68bdd867d9e5"}, { new Terminator(KeywordType.Divide), "9192ebb2-aa8a-414d-ac9e-7206883933e6"}, { new Terminator(KeywordType.Mod), "ae32e061-59db-49fd-addd-2d5c3e9014a3"}, { new Terminator(KeywordType.And), "5c4b371b-c686-4545-b641-4c196ca6b25f"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "4101b78d-0b68-4d0a-819e-ec5221d8ad24") },
		{ "87546012-c006-4323-84d2-09d75a8399c5", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, }, "87546012-c006-4323-84d2-09d75a8399c5") },
		{ "fa97211a-80b7-4975-9eee-548312d9b847", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "fa97211a-80b7-4975-9eee-548312d9b847") },
		{ "18fb7818-280d-424d-acd7-c30c11393bbf", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "18fb7818-280d-424d-acd7-c30c11393bbf") },
		{ "b9e87be5-24b1-455e-8377-274fab33e19a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "9342a1fb-ddc8-436f-b10d-c0b3b0a3d6e9"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "38eec331-d4ae-4b06-a659-98475fdb5aa0"}, { new NonTerminator(NonTerminatorType.Term), "5a5a9a53-cfe3-4e81-8fcf-2e6d11b16b0e"}, { new NonTerminator(NonTerminatorType.Factor), "175e095a-0963-472f-b84a-ece518bd2cf2"}, { Terminator.NumberTerminator, "3861f05c-791c-4f64-b291-d985540c7651"}, { new NonTerminator(NonTerminatorType.Variable), "c693eb95-cdcd-4c7d-8717-3ead45ef8bfc"}, { new Terminator(DelimiterType.LeftParenthesis), "efc9be5f-b54d-4b96-8aec-f3540a407dc8"}, { Terminator.IdentifierTerminator, "a8ce470f-1537-428f-88c2-9270bfed81b6"}, { new Terminator(KeywordType.Not), "addb6420-8367-48a9-9f28-327cf2a88028"}, { new Terminator(OperatorType.Minus), "75101aaf-bd06-4db5-809c-9e8b4533dc5d"},}, new Dictionary<Terminator, ReduceInformation>{ }, "b9e87be5-24b1-455e-8377-274fab33e19a") },
		{ "2568b015-15cf-42e5-bacf-64d5d56c3b0e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.LeftParenthesis), "b71eba49-3129-4617-b054-7280e83921ec"}, { new NonTerminator(NonTerminatorType.IdVarPart), "fe05a345-7dd1-4126-9002-64a5775d163f"}, { new Terminator(DelimiterType.LeftSquareBracket), "91b63ce6-a1e3-4083-9c56-27d8bd63c511"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Equal), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Less), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Greater), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "2568b015-15cf-42e5-bacf-64d5d56c3b0e") },
		{ "bb6682cd-c5f2-49f5-a560-001ce41b69a5", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "cbf67630-94d3-48ab-b3c4-95ec5ba223e1"}, { Terminator.NumberTerminator, "fa97211a-80b7-4975-9eee-548312d9b847"}, { new NonTerminator(NonTerminatorType.Variable), "18fb7818-280d-424d-acd7-c30c11393bbf"}, { new Terminator(DelimiterType.LeftParenthesis), "b9e87be5-24b1-455e-8377-274fab33e19a"}, { Terminator.IdentifierTerminator, "2568b015-15cf-42e5-bacf-64d5d56c3b0e"}, { new Terminator(KeywordType.Not), "bb6682cd-c5f2-49f5-a560-001ce41b69a5"}, { new Terminator(OperatorType.Minus), "0ebd6126-1360-4d5b-ba9c-0b49e8d814fa"},}, new Dictionary<Terminator, ReduceInformation>{ }, "bb6682cd-c5f2-49f5-a560-001ce41b69a5") },
		{ "0ebd6126-1360-4d5b-ba9c-0b49e8d814fa", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "0ba032d7-8bf5-4092-befb-8119f507e038"}, { Terminator.NumberTerminator, "fa97211a-80b7-4975-9eee-548312d9b847"}, { new NonTerminator(NonTerminatorType.Variable), "18fb7818-280d-424d-acd7-c30c11393bbf"}, { new Terminator(DelimiterType.LeftParenthesis), "b9e87be5-24b1-455e-8377-274fab33e19a"}, { Terminator.IdentifierTerminator, "2568b015-15cf-42e5-bacf-64d5d56c3b0e"}, { new Terminator(KeywordType.Not), "bb6682cd-c5f2-49f5-a560-001ce41b69a5"}, { new Terminator(OperatorType.Minus), "0ebd6126-1360-4d5b-ba9c-0b49e8d814fa"},}, new Dictionary<Terminator, ReduceInformation>{ }, "0ebd6126-1360-4d5b-ba9c-0b49e8d814fa") },
		{ "9b162179-6984-440a-a586-81893a049329", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(OperatorType.Assign), "9045ad7a-6b46-4794-b88f-377068357369"},}, new Dictionary<Terminator, ReduceInformation>{ }, "9b162179-6984-440a-a586-81893a049329") },
		{ "38ae7a30-a988-4208-b44d-5ed83fb9e099", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(KeywordType.End), "082995dc-2c94-48ec-8c49-c5524d1c27c0"}, { new Terminator(DelimiterType.Semicolon), "f83f97d0-e0c9-462f-b6fa-0bc1226d63fe"},}, new Dictionary<Terminator, ReduceInformation>{ }, "38ae7a30-a988-4208-b44d-5ed83fb9e099") },
		{ "8fbeee48-0771-43a8-8814-000495800718", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Subprogram))}, }, "8fbeee48-0771-43a8-8814-000495800718") },
		{ "338d85a3-9e41-41a3-b097-d5ea8e3b9cd0", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.VarDeclarations), "a56f5613-4494-4f75-8cf7-2ada0a4b24c7"}, { new Terminator(KeywordType.Var), "07b6153d-bdb1-4356-8c72-ff4c3614d9b8"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Begin), new ReduceInformation(0, new NonTerminator(NonTerminatorType.VarDeclarations))}, }, "338d85a3-9e41-41a3-b097-d5ea8e3b9cd0") },
		{ "33c4b030-faaf-4171-8f65-a12f1985bc86", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ConstDeclaration), "1477df97-a3c9-4545-bc37-6e43589e9142"}, { Terminator.IdentifierTerminator, "8ed7d885-b402-4805-b7c7-a25113fab7a4"},}, new Dictionary<Terminator, ReduceInformation>{ }, "33c4b030-faaf-4171-8f65-a12f1985bc86") },
		{ "eb54ac6f-8637-4218-8863-2d4022241177", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SubprogramHead))}, }, "eb54ac6f-8637-4218-8863-2d4022241177") },
		{ "cd8dafe8-35e0-4b3d-a6e0-af382fe9aa6a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ParameterList), "0138ca0c-7f0d-4cab-8ba5-0c896f14af03"}, { new NonTerminator(NonTerminatorType.Parameter), "57f152c5-b52e-43da-938f-dcd99f48515d"}, { new NonTerminator(NonTerminatorType.VarParameter), "1c54abd3-ad5a-460b-b84c-508c30c2925b"}, { new NonTerminator(NonTerminatorType.ValueParameter), "7d1b8c5c-7295-4e38-9399-c7e99513b3af"}, { new Terminator(KeywordType.Var), "eb3fbdf4-847d-4499-9851-d06d9fc995b9"}, { new NonTerminator(NonTerminatorType.IdentifierList), "8129be83-68fb-4cb1-b760-6daf04d786a9"}, { Terminator.IdentifierTerminator, "3ed14f62-8e9f-4c1a-8e1e-5d522192093b"},}, new Dictionary<Terminator, ReduceInformation>{ }, "cd8dafe8-35e0-4b3d-a6e0-af382fe9aa6a") },
		{ "1253842a-e217-450f-8418-1c91176b8005", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.Colon), "14ab58de-ab30-47ea-9429-f61a28fbbf18"},}, new Dictionary<Terminator, ReduceInformation>{ }, "1253842a-e217-450f-8418-1c91176b8005") },
		{ "bc3d5dcb-a209-4fe0-ad56-92e056a92e4a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ParameterList), "f2561476-ccd0-4b72-a032-d67ab6815303"}, { new NonTerminator(NonTerminatorType.Parameter), "57f152c5-b52e-43da-938f-dcd99f48515d"}, { new NonTerminator(NonTerminatorType.VarParameter), "1c54abd3-ad5a-460b-b84c-508c30c2925b"}, { new NonTerminator(NonTerminatorType.ValueParameter), "7d1b8c5c-7295-4e38-9399-c7e99513b3af"}, { new Terminator(KeywordType.Var), "eb3fbdf4-847d-4499-9851-d06d9fc995b9"}, { new NonTerminator(NonTerminatorType.IdentifierList), "8129be83-68fb-4cb1-b760-6daf04d786a9"}, { Terminator.IdentifierTerminator, "3ed14f62-8e9f-4c1a-8e1e-5d522192093b"},}, new Dictionary<Terminator, ReduceInformation>{ }, "bc3d5dcb-a209-4fe0-ad56-92e056a92e4a") },
		{ "28be05cb-9b3f-4084-8071-4a5f478ba86f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Type), "32848038-9d3c-4589-b6ef-001ab9c0d5d7"}, { new NonTerminator(NonTerminatorType.BasicType), "03fa10c0-f221-4476-8ac3-a84cb91df27a"}, { new Terminator(KeywordType.Array), "6208cee5-0a9f-4d9e-9149-5efa773fbcf9"}, { new Terminator(KeywordType.Integer), "e3c9fab2-9cd8-44ad-a33c-54482b5323d5"}, { new Terminator(KeywordType.Real), "3bb38fcd-bf5c-401a-a2e4-f6f4d91bc650"}, { new Terminator(KeywordType.Boolean), "01a76185-0283-464f-b7d6-9f5bb8517edd"}, { new Terminator(KeywordType.Character), "ee488773-f67c-418c-a1a5-ddd934d4253b"},}, new Dictionary<Terminator, ReduceInformation>{ }, "28be05cb-9b3f-4084-8071-4a5f478ba86f") },
		{ "00749ffb-5285-4597-aaa9-01e9ebe5ff52", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Period), "9e8bb750-816a-4fef-a639-edc142eeec24"}, { Terminator.NumberTerminator, "f0f673dc-bbe7-4f92-ae61-9169b47aa68f"},}, new Dictionary<Terminator, ReduceInformation>{ }, "00749ffb-5285-4597-aaa9-01e9ebe5ff52") },
		{ "606154c3-6112-4b9f-a20b-297158b92fe9", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(5, new NonTerminator(NonTerminatorType.ConstDeclaration))}, }, "606154c3-6112-4b9f-a20b-297158b92fe9") },
		{ "f5884ee6-969a-4b78-9680-485689fa42e3", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.StatementList))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.StatementList))}, }, "f5884ee6-969a-4b78-9680-485689fa42e3") },
		{ "3880860f-586b-4713-b98c-2f753b13f897", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Statement))}, }, "3880860f-586b-4713-b98c-2f753b13f897") },
		{ "bd8e2acc-96f6-473c-8fd4-7f806bc7970a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.RelationOperator), "80000b15-c90f-413e-bf26-7daf48e05eac"}, { new Terminator(OperatorType.Equal), "6112dd54-ce34-4dd2-be4c-b06dd725bbd8"}, { new Terminator(OperatorType.NotEqual), "1338afe1-da9e-4380-8a28-7f2d82ef2d68"}, { new Terminator(OperatorType.Less), "fc9b446b-c4b5-4720-bc46-49344508202f"}, { new Terminator(OperatorType.LessEqual), "39ac36f9-efe7-4a1b-ba37-a9729fbf341a"}, { new Terminator(OperatorType.Greater), "fc85ac1b-5a0f-405f-a8f0-a1e20d44351d"}, { new Terminator(OperatorType.GreaterEqual), "f0598e66-3db3-44bd-8b9d-cd6c038df056"}, { new NonTerminator(NonTerminatorType.AddOperator), "e31b812e-51e9-4a26-8a57-69e1398856a1"}, { new Terminator(OperatorType.Plus), "493d6d76-ed65-4308-aee5-65c5fe502546"}, { new Terminator(OperatorType.Minus), "c41b579b-0a72-411a-b9ed-f3ca30f5a615"}, { new Terminator(KeywordType.Or), "22878828-eed7-470f-a07f-595bb2eb7e2a"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Expression))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Expression))}, }, "bd8e2acc-96f6-473c-8fd4-7f806bc7970a") },
		{ "597008e4-1243-4ae2-ba90-329e4dc5ebd9", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "7d5842f8-4715-4558-b43b-bdcfd821ece2"}, { new Terminator(OperatorType.Multiply), "fd20e81d-52b7-4920-adab-e2a1e49a6b6a"}, { new Terminator(OperatorType.Divide), "ef7280a1-85cf-4590-ab87-68bdd867d9e5"}, { new Terminator(KeywordType.Divide), "9192ebb2-aa8a-414d-ac9e-7206883933e6"}, { new Terminator(KeywordType.Mod), "ae32e061-59db-49fd-addd-2d5c3e9014a3"}, { new Terminator(KeywordType.And), "5c4b371b-c686-4545-b641-4c196ca6b25f"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "597008e4-1243-4ae2-ba90-329e4dc5ebd9") },
		{ "ed166eeb-11fc-4025-a5ff-df1fb55129bf", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, }, "ed166eeb-11fc-4025-a5ff-df1fb55129bf") },
		{ "da7ed270-1de2-409d-8992-b6e78325e8ec", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "da7ed270-1de2-409d-8992-b6e78325e8ec") },
		{ "16141db6-7be2-4636-b42e-2eb50645928b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "16141db6-7be2-4636-b42e-2eb50645928b") },
		{ "39c7ef5b-d272-43f7-83ac-975e6142661c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "692d8ac3-7f58-4b33-9866-edc32b01bd8f"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "38eec331-d4ae-4b06-a659-98475fdb5aa0"}, { new NonTerminator(NonTerminatorType.Term), "5a5a9a53-cfe3-4e81-8fcf-2e6d11b16b0e"}, { new NonTerminator(NonTerminatorType.Factor), "175e095a-0963-472f-b84a-ece518bd2cf2"}, { Terminator.NumberTerminator, "3861f05c-791c-4f64-b291-d985540c7651"}, { new NonTerminator(NonTerminatorType.Variable), "c693eb95-cdcd-4c7d-8717-3ead45ef8bfc"}, { new Terminator(DelimiterType.LeftParenthesis), "efc9be5f-b54d-4b96-8aec-f3540a407dc8"}, { Terminator.IdentifierTerminator, "a8ce470f-1537-428f-88c2-9270bfed81b6"}, { new Terminator(KeywordType.Not), "addb6420-8367-48a9-9f28-327cf2a88028"}, { new Terminator(OperatorType.Minus), "75101aaf-bd06-4db5-809c-9e8b4533dc5d"},}, new Dictionary<Terminator, ReduceInformation>{ }, "39c7ef5b-d272-43f7-83ac-975e6142661c") },
		{ "97c62080-8f8c-4d05-90bb-f8e531abdc6e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.LeftParenthesis), "22ab57e0-2d9d-4941-80a4-aba37acec74d"}, { new NonTerminator(NonTerminatorType.IdVarPart), "6d7b6d7d-3a6f-4dad-b34f-e3f93924f032"}, { new Terminator(DelimiterType.LeftSquareBracket), "fd1d9b8e-35a3-47cc-8259-87fcda434d9d"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Equal), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Less), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Greater), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "97c62080-8f8c-4d05-90bb-f8e531abdc6e") },
		{ "753e7f13-2ad6-442a-b578-5d4205731d5d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "d2312a91-ced3-4372-8013-67cd3a25b751"}, { Terminator.NumberTerminator, "da7ed270-1de2-409d-8992-b6e78325e8ec"}, { new NonTerminator(NonTerminatorType.Variable), "16141db6-7be2-4636-b42e-2eb50645928b"}, { new Terminator(DelimiterType.LeftParenthesis), "39c7ef5b-d272-43f7-83ac-975e6142661c"}, { Terminator.IdentifierTerminator, "97c62080-8f8c-4d05-90bb-f8e531abdc6e"}, { new Terminator(KeywordType.Not), "753e7f13-2ad6-442a-b578-5d4205731d5d"}, { new Terminator(OperatorType.Minus), "5771e46b-f864-4859-a24b-ed54f3875040"},}, new Dictionary<Terminator, ReduceInformation>{ }, "753e7f13-2ad6-442a-b578-5d4205731d5d") },
		{ "5771e46b-f864-4859-a24b-ed54f3875040", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "797625c3-fa5e-4a9c-8618-43cd9e880bbf"}, { Terminator.NumberTerminator, "da7ed270-1de2-409d-8992-b6e78325e8ec"}, { new NonTerminator(NonTerminatorType.Variable), "16141db6-7be2-4636-b42e-2eb50645928b"}, { new Terminator(DelimiterType.LeftParenthesis), "39c7ef5b-d272-43f7-83ac-975e6142661c"}, { Terminator.IdentifierTerminator, "97c62080-8f8c-4d05-90bb-f8e531abdc6e"}, { new Terminator(KeywordType.Not), "753e7f13-2ad6-442a-b578-5d4205731d5d"}, { new Terminator(OperatorType.Minus), "5771e46b-f864-4859-a24b-ed54f3875040"},}, new Dictionary<Terminator, ReduceInformation>{ }, "5771e46b-f864-4859-a24b-ed54f3875040") },
		{ "f2b7134e-98b0-4231-8f9d-7d7f0ab65912", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Statement))}, }, "f2b7134e-98b0-4231-8f9d-7d7f0ab65912") },
		{ "47f09497-f8cd-42be-8eac-22f162dfc5b4", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightSquareBracket), "2d4bfcda-0d59-48fb-b52c-95e9843a3cd3"}, { new Terminator(DelimiterType.Comma), "ecf84275-ae5c-4a59-96fb-d2ef13a595a8"},}, new Dictionary<Terminator, ReduceInformation>{ }, "47f09497-f8cd-42be-8eac-22f162dfc5b4") },
		{ "10204ac0-c523-4b6c-890d-d508a8e83f48", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(1, new NonTerminator(NonTerminatorType.ExpressionList))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.ExpressionList))}, }, "10204ac0-c523-4b6c-890d-d508a8e83f48") },
		{ "72d6e812-1720-408a-ac1c-5ac20fa267d1", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.RelationOperator), "2a4bc2d8-0766-4529-bb16-5702b468cbd3"}, { new Terminator(OperatorType.Equal), "6112dd54-ce34-4dd2-be4c-b06dd725bbd8"}, { new Terminator(OperatorType.NotEqual), "1338afe1-da9e-4380-8a28-7f2d82ef2d68"}, { new Terminator(OperatorType.Less), "fc9b446b-c4b5-4720-bc46-49344508202f"}, { new Terminator(OperatorType.LessEqual), "39ac36f9-efe7-4a1b-ba37-a9729fbf341a"}, { new Terminator(OperatorType.Greater), "fc85ac1b-5a0f-405f-a8f0-a1e20d44351d"}, { new Terminator(OperatorType.GreaterEqual), "f0598e66-3db3-44bd-8b9d-cd6c038df056"}, { new NonTerminator(NonTerminatorType.AddOperator), "1a33374a-916c-4e8c-9dab-a16669c0132b"}, { new Terminator(OperatorType.Plus), "493d6d76-ed65-4308-aee5-65c5fe502546"}, { new Terminator(OperatorType.Minus), "c41b579b-0a72-411a-b9ed-f3ca30f5a615"}, { new Terminator(KeywordType.Or), "22878828-eed7-470f-a07f-595bb2eb7e2a"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Expression))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Expression))}, }, "72d6e812-1720-408a-ac1c-5ac20fa267d1") },
		{ "0b3b44d2-8458-48d1-b7c4-2329d479b56c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "05f80b0f-765b-49de-8b00-897364786373"}, { new Terminator(OperatorType.Multiply), "fd20e81d-52b7-4920-adab-e2a1e49a6b6a"}, { new Terminator(OperatorType.Divide), "ef7280a1-85cf-4590-ab87-68bdd867d9e5"}, { new Terminator(KeywordType.Divide), "9192ebb2-aa8a-414d-ac9e-7206883933e6"}, { new Terminator(KeywordType.Mod), "ae32e061-59db-49fd-addd-2d5c3e9014a3"}, { new Terminator(KeywordType.And), "5c4b371b-c686-4545-b641-4c196ca6b25f"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "0b3b44d2-8458-48d1-b7c4-2329d479b56c") },
		{ "2f595e0e-c29e-494d-a5bb-a570db3254fd", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, }, "2f595e0e-c29e-494d-a5bb-a570db3254fd") },
		{ "c86f637c-4e17-4d3d-bfd7-f1fc5abdf7f4", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "c86f637c-4e17-4d3d-bfd7-f1fc5abdf7f4") },
		{ "fa555a17-89d0-4cf7-989a-0b0775ade6d4", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "fa555a17-89d0-4cf7-989a-0b0775ade6d4") },
		{ "8fab798a-03a8-45b7-8380-7b7eaee651fa", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "405c6e37-2298-45d6-a73a-164eafdabaa7"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "38eec331-d4ae-4b06-a659-98475fdb5aa0"}, { new NonTerminator(NonTerminatorType.Term), "5a5a9a53-cfe3-4e81-8fcf-2e6d11b16b0e"}, { new NonTerminator(NonTerminatorType.Factor), "175e095a-0963-472f-b84a-ece518bd2cf2"}, { Terminator.NumberTerminator, "3861f05c-791c-4f64-b291-d985540c7651"}, { new NonTerminator(NonTerminatorType.Variable), "c693eb95-cdcd-4c7d-8717-3ead45ef8bfc"}, { new Terminator(DelimiterType.LeftParenthesis), "efc9be5f-b54d-4b96-8aec-f3540a407dc8"}, { Terminator.IdentifierTerminator, "a8ce470f-1537-428f-88c2-9270bfed81b6"}, { new Terminator(KeywordType.Not), "addb6420-8367-48a9-9f28-327cf2a88028"}, { new Terminator(OperatorType.Minus), "75101aaf-bd06-4db5-809c-9e8b4533dc5d"},}, new Dictionary<Terminator, ReduceInformation>{ }, "8fab798a-03a8-45b7-8380-7b7eaee651fa") },
		{ "513b9c84-fc68-4334-976d-7547b1ec521f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.LeftParenthesis), "157e56f1-333f-452b-b1f3-259c853036e1"}, { new NonTerminator(NonTerminatorType.IdVarPart), "1c80bd03-b6b0-416f-82c0-97c5438ebd3f"}, { new Terminator(DelimiterType.LeftSquareBracket), "c9feaf42-307a-4617-ba76-11ba0b77026b"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Equal), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Less), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Greater), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "513b9c84-fc68-4334-976d-7547b1ec521f") },
		{ "928a980c-85a8-4cf5-a5d1-117516baef4e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "0c1bde85-24a9-4869-9280-92292c185727"}, { Terminator.NumberTerminator, "c86f637c-4e17-4d3d-bfd7-f1fc5abdf7f4"}, { new NonTerminator(NonTerminatorType.Variable), "fa555a17-89d0-4cf7-989a-0b0775ade6d4"}, { new Terminator(DelimiterType.LeftParenthesis), "8fab798a-03a8-45b7-8380-7b7eaee651fa"}, { Terminator.IdentifierTerminator, "513b9c84-fc68-4334-976d-7547b1ec521f"}, { new Terminator(KeywordType.Not), "928a980c-85a8-4cf5-a5d1-117516baef4e"}, { new Terminator(OperatorType.Minus), "e2c085de-e99b-4f3a-b869-c3ca5f5dcf75"},}, new Dictionary<Terminator, ReduceInformation>{ }, "928a980c-85a8-4cf5-a5d1-117516baef4e") },
		{ "e2c085de-e99b-4f3a-b869-c3ca5f5dcf75", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "1a7da938-83c9-4ef7-b1b7-4bee5c69c377"}, { Terminator.NumberTerminator, "c86f637c-4e17-4d3d-bfd7-f1fc5abdf7f4"}, { new NonTerminator(NonTerminatorType.Variable), "fa555a17-89d0-4cf7-989a-0b0775ade6d4"}, { new Terminator(DelimiterType.LeftParenthesis), "8fab798a-03a8-45b7-8380-7b7eaee651fa"}, { Terminator.IdentifierTerminator, "513b9c84-fc68-4334-976d-7547b1ec521f"}, { new Terminator(KeywordType.Not), "928a980c-85a8-4cf5-a5d1-117516baef4e"}, { new Terminator(OperatorType.Minus), "e2c085de-e99b-4f3a-b869-c3ca5f5dcf75"},}, new Dictionary<Terminator, ReduceInformation>{ }, "e2c085de-e99b-4f3a-b869-c3ca5f5dcf75") },
		{ "30c7c97e-db74-4795-8faa-8d8ac2ff7627", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "9bb1761c-62b0-40f5-a91e-591a8edf560c"}, { new Terminator(DelimiterType.Comma), "dd4993c2-1db9-4e5a-a3f2-c4660ec4e365"},}, new Dictionary<Terminator, ReduceInformation>{ }, "30c7c97e-db74-4795-8faa-8d8ac2ff7627") },
		{ "a582ab7f-9aa1-4130-9df3-2ba9dd0fe6f3", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.ExpressionList))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.ExpressionList))}, }, "a582ab7f-9aa1-4130-9df3-2ba9dd0fe6f3") },
		{ "c3098ddd-d7d7-4d8b-b9ab-1ebedac5467f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.RelationOperator), "a9416155-af8f-4c09-90a9-8ac8c574bf91"}, { new Terminator(OperatorType.Equal), "6112dd54-ce34-4dd2-be4c-b06dd725bbd8"}, { new Terminator(OperatorType.NotEqual), "1338afe1-da9e-4380-8a28-7f2d82ef2d68"}, { new Terminator(OperatorType.Less), "fc9b446b-c4b5-4720-bc46-49344508202f"}, { new Terminator(OperatorType.LessEqual), "39ac36f9-efe7-4a1b-ba37-a9729fbf341a"}, { new Terminator(OperatorType.Greater), "fc85ac1b-5a0f-405f-a8f0-a1e20d44351d"}, { new Terminator(OperatorType.GreaterEqual), "f0598e66-3db3-44bd-8b9d-cd6c038df056"}, { new NonTerminator(NonTerminatorType.AddOperator), "eff6eeb5-c251-4d32-89b4-2fd1a8cd2069"}, { new Terminator(OperatorType.Plus), "493d6d76-ed65-4308-aee5-65c5fe502546"}, { new Terminator(OperatorType.Minus), "c41b579b-0a72-411a-b9ed-f3ca30f5a615"}, { new Terminator(KeywordType.Or), "22878828-eed7-470f-a07f-595bb2eb7e2a"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Expression))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Expression))}, }, "c3098ddd-d7d7-4d8b-b9ab-1ebedac5467f") },
		{ "c83de3f5-4130-4b2e-8715-e72b2e35275e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "7964d212-7d90-4920-b7c1-abae26705284"}, { new Terminator(OperatorType.Multiply), "fd20e81d-52b7-4920-adab-e2a1e49a6b6a"}, { new Terminator(OperatorType.Divide), "ef7280a1-85cf-4590-ab87-68bdd867d9e5"}, { new Terminator(KeywordType.Divide), "9192ebb2-aa8a-414d-ac9e-7206883933e6"}, { new Terminator(KeywordType.Mod), "ae32e061-59db-49fd-addd-2d5c3e9014a3"}, { new Terminator(KeywordType.And), "5c4b371b-c686-4545-b641-4c196ca6b25f"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "c83de3f5-4130-4b2e-8715-e72b2e35275e") },
		{ "9c184da0-a0b4-4ae3-9b1e-93b0fe09fbf1", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, }, "9c184da0-a0b4-4ae3-9b1e-93b0fe09fbf1") },
		{ "eaf4d875-7318-4f6e-b25b-829d30a888b8", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "eaf4d875-7318-4f6e-b25b-829d30a888b8") },
		{ "eb827526-1aa2-47ab-8b01-b1b0792e4297", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "eb827526-1aa2-47ab-8b01-b1b0792e4297") },
		{ "2894d0f9-3faf-4be5-b2b6-c499611fc58d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "ca0075b4-b452-481e-a378-faec2699ad4c"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "38eec331-d4ae-4b06-a659-98475fdb5aa0"}, { new NonTerminator(NonTerminatorType.Term), "5a5a9a53-cfe3-4e81-8fcf-2e6d11b16b0e"}, { new NonTerminator(NonTerminatorType.Factor), "175e095a-0963-472f-b84a-ece518bd2cf2"}, { Terminator.NumberTerminator, "3861f05c-791c-4f64-b291-d985540c7651"}, { new NonTerminator(NonTerminatorType.Variable), "c693eb95-cdcd-4c7d-8717-3ead45ef8bfc"}, { new Terminator(DelimiterType.LeftParenthesis), "efc9be5f-b54d-4b96-8aec-f3540a407dc8"}, { Terminator.IdentifierTerminator, "a8ce470f-1537-428f-88c2-9270bfed81b6"}, { new Terminator(KeywordType.Not), "addb6420-8367-48a9-9f28-327cf2a88028"}, { new Terminator(OperatorType.Minus), "75101aaf-bd06-4db5-809c-9e8b4533dc5d"},}, new Dictionary<Terminator, ReduceInformation>{ }, "2894d0f9-3faf-4be5-b2b6-c499611fc58d") },
		{ "8efce527-7ba0-435f-a58f-b048a7a0e505", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.LeftParenthesis), "bcbae121-f455-43db-ad52-0d675a786634"}, { new NonTerminator(NonTerminatorType.IdVarPart), "07a78aea-a3f7-4f15-87c9-3c42b7914284"}, { new Terminator(DelimiterType.LeftSquareBracket), "af07aa24-5fd4-4118-acb1-c007eae6ed21"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Equal), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Less), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Greater), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "8efce527-7ba0-435f-a58f-b048a7a0e505") },
		{ "dd22d779-6929-400f-832a-6d4e7eda69e6", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "18804690-9067-487a-9ea1-75c6519285ba"}, { Terminator.NumberTerminator, "eaf4d875-7318-4f6e-b25b-829d30a888b8"}, { new NonTerminator(NonTerminatorType.Variable), "eb827526-1aa2-47ab-8b01-b1b0792e4297"}, { new Terminator(DelimiterType.LeftParenthesis), "2894d0f9-3faf-4be5-b2b6-c499611fc58d"}, { Terminator.IdentifierTerminator, "8efce527-7ba0-435f-a58f-b048a7a0e505"}, { new Terminator(KeywordType.Not), "dd22d779-6929-400f-832a-6d4e7eda69e6"}, { new Terminator(OperatorType.Minus), "28fb603c-e1e9-4bfa-ab9b-f8887e1996c0"},}, new Dictionary<Terminator, ReduceInformation>{ }, "dd22d779-6929-400f-832a-6d4e7eda69e6") },
		{ "28fb603c-e1e9-4bfa-ab9b-f8887e1996c0", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "708740cb-62db-4187-898f-e66eb17abe2c"}, { Terminator.NumberTerminator, "eaf4d875-7318-4f6e-b25b-829d30a888b8"}, { new NonTerminator(NonTerminatorType.Variable), "eb827526-1aa2-47ab-8b01-b1b0792e4297"}, { new Terminator(DelimiterType.LeftParenthesis), "2894d0f9-3faf-4be5-b2b6-c499611fc58d"}, { Terminator.IdentifierTerminator, "8efce527-7ba0-435f-a58f-b048a7a0e505"}, { new Terminator(KeywordType.Not), "dd22d779-6929-400f-832a-6d4e7eda69e6"}, { new Terminator(OperatorType.Minus), "28fb603c-e1e9-4bfa-ab9b-f8887e1996c0"},}, new Dictionary<Terminator, ReduceInformation>{ }, "28fb603c-e1e9-4bfa-ab9b-f8887e1996c0") },
		{ "15d68093-a965-4e25-928e-73f8efefffb8", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Statement), "6b1deb83-9d0c-429d-8a32-c51d9db7cd8d"}, { new NonTerminator(NonTerminatorType.Variable), "c5d258c5-490b-4c90-8b5b-8242a6dbfc6d"}, { Terminator.IdentifierTerminator, "9a2b8a88-a48e-46e2-af69-f3561ad39bb7"}, { new NonTerminator(NonTerminatorType.ProcedureCall), "e9f86400-0193-40f4-a076-09e7b9b028e9"}, { new NonTerminator(NonTerminatorType.CompoundStatement), "48fe99c9-db82-494e-b751-8bda645590d0"}, { new Terminator(KeywordType.If), "79db3815-bc73-4868-ad20-1fc22d7453b9"}, { new Terminator(KeywordType.For), "ec4898d0-5ba7-47f1-885e-07725f2e43dc"}, { new Terminator(KeywordType.Begin), "65fb1933-38c9-42db-9907-36fe3c05573a"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(KeywordType.Else), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, }, "15d68093-a965-4e25-928e-73f8efefffb8") },
		{ "e110f401-b0ba-4e86-aede-81893526197f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.SimpleExpression), "1bd301f5-2851-43b5-99aa-2b2e9c756746"}, { new NonTerminator(NonTerminatorType.Term), "dda70d13-8e10-4723-a967-73c2d4560abf"}, { new NonTerminator(NonTerminatorType.Factor), "44bb095c-dbed-40a4-a4e5-ca65d38ca2ac"}, { Terminator.NumberTerminator, "6abd39fc-1d52-41a2-ae4b-43a6b52f2537"}, { new NonTerminator(NonTerminatorType.Variable), "7b2c56ec-cb0f-41f6-a54b-d73c18298449"}, { new Terminator(DelimiterType.LeftParenthesis), "3e5d0461-383e-4afd-8bd2-4c3ac4afbf0b"}, { Terminator.IdentifierTerminator, "068b589e-cf17-40e4-baae-3f3b18ddb76e"}, { new Terminator(KeywordType.Not), "f4285e75-83ea-4c5d-8403-e01bf0d9bbfb"}, { new Terminator(OperatorType.Minus), "a29a7e8a-8e5a-458a-8b41-cf17752ff1b1"},}, new Dictionary<Terminator, ReduceInformation>{ }, "e110f401-b0ba-4e86-aede-81893526197f") },
		{ "6112dd54-ce34-4dd2-be4c-b06dd725bbd8", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { Terminator.NumberTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { Terminator.IdentifierTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(DelimiterType.LeftParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(KeywordType.Not), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, }, "6112dd54-ce34-4dd2-be4c-b06dd725bbd8") },
		{ "1338afe1-da9e-4380-8a28-7f2d82ef2d68", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { Terminator.NumberTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { Terminator.IdentifierTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(DelimiterType.LeftParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(KeywordType.Not), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, }, "1338afe1-da9e-4380-8a28-7f2d82ef2d68") },
		{ "fc9b446b-c4b5-4720-bc46-49344508202f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { Terminator.NumberTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { Terminator.IdentifierTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(DelimiterType.LeftParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(KeywordType.Not), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, }, "fc9b446b-c4b5-4720-bc46-49344508202f") },
		{ "39ac36f9-efe7-4a1b-ba37-a9729fbf341a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { Terminator.NumberTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { Terminator.IdentifierTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(DelimiterType.LeftParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(KeywordType.Not), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, }, "39ac36f9-efe7-4a1b-ba37-a9729fbf341a") },
		{ "fc85ac1b-5a0f-405f-a8f0-a1e20d44351d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { Terminator.NumberTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { Terminator.IdentifierTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(DelimiterType.LeftParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(KeywordType.Not), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, }, "fc85ac1b-5a0f-405f-a8f0-a1e20d44351d") },
		{ "f0598e66-3db3-44bd-8b9d-cd6c038df056", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { Terminator.NumberTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { Terminator.IdentifierTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(DelimiterType.LeftParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(KeywordType.Not), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, }, "f0598e66-3db3-44bd-8b9d-cd6c038df056") },
		{ "a380c80b-dee6-45ff-a9a1-8c1f5e13207a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Term), "2d37c287-549e-42c5-8326-211797dd6131"}, { new NonTerminator(NonTerminatorType.Factor), "87546012-c006-4323-84d2-09d75a8399c5"}, { Terminator.NumberTerminator, "fa97211a-80b7-4975-9eee-548312d9b847"}, { new NonTerminator(NonTerminatorType.Variable), "18fb7818-280d-424d-acd7-c30c11393bbf"}, { new Terminator(DelimiterType.LeftParenthesis), "b9e87be5-24b1-455e-8377-274fab33e19a"}, { Terminator.IdentifierTerminator, "2568b015-15cf-42e5-bacf-64d5d56c3b0e"}, { new Terminator(KeywordType.Not), "bb6682cd-c5f2-49f5-a560-001ce41b69a5"}, { new Terminator(OperatorType.Minus), "0ebd6126-1360-4d5b-ba9c-0b49e8d814fa"},}, new Dictionary<Terminator, ReduceInformation>{ }, "a380c80b-dee6-45ff-a9a1-8c1f5e13207a") },
		{ "493d6d76-ed65-4308-aee5-65c5fe502546", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { Terminator.NumberTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.AddOperator))}, { Terminator.IdentifierTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.AddOperator))}, { new Terminator(DelimiterType.LeftParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.AddOperator))}, { new Terminator(KeywordType.Not), new ReduceInformation(1, new NonTerminator(NonTerminatorType.AddOperator))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.AddOperator))}, }, "493d6d76-ed65-4308-aee5-65c5fe502546") },
		{ "c41b579b-0a72-411a-b9ed-f3ca30f5a615", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { Terminator.NumberTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.AddOperator))}, { Terminator.IdentifierTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.AddOperator))}, { new Terminator(DelimiterType.LeftParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.AddOperator))}, { new Terminator(KeywordType.Not), new ReduceInformation(1, new NonTerminator(NonTerminatorType.AddOperator))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.AddOperator))}, }, "c41b579b-0a72-411a-b9ed-f3ca30f5a615") },
		{ "22878828-eed7-470f-a07f-595bb2eb7e2a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { Terminator.NumberTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.AddOperator))}, { Terminator.IdentifierTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.AddOperator))}, { new Terminator(DelimiterType.LeftParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.AddOperator))}, { new Terminator(KeywordType.Not), new ReduceInformation(1, new NonTerminator(NonTerminatorType.AddOperator))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.AddOperator))}, }, "22878828-eed7-470f-a07f-595bb2eb7e2a") },
		{ "abc7c9b1-5843-493b-a7fc-ebb63e19a9b5", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "bca1aaa9-0dda-477e-a6b4-f2e36b5a2d3f"}, { Terminator.NumberTerminator, "fa97211a-80b7-4975-9eee-548312d9b847"}, { new NonTerminator(NonTerminatorType.Variable), "18fb7818-280d-424d-acd7-c30c11393bbf"}, { new Terminator(DelimiterType.LeftParenthesis), "b9e87be5-24b1-455e-8377-274fab33e19a"}, { Terminator.IdentifierTerminator, "2568b015-15cf-42e5-bacf-64d5d56c3b0e"}, { new Terminator(KeywordType.Not), "bb6682cd-c5f2-49f5-a560-001ce41b69a5"}, { new Terminator(OperatorType.Minus), "0ebd6126-1360-4d5b-ba9c-0b49e8d814fa"},}, new Dictionary<Terminator, ReduceInformation>{ }, "abc7c9b1-5843-493b-a7fc-ebb63e19a9b5") },
		{ "fd20e81d-52b7-4920-adab-e2a1e49a6b6a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { Terminator.NumberTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { Terminator.IdentifierTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { new Terminator(DelimiterType.LeftParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { new Terminator(KeywordType.Not), new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, }, "fd20e81d-52b7-4920-adab-e2a1e49a6b6a") },
		{ "ef7280a1-85cf-4590-ab87-68bdd867d9e5", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { Terminator.NumberTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { Terminator.IdentifierTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { new Terminator(DelimiterType.LeftParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { new Terminator(KeywordType.Not), new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, }, "ef7280a1-85cf-4590-ab87-68bdd867d9e5") },
		{ "9192ebb2-aa8a-414d-ac9e-7206883933e6", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { Terminator.NumberTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { Terminator.IdentifierTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { new Terminator(DelimiterType.LeftParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { new Terminator(KeywordType.Not), new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, }, "9192ebb2-aa8a-414d-ac9e-7206883933e6") },
		{ "ae32e061-59db-49fd-addd-2d5c3e9014a3", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { Terminator.NumberTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { Terminator.IdentifierTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { new Terminator(DelimiterType.LeftParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { new Terminator(KeywordType.Not), new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, }, "ae32e061-59db-49fd-addd-2d5c3e9014a3") },
		{ "5c4b371b-c686-4545-b641-4c196ca6b25f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { Terminator.NumberTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { Terminator.IdentifierTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { new Terminator(DelimiterType.LeftParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { new Terminator(KeywordType.Not), new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, }, "5c4b371b-c686-4545-b641-4c196ca6b25f") },
		{ "9342a1fb-ddc8-436f-b10d-c0b3b0a3d6e9", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "eb94136a-d85e-4562-aca7-5f9c8279483a"},}, new Dictionary<Terminator, ReduceInformation>{ }, "9342a1fb-ddc8-436f-b10d-c0b3b0a3d6e9") },
		{ "38eec331-d4ae-4b06-a659-98475fdb5aa0", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.RelationOperator), "6c1cf854-f573-4675-95fc-d0289e4e8ca2"}, { new Terminator(OperatorType.Equal), "6112dd54-ce34-4dd2-be4c-b06dd725bbd8"}, { new Terminator(OperatorType.NotEqual), "1338afe1-da9e-4380-8a28-7f2d82ef2d68"}, { new Terminator(OperatorType.Less), "fc9b446b-c4b5-4720-bc46-49344508202f"}, { new Terminator(OperatorType.LessEqual), "39ac36f9-efe7-4a1b-ba37-a9729fbf341a"}, { new Terminator(OperatorType.Greater), "fc85ac1b-5a0f-405f-a8f0-a1e20d44351d"}, { new Terminator(OperatorType.GreaterEqual), "f0598e66-3db3-44bd-8b9d-cd6c038df056"}, { new NonTerminator(NonTerminatorType.AddOperator), "f909a793-4d7a-48c0-b17f-90aec0056eef"}, { new Terminator(OperatorType.Plus), "493d6d76-ed65-4308-aee5-65c5fe502546"}, { new Terminator(OperatorType.Minus), "c41b579b-0a72-411a-b9ed-f3ca30f5a615"}, { new Terminator(KeywordType.Or), "22878828-eed7-470f-a07f-595bb2eb7e2a"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Expression))}, }, "38eec331-d4ae-4b06-a659-98475fdb5aa0") },
		{ "5a5a9a53-cfe3-4e81-8fcf-2e6d11b16b0e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "a67b3d92-1e95-452c-aeb4-95fe5c85f9bc"}, { new Terminator(OperatorType.Multiply), "fd20e81d-52b7-4920-adab-e2a1e49a6b6a"}, { new Terminator(OperatorType.Divide), "ef7280a1-85cf-4590-ab87-68bdd867d9e5"}, { new Terminator(KeywordType.Divide), "9192ebb2-aa8a-414d-ac9e-7206883933e6"}, { new Terminator(KeywordType.Mod), "ae32e061-59db-49fd-addd-2d5c3e9014a3"}, { new Terminator(KeywordType.And), "5c4b371b-c686-4545-b641-4c196ca6b25f"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "5a5a9a53-cfe3-4e81-8fcf-2e6d11b16b0e") },
		{ "175e095a-0963-472f-b84a-ece518bd2cf2", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, }, "175e095a-0963-472f-b84a-ece518bd2cf2") },
		{ "3861f05c-791c-4f64-b291-d985540c7651", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "3861f05c-791c-4f64-b291-d985540c7651") },
		{ "c693eb95-cdcd-4c7d-8717-3ead45ef8bfc", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "c693eb95-cdcd-4c7d-8717-3ead45ef8bfc") },
		{ "efc9be5f-b54d-4b96-8aec-f3540a407dc8", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "c2dadcf2-f088-4a0c-bafa-8bbd3f52ed13"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "38eec331-d4ae-4b06-a659-98475fdb5aa0"}, { new NonTerminator(NonTerminatorType.Term), "5a5a9a53-cfe3-4e81-8fcf-2e6d11b16b0e"}, { new NonTerminator(NonTerminatorType.Factor), "175e095a-0963-472f-b84a-ece518bd2cf2"}, { Terminator.NumberTerminator, "3861f05c-791c-4f64-b291-d985540c7651"}, { new NonTerminator(NonTerminatorType.Variable), "c693eb95-cdcd-4c7d-8717-3ead45ef8bfc"}, { new Terminator(DelimiterType.LeftParenthesis), "efc9be5f-b54d-4b96-8aec-f3540a407dc8"}, { Terminator.IdentifierTerminator, "a8ce470f-1537-428f-88c2-9270bfed81b6"}, { new Terminator(KeywordType.Not), "addb6420-8367-48a9-9f28-327cf2a88028"}, { new Terminator(OperatorType.Minus), "75101aaf-bd06-4db5-809c-9e8b4533dc5d"},}, new Dictionary<Terminator, ReduceInformation>{ }, "efc9be5f-b54d-4b96-8aec-f3540a407dc8") },
		{ "a8ce470f-1537-428f-88c2-9270bfed81b6", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.LeftParenthesis), "5eea440c-8df5-4a70-8341-7a02a72fd8a0"}, { new NonTerminator(NonTerminatorType.IdVarPart), "c413807e-fe57-41d5-9d0a-c05a17f5f476"}, { new Terminator(DelimiterType.LeftSquareBracket), "1185a1ac-c815-461a-96a7-256441a0df65"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Equal), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Less), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Greater), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "a8ce470f-1537-428f-88c2-9270bfed81b6") },
		{ "addb6420-8367-48a9-9f28-327cf2a88028", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "5cfb5049-bea0-478f-8923-d9a46e76ad4a"}, { Terminator.NumberTerminator, "3861f05c-791c-4f64-b291-d985540c7651"}, { new NonTerminator(NonTerminatorType.Variable), "c693eb95-cdcd-4c7d-8717-3ead45ef8bfc"}, { new Terminator(DelimiterType.LeftParenthesis), "efc9be5f-b54d-4b96-8aec-f3540a407dc8"}, { Terminator.IdentifierTerminator, "a8ce470f-1537-428f-88c2-9270bfed81b6"}, { new Terminator(KeywordType.Not), "addb6420-8367-48a9-9f28-327cf2a88028"}, { new Terminator(OperatorType.Minus), "75101aaf-bd06-4db5-809c-9e8b4533dc5d"},}, new Dictionary<Terminator, ReduceInformation>{ }, "addb6420-8367-48a9-9f28-327cf2a88028") },
		{ "75101aaf-bd06-4db5-809c-9e8b4533dc5d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "99d45867-6a29-4bd7-9323-016a9a6b8757"}, { Terminator.NumberTerminator, "3861f05c-791c-4f64-b291-d985540c7651"}, { new NonTerminator(NonTerminatorType.Variable), "c693eb95-cdcd-4c7d-8717-3ead45ef8bfc"}, { new Terminator(DelimiterType.LeftParenthesis), "efc9be5f-b54d-4b96-8aec-f3540a407dc8"}, { Terminator.IdentifierTerminator, "a8ce470f-1537-428f-88c2-9270bfed81b6"}, { new Terminator(KeywordType.Not), "addb6420-8367-48a9-9f28-327cf2a88028"}, { new Terminator(OperatorType.Minus), "75101aaf-bd06-4db5-809c-9e8b4533dc5d"},}, new Dictionary<Terminator, ReduceInformation>{ }, "75101aaf-bd06-4db5-809c-9e8b4533dc5d") },
		{ "b71eba49-3129-4617-b054-7280e83921ec", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "ae2cf852-96b8-453d-adb2-47f0a326e8a0"}, { new NonTerminator(NonTerminatorType.Expression), "a582ab7f-9aa1-4130-9df3-2ba9dd0fe6f3"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "c3098ddd-d7d7-4d8b-b9ab-1ebedac5467f"}, { new NonTerminator(NonTerminatorType.Term), "c83de3f5-4130-4b2e-8715-e72b2e35275e"}, { new NonTerminator(NonTerminatorType.Factor), "9c184da0-a0b4-4ae3-9b1e-93b0fe09fbf1"}, { Terminator.NumberTerminator, "eaf4d875-7318-4f6e-b25b-829d30a888b8"}, { new NonTerminator(NonTerminatorType.Variable), "eb827526-1aa2-47ab-8b01-b1b0792e4297"}, { new Terminator(DelimiterType.LeftParenthesis), "2894d0f9-3faf-4be5-b2b6-c499611fc58d"}, { Terminator.IdentifierTerminator, "8efce527-7ba0-435f-a58f-b048a7a0e505"}, { new Terminator(KeywordType.Not), "dd22d779-6929-400f-832a-6d4e7eda69e6"}, { new Terminator(OperatorType.Minus), "28fb603c-e1e9-4bfa-ab9b-f8887e1996c0"},}, new Dictionary<Terminator, ReduceInformation>{ }, "b71eba49-3129-4617-b054-7280e83921ec") },
		{ "fe05a345-7dd1-4126-9002-64a5775d163f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, }, "fe05a345-7dd1-4126-9002-64a5775d163f") },
		{ "91b63ce6-a1e3-4083-9c56-27d8bd63c511", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "23e6496a-387e-45e7-9343-1727af61d29a"}, { new NonTerminator(NonTerminatorType.Expression), "10204ac0-c523-4b6c-890d-d508a8e83f48"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "72d6e812-1720-408a-ac1c-5ac20fa267d1"}, { new NonTerminator(NonTerminatorType.Term), "0b3b44d2-8458-48d1-b7c4-2329d479b56c"}, { new NonTerminator(NonTerminatorType.Factor), "2f595e0e-c29e-494d-a5bb-a570db3254fd"}, { Terminator.NumberTerminator, "c86f637c-4e17-4d3d-bfd7-f1fc5abdf7f4"}, { new NonTerminator(NonTerminatorType.Variable), "fa555a17-89d0-4cf7-989a-0b0775ade6d4"}, { new Terminator(DelimiterType.LeftParenthesis), "8fab798a-03a8-45b7-8380-7b7eaee651fa"}, { Terminator.IdentifierTerminator, "513b9c84-fc68-4334-976d-7547b1ec521f"}, { new Terminator(KeywordType.Not), "928a980c-85a8-4cf5-a5d1-117516baef4e"}, { new Terminator(OperatorType.Minus), "e2c085de-e99b-4f3a-b869-c3ca5f5dcf75"},}, new Dictionary<Terminator, ReduceInformation>{ }, "91b63ce6-a1e3-4083-9c56-27d8bd63c511") },
		{ "cbf67630-94d3-48ab-b3c4-95ec5ba223e1", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "cbf67630-94d3-48ab-b3c4-95ec5ba223e1") },
		{ "0ba032d7-8bf5-4092-befb-8119f507e038", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "0ba032d7-8bf5-4092-befb-8119f507e038") },
		{ "9045ad7a-6b46-4794-b88f-377068357369", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "8f2d1e59-ae2a-4a98-b7a5-ec5d38c46abd"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "cd7ff125-4a37-4908-ae74-cd22185c30cb"}, { new NonTerminator(NonTerminatorType.Term), "1b085cbe-da51-490b-a30b-b52e6f8a3897"}, { new NonTerminator(NonTerminatorType.Factor), "e8c6655e-9ea9-40f9-99d2-7d55a1cdf5dd"}, { Terminator.NumberTerminator, "a3cb4c23-4b02-4dd3-a2f3-1c50b7a1a951"}, { new NonTerminator(NonTerminatorType.Variable), "138c0386-8ea0-455a-a824-c6b28e3537c9"}, { new Terminator(DelimiterType.LeftParenthesis), "5b8d9c0d-8dbc-4eb2-bd3f-537c3984b48e"}, { Terminator.IdentifierTerminator, "8e45f22e-454f-47f0-8ef0-2b3c0fa1878b"}, { new Terminator(KeywordType.Not), "14e584b7-b902-49cf-95c1-16d6d0e40ac1"}, { new Terminator(OperatorType.Minus), "b723344c-19d8-4ecf-a0aa-eb8cc3782aff"},}, new Dictionary<Terminator, ReduceInformation>{ }, "9045ad7a-6b46-4794-b88f-377068357369") },
		{ "082995dc-2c94-48ec-8c49-c5524d1c27c0", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.CompoundStatement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.CompoundStatement))}, }, "082995dc-2c94-48ec-8c49-c5524d1c27c0") },
		{ "a56f5613-4494-4f75-8cf7-2ada0a4b24c7", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.CompoundStatement), "63a3d7e2-f9f7-43d7-8941-6fbb0db589b5"}, { new Terminator(KeywordType.Begin), "fe33641c-ef48-4618-a1af-f852cefa2062"},}, new Dictionary<Terminator, ReduceInformation>{ }, "a56f5613-4494-4f75-8cf7-2ada0a4b24c7") },
		{ "07b6153d-bdb1-4356-8c72-ff4c3614d9b8", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.VarDeclaration), "194ca5b3-e196-43f8-b1b5-fe180e9d3da2"}, { new NonTerminator(NonTerminatorType.IdentifierList), "98e439b2-6456-497d-be3a-2d44c50a720f"}, { Terminator.IdentifierTerminator, "3ed14f62-8e9f-4c1a-8e1e-5d522192093b"},}, new Dictionary<Terminator, ReduceInformation>{ }, "07b6153d-bdb1-4356-8c72-ff4c3614d9b8") },
		{ "1477df97-a3c9-4545-bc37-6e43589e9142", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.Semicolon), "58fe0694-747e-4b48-8ac1-bd58f454f186"},}, new Dictionary<Terminator, ReduceInformation>{ }, "1477df97-a3c9-4545-bc37-6e43589e9142") },
		{ "0138ca0c-7f0d-4cab-8ba5-0c896f14af03", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "5baa3173-72cb-47f6-9101-5a6016508ff6"}, { new Terminator(DelimiterType.Semicolon), "1d3a1a40-39ba-4bb3-9015-595ae01650de"},}, new Dictionary<Terminator, ReduceInformation>{ }, "0138ca0c-7f0d-4cab-8ba5-0c896f14af03") },
		{ "57f152c5-b52e-43da-938f-dcd99f48515d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.ParameterList))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.ParameterList))}, }, "57f152c5-b52e-43da-938f-dcd99f48515d") },
		{ "1c54abd3-ad5a-460b-b84c-508c30c2925b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Parameter))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Parameter))}, }, "1c54abd3-ad5a-460b-b84c-508c30c2925b") },
		{ "7d1b8c5c-7295-4e38-9399-c7e99513b3af", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Parameter))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Parameter))}, }, "7d1b8c5c-7295-4e38-9399-c7e99513b3af") },
		{ "eb3fbdf4-847d-4499-9851-d06d9fc995b9", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ValueParameter), "acb91407-eb13-42fa-8719-77d9a4a905d9"}, { new NonTerminator(NonTerminatorType.IdentifierList), "8129be83-68fb-4cb1-b760-6daf04d786a9"}, { Terminator.IdentifierTerminator, "3ed14f62-8e9f-4c1a-8e1e-5d522192093b"},}, new Dictionary<Terminator, ReduceInformation>{ }, "eb3fbdf4-847d-4499-9851-d06d9fc995b9") },
		{ "8129be83-68fb-4cb1-b760-6daf04d786a9", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.Colon), "1cf3f054-1f43-4c28-9181-45664b9775b4"}, { new Terminator(DelimiterType.Comma), "19b90e6e-1e3b-4063-b92b-d3f3ad667ca2"},}, new Dictionary<Terminator, ReduceInformation>{ }, "8129be83-68fb-4cb1-b760-6daf04d786a9") },
		{ "14ab58de-ab30-47ea-9429-f61a28fbbf18", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.BasicType), "bcbc127a-aa3f-4d7d-9df6-0dd240787f8e"}, { new Terminator(KeywordType.Integer), "e3c9fab2-9cd8-44ad-a33c-54482b5323d5"}, { new Terminator(KeywordType.Real), "3bb38fcd-bf5c-401a-a2e4-f6f4d91bc650"}, { new Terminator(KeywordType.Boolean), "01a76185-0283-464f-b7d6-9f5bb8517edd"}, { new Terminator(KeywordType.Character), "ee488773-f67c-418c-a1a5-ddd934d4253b"},}, new Dictionary<Terminator, ReduceInformation>{ }, "14ab58de-ab30-47ea-9429-f61a28fbbf18") },
		{ "f2561476-ccd0-4b72-a032-d67ab6815303", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "d591c2ea-49de-49d4-8026-f7cbb1ff0004"}, { new Terminator(DelimiterType.Semicolon), "1d3a1a40-39ba-4bb3-9015-595ae01650de"},}, new Dictionary<Terminator, ReduceInformation>{ }, "f2561476-ccd0-4b72-a032-d67ab6815303") },
		{ "32848038-9d3c-4589-b6ef-001ab9c0d5d7", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(5, new NonTerminator(NonTerminatorType.VarDeclaration))}, }, "32848038-9d3c-4589-b6ef-001ab9c0d5d7") },
		{ "9e8bb750-816a-4fef-a639-edc142eeec24", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightSquareBracket), "3bb5f984-8ec0-4021-b6a4-5fedcd5b64d2"}, { new Terminator(DelimiterType.Comma), "a0a8f3b7-6141-4883-83c9-fc7585981ef6"},}, new Dictionary<Terminator, ReduceInformation>{ }, "9e8bb750-816a-4fef-a639-edc142eeec24") },
		{ "f0f673dc-bbe7-4f92-ae61-9169b47aa68f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.DoubleDots), "6ac877b0-965b-4a98-8109-50fadbccdba4"},}, new Dictionary<Terminator, ReduceInformation>{ }, "f0f673dc-bbe7-4f92-ae61-9169b47aa68f") },
		{ "80000b15-c90f-413e-bf26-7daf48e05eac", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.SimpleExpression), "c7e82e7f-a75b-4f20-90d7-5af1e60eeeae"}, { new NonTerminator(NonTerminatorType.Term), "e61b6949-0204-4809-a7a8-c14193efa53c"}, { new NonTerminator(NonTerminatorType.Factor), "19cfe0eb-0189-4bfa-83ef-442a36cd1138"}, { Terminator.NumberTerminator, "ff4e43b4-9264-41ed-8fd4-fe02b4673621"}, { new NonTerminator(NonTerminatorType.Variable), "70dc3750-b84d-4199-808f-50c3ab8858e0"}, { new Terminator(DelimiterType.LeftParenthesis), "be22425a-a8f1-4ac2-b180-9c579b063d86"}, { Terminator.IdentifierTerminator, "75199901-29ad-4fde-8e16-2a9ce7439b9d"}, { new Terminator(KeywordType.Not), "ffcfe34e-f488-4c19-b03c-f14a0c709964"}, { new Terminator(OperatorType.Minus), "be7ba5b3-c18b-4687-82d8-69e621837916"},}, new Dictionary<Terminator, ReduceInformation>{ }, "80000b15-c90f-413e-bf26-7daf48e05eac") },
		{ "e31b812e-51e9-4a26-8a57-69e1398856a1", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Term), "a5d072d8-4dbe-401f-bffd-52987b4304f9"}, { new NonTerminator(NonTerminatorType.Factor), "ed166eeb-11fc-4025-a5ff-df1fb55129bf"}, { Terminator.NumberTerminator, "da7ed270-1de2-409d-8992-b6e78325e8ec"}, { new NonTerminator(NonTerminatorType.Variable), "16141db6-7be2-4636-b42e-2eb50645928b"}, { new Terminator(DelimiterType.LeftParenthesis), "39c7ef5b-d272-43f7-83ac-975e6142661c"}, { Terminator.IdentifierTerminator, "97c62080-8f8c-4d05-90bb-f8e531abdc6e"}, { new Terminator(KeywordType.Not), "753e7f13-2ad6-442a-b578-5d4205731d5d"}, { new Terminator(OperatorType.Minus), "5771e46b-f864-4859-a24b-ed54f3875040"},}, new Dictionary<Terminator, ReduceInformation>{ }, "e31b812e-51e9-4a26-8a57-69e1398856a1") },
		{ "7d5842f8-4715-4558-b43b-bdcfd821ece2", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "5903b318-099a-4afc-b2d0-c33cbd915a07"}, { Terminator.NumberTerminator, "da7ed270-1de2-409d-8992-b6e78325e8ec"}, { new NonTerminator(NonTerminatorType.Variable), "16141db6-7be2-4636-b42e-2eb50645928b"}, { new Terminator(DelimiterType.LeftParenthesis), "39c7ef5b-d272-43f7-83ac-975e6142661c"}, { Terminator.IdentifierTerminator, "97c62080-8f8c-4d05-90bb-f8e531abdc6e"}, { new Terminator(KeywordType.Not), "753e7f13-2ad6-442a-b578-5d4205731d5d"}, { new Terminator(OperatorType.Minus), "5771e46b-f864-4859-a24b-ed54f3875040"},}, new Dictionary<Terminator, ReduceInformation>{ }, "7d5842f8-4715-4558-b43b-bdcfd821ece2") },
		{ "692d8ac3-7f58-4b33-9866-edc32b01bd8f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "4cc3390c-dc0a-4bdd-bffb-90e9c1e979e5"},}, new Dictionary<Terminator, ReduceInformation>{ }, "692d8ac3-7f58-4b33-9866-edc32b01bd8f") },
		{ "22ab57e0-2d9d-4941-80a4-aba37acec74d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "a0b14868-8772-40ad-a8bb-28cd4a33e54b"}, { new NonTerminator(NonTerminatorType.Expression), "a582ab7f-9aa1-4130-9df3-2ba9dd0fe6f3"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "c3098ddd-d7d7-4d8b-b9ab-1ebedac5467f"}, { new NonTerminator(NonTerminatorType.Term), "c83de3f5-4130-4b2e-8715-e72b2e35275e"}, { new NonTerminator(NonTerminatorType.Factor), "9c184da0-a0b4-4ae3-9b1e-93b0fe09fbf1"}, { Terminator.NumberTerminator, "eaf4d875-7318-4f6e-b25b-829d30a888b8"}, { new NonTerminator(NonTerminatorType.Variable), "eb827526-1aa2-47ab-8b01-b1b0792e4297"}, { new Terminator(DelimiterType.LeftParenthesis), "2894d0f9-3faf-4be5-b2b6-c499611fc58d"}, { Terminator.IdentifierTerminator, "8efce527-7ba0-435f-a58f-b048a7a0e505"}, { new Terminator(KeywordType.Not), "dd22d779-6929-400f-832a-6d4e7eda69e6"}, { new Terminator(OperatorType.Minus), "28fb603c-e1e9-4bfa-ab9b-f8887e1996c0"},}, new Dictionary<Terminator, ReduceInformation>{ }, "22ab57e0-2d9d-4941-80a4-aba37acec74d") },
		{ "6d7b6d7d-3a6f-4dad-b34f-e3f93924f032", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, }, "6d7b6d7d-3a6f-4dad-b34f-e3f93924f032") },
		{ "fd1d9b8e-35a3-47cc-8259-87fcda434d9d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "2339d6bf-8732-4a03-aa14-4f0f2c574920"}, { new NonTerminator(NonTerminatorType.Expression), "10204ac0-c523-4b6c-890d-d508a8e83f48"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "72d6e812-1720-408a-ac1c-5ac20fa267d1"}, { new NonTerminator(NonTerminatorType.Term), "0b3b44d2-8458-48d1-b7c4-2329d479b56c"}, { new NonTerminator(NonTerminatorType.Factor), "2f595e0e-c29e-494d-a5bb-a570db3254fd"}, { Terminator.NumberTerminator, "c86f637c-4e17-4d3d-bfd7-f1fc5abdf7f4"}, { new NonTerminator(NonTerminatorType.Variable), "fa555a17-89d0-4cf7-989a-0b0775ade6d4"}, { new Terminator(DelimiterType.LeftParenthesis), "8fab798a-03a8-45b7-8380-7b7eaee651fa"}, { Terminator.IdentifierTerminator, "513b9c84-fc68-4334-976d-7547b1ec521f"}, { new Terminator(KeywordType.Not), "928a980c-85a8-4cf5-a5d1-117516baef4e"}, { new Terminator(OperatorType.Minus), "e2c085de-e99b-4f3a-b869-c3ca5f5dcf75"},}, new Dictionary<Terminator, ReduceInformation>{ }, "fd1d9b8e-35a3-47cc-8259-87fcda434d9d") },
		{ "d2312a91-ced3-4372-8013-67cd3a25b751", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "d2312a91-ced3-4372-8013-67cd3a25b751") },
		{ "797625c3-fa5e-4a9c-8618-43cd9e880bbf", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "797625c3-fa5e-4a9c-8618-43cd9e880bbf") },
		{ "2d4bfcda-0d59-48fb-b52c-95e9843a3cd3", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(OperatorType.Assign), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "2d4bfcda-0d59-48fb-b52c-95e9843a3cd3") },
		{ "ecf84275-ae5c-4a59-96fb-d2ef13a595a8", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "fd16c929-4cb4-4445-a854-68564b63eb12"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "72d6e812-1720-408a-ac1c-5ac20fa267d1"}, { new NonTerminator(NonTerminatorType.Term), "0b3b44d2-8458-48d1-b7c4-2329d479b56c"}, { new NonTerminator(NonTerminatorType.Factor), "2f595e0e-c29e-494d-a5bb-a570db3254fd"}, { Terminator.NumberTerminator, "c86f637c-4e17-4d3d-bfd7-f1fc5abdf7f4"}, { new NonTerminator(NonTerminatorType.Variable), "fa555a17-89d0-4cf7-989a-0b0775ade6d4"}, { new Terminator(DelimiterType.LeftParenthesis), "8fab798a-03a8-45b7-8380-7b7eaee651fa"}, { Terminator.IdentifierTerminator, "513b9c84-fc68-4334-976d-7547b1ec521f"}, { new Terminator(KeywordType.Not), "928a980c-85a8-4cf5-a5d1-117516baef4e"}, { new Terminator(OperatorType.Minus), "e2c085de-e99b-4f3a-b869-c3ca5f5dcf75"},}, new Dictionary<Terminator, ReduceInformation>{ }, "ecf84275-ae5c-4a59-96fb-d2ef13a595a8") },
		{ "2a4bc2d8-0766-4529-bb16-5702b468cbd3", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.SimpleExpression), "6cd3b153-5fda-4872-9c55-04ff10ff0ecc"}, { new NonTerminator(NonTerminatorType.Term), "1ecca4ed-8722-4b0d-afb1-365c050b9b25"}, { new NonTerminator(NonTerminatorType.Factor), "13f24f9d-eafa-49d3-981d-e9708deefa70"}, { Terminator.NumberTerminator, "c36a5a82-356a-4f27-9507-4853647330f8"}, { new NonTerminator(NonTerminatorType.Variable), "25d3aea4-fc9e-41ac-a0ba-db8ef11cee57"}, { new Terminator(DelimiterType.LeftParenthesis), "801568bc-0c2a-423f-9fd4-7f216d41b5c8"}, { Terminator.IdentifierTerminator, "62d908d5-bc3f-41cd-9d71-fff26a1343c1"}, { new Terminator(KeywordType.Not), "2269c88f-87d2-4c54-9270-380d70d9ec35"}, { new Terminator(OperatorType.Minus), "3c697023-0a28-43a8-be5b-ef957bbf16a6"},}, new Dictionary<Terminator, ReduceInformation>{ }, "2a4bc2d8-0766-4529-bb16-5702b468cbd3") },
		{ "1a33374a-916c-4e8c-9dab-a16669c0132b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Term), "4f68cbaf-b771-4f09-83ab-8d7283eb0c3c"}, { new NonTerminator(NonTerminatorType.Factor), "2f595e0e-c29e-494d-a5bb-a570db3254fd"}, { Terminator.NumberTerminator, "c86f637c-4e17-4d3d-bfd7-f1fc5abdf7f4"}, { new NonTerminator(NonTerminatorType.Variable), "fa555a17-89d0-4cf7-989a-0b0775ade6d4"}, { new Terminator(DelimiterType.LeftParenthesis), "8fab798a-03a8-45b7-8380-7b7eaee651fa"}, { Terminator.IdentifierTerminator, "513b9c84-fc68-4334-976d-7547b1ec521f"}, { new Terminator(KeywordType.Not), "928a980c-85a8-4cf5-a5d1-117516baef4e"}, { new Terminator(OperatorType.Minus), "e2c085de-e99b-4f3a-b869-c3ca5f5dcf75"},}, new Dictionary<Terminator, ReduceInformation>{ }, "1a33374a-916c-4e8c-9dab-a16669c0132b") },
		{ "05f80b0f-765b-49de-8b00-897364786373", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "787200ba-571b-433b-9e83-c4ed871f52c5"}, { Terminator.NumberTerminator, "c86f637c-4e17-4d3d-bfd7-f1fc5abdf7f4"}, { new NonTerminator(NonTerminatorType.Variable), "fa555a17-89d0-4cf7-989a-0b0775ade6d4"}, { new Terminator(DelimiterType.LeftParenthesis), "8fab798a-03a8-45b7-8380-7b7eaee651fa"}, { Terminator.IdentifierTerminator, "513b9c84-fc68-4334-976d-7547b1ec521f"}, { new Terminator(KeywordType.Not), "928a980c-85a8-4cf5-a5d1-117516baef4e"}, { new Terminator(OperatorType.Minus), "e2c085de-e99b-4f3a-b869-c3ca5f5dcf75"},}, new Dictionary<Terminator, ReduceInformation>{ }, "05f80b0f-765b-49de-8b00-897364786373") },
		{ "405c6e37-2298-45d6-a73a-164eafdabaa7", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "0c210077-0859-488b-9991-5fcac82cce1a"},}, new Dictionary<Terminator, ReduceInformation>{ }, "405c6e37-2298-45d6-a73a-164eafdabaa7") },
		{ "157e56f1-333f-452b-b1f3-259c853036e1", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "06846c83-218a-4305-b749-00df4561ab21"}, { new NonTerminator(NonTerminatorType.Expression), "a582ab7f-9aa1-4130-9df3-2ba9dd0fe6f3"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "c3098ddd-d7d7-4d8b-b9ab-1ebedac5467f"}, { new NonTerminator(NonTerminatorType.Term), "c83de3f5-4130-4b2e-8715-e72b2e35275e"}, { new NonTerminator(NonTerminatorType.Factor), "9c184da0-a0b4-4ae3-9b1e-93b0fe09fbf1"}, { Terminator.NumberTerminator, "eaf4d875-7318-4f6e-b25b-829d30a888b8"}, { new NonTerminator(NonTerminatorType.Variable), "eb827526-1aa2-47ab-8b01-b1b0792e4297"}, { new Terminator(DelimiterType.LeftParenthesis), "2894d0f9-3faf-4be5-b2b6-c499611fc58d"}, { Terminator.IdentifierTerminator, "8efce527-7ba0-435f-a58f-b048a7a0e505"}, { new Terminator(KeywordType.Not), "dd22d779-6929-400f-832a-6d4e7eda69e6"}, { new Terminator(OperatorType.Minus), "28fb603c-e1e9-4bfa-ab9b-f8887e1996c0"},}, new Dictionary<Terminator, ReduceInformation>{ }, "157e56f1-333f-452b-b1f3-259c853036e1") },
		{ "1c80bd03-b6b0-416f-82c0-97c5438ebd3f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, }, "1c80bd03-b6b0-416f-82c0-97c5438ebd3f") },
		{ "c9feaf42-307a-4617-ba76-11ba0b77026b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "78bebc29-c88c-4d86-bd44-d5f34b99570b"}, { new NonTerminator(NonTerminatorType.Expression), "10204ac0-c523-4b6c-890d-d508a8e83f48"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "72d6e812-1720-408a-ac1c-5ac20fa267d1"}, { new NonTerminator(NonTerminatorType.Term), "0b3b44d2-8458-48d1-b7c4-2329d479b56c"}, { new NonTerminator(NonTerminatorType.Factor), "2f595e0e-c29e-494d-a5bb-a570db3254fd"}, { Terminator.NumberTerminator, "c86f637c-4e17-4d3d-bfd7-f1fc5abdf7f4"}, { new NonTerminator(NonTerminatorType.Variable), "fa555a17-89d0-4cf7-989a-0b0775ade6d4"}, { new Terminator(DelimiterType.LeftParenthesis), "8fab798a-03a8-45b7-8380-7b7eaee651fa"}, { Terminator.IdentifierTerminator, "513b9c84-fc68-4334-976d-7547b1ec521f"}, { new Terminator(KeywordType.Not), "928a980c-85a8-4cf5-a5d1-117516baef4e"}, { new Terminator(OperatorType.Minus), "e2c085de-e99b-4f3a-b869-c3ca5f5dcf75"},}, new Dictionary<Terminator, ReduceInformation>{ }, "c9feaf42-307a-4617-ba76-11ba0b77026b") },
		{ "0c1bde85-24a9-4869-9280-92292c185727", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "0c1bde85-24a9-4869-9280-92292c185727") },
		{ "1a7da938-83c9-4ef7-b1b7-4bee5c69c377", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "1a7da938-83c9-4ef7-b1b7-4bee5c69c377") },
		{ "9bb1761c-62b0-40f5-a91e-591a8edf560c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(4, new NonTerminator(NonTerminatorType.ProcedureCall))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(4, new NonTerminator(NonTerminatorType.ProcedureCall))}, }, "9bb1761c-62b0-40f5-a91e-591a8edf560c") },
		{ "dd4993c2-1db9-4e5a-a3f2-c4660ec4e365", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "43891951-79a3-429b-b599-3d57bf0975f7"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "c3098ddd-d7d7-4d8b-b9ab-1ebedac5467f"}, { new NonTerminator(NonTerminatorType.Term), "c83de3f5-4130-4b2e-8715-e72b2e35275e"}, { new NonTerminator(NonTerminatorType.Factor), "9c184da0-a0b4-4ae3-9b1e-93b0fe09fbf1"}, { Terminator.NumberTerminator, "eaf4d875-7318-4f6e-b25b-829d30a888b8"}, { new NonTerminator(NonTerminatorType.Variable), "eb827526-1aa2-47ab-8b01-b1b0792e4297"}, { new Terminator(DelimiterType.LeftParenthesis), "2894d0f9-3faf-4be5-b2b6-c499611fc58d"}, { Terminator.IdentifierTerminator, "8efce527-7ba0-435f-a58f-b048a7a0e505"}, { new Terminator(KeywordType.Not), "dd22d779-6929-400f-832a-6d4e7eda69e6"}, { new Terminator(OperatorType.Minus), "28fb603c-e1e9-4bfa-ab9b-f8887e1996c0"},}, new Dictionary<Terminator, ReduceInformation>{ }, "dd4993c2-1db9-4e5a-a3f2-c4660ec4e365") },
		{ "a9416155-af8f-4c09-90a9-8ac8c574bf91", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.SimpleExpression), "069f86f9-ca28-44c5-8f86-a470b4d1441c"}, { new NonTerminator(NonTerminatorType.Term), "687ca7fc-755d-4d28-826e-19f6f2e5359f"}, { new NonTerminator(NonTerminatorType.Factor), "a6ee60f9-e7a6-4693-a4e8-cd41934b1727"}, { Terminator.NumberTerminator, "a7df07cd-c0da-4e90-8871-a13944fae98c"}, { new NonTerminator(NonTerminatorType.Variable), "265ce997-0847-47ff-84ad-380765fb1439"}, { new Terminator(DelimiterType.LeftParenthesis), "1e10eeed-1b88-45a5-a914-bbcf404cda86"}, { Terminator.IdentifierTerminator, "55db5714-9404-40ae-a5cc-d7cd7968f2be"}, { new Terminator(KeywordType.Not), "cba1e33d-813e-4f0e-9bda-ba7f1020c426"}, { new Terminator(OperatorType.Minus), "4689d95f-7bbb-41de-9f96-ba34da344607"},}, new Dictionary<Terminator, ReduceInformation>{ }, "a9416155-af8f-4c09-90a9-8ac8c574bf91") },
		{ "eff6eeb5-c251-4d32-89b4-2fd1a8cd2069", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Term), "ef1bad8e-e7b6-4bc6-b0fb-242cff06ea74"}, { new NonTerminator(NonTerminatorType.Factor), "9c184da0-a0b4-4ae3-9b1e-93b0fe09fbf1"}, { Terminator.NumberTerminator, "eaf4d875-7318-4f6e-b25b-829d30a888b8"}, { new NonTerminator(NonTerminatorType.Variable), "eb827526-1aa2-47ab-8b01-b1b0792e4297"}, { new Terminator(DelimiterType.LeftParenthesis), "2894d0f9-3faf-4be5-b2b6-c499611fc58d"}, { Terminator.IdentifierTerminator, "8efce527-7ba0-435f-a58f-b048a7a0e505"}, { new Terminator(KeywordType.Not), "dd22d779-6929-400f-832a-6d4e7eda69e6"}, { new Terminator(OperatorType.Minus), "28fb603c-e1e9-4bfa-ab9b-f8887e1996c0"},}, new Dictionary<Terminator, ReduceInformation>{ }, "eff6eeb5-c251-4d32-89b4-2fd1a8cd2069") },
		{ "7964d212-7d90-4920-b7c1-abae26705284", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "2585b2b3-825d-4918-a65f-6bf0e2b91a04"}, { Terminator.NumberTerminator, "eaf4d875-7318-4f6e-b25b-829d30a888b8"}, { new NonTerminator(NonTerminatorType.Variable), "eb827526-1aa2-47ab-8b01-b1b0792e4297"}, { new Terminator(DelimiterType.LeftParenthesis), "2894d0f9-3faf-4be5-b2b6-c499611fc58d"}, { Terminator.IdentifierTerminator, "8efce527-7ba0-435f-a58f-b048a7a0e505"}, { new Terminator(KeywordType.Not), "dd22d779-6929-400f-832a-6d4e7eda69e6"}, { new Terminator(OperatorType.Minus), "28fb603c-e1e9-4bfa-ab9b-f8887e1996c0"},}, new Dictionary<Terminator, ReduceInformation>{ }, "7964d212-7d90-4920-b7c1-abae26705284") },
		{ "ca0075b4-b452-481e-a378-faec2699ad4c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "7e010031-8e0f-4159-a5ca-a4d6a43f32c4"},}, new Dictionary<Terminator, ReduceInformation>{ }, "ca0075b4-b452-481e-a378-faec2699ad4c") },
		{ "bcbae121-f455-43db-ad52-0d675a786634", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "872686a0-a439-401f-aef4-fd55a5e4de9c"}, { new NonTerminator(NonTerminatorType.Expression), "a582ab7f-9aa1-4130-9df3-2ba9dd0fe6f3"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "c3098ddd-d7d7-4d8b-b9ab-1ebedac5467f"}, { new NonTerminator(NonTerminatorType.Term), "c83de3f5-4130-4b2e-8715-e72b2e35275e"}, { new NonTerminator(NonTerminatorType.Factor), "9c184da0-a0b4-4ae3-9b1e-93b0fe09fbf1"}, { Terminator.NumberTerminator, "eaf4d875-7318-4f6e-b25b-829d30a888b8"}, { new NonTerminator(NonTerminatorType.Variable), "eb827526-1aa2-47ab-8b01-b1b0792e4297"}, { new Terminator(DelimiterType.LeftParenthesis), "2894d0f9-3faf-4be5-b2b6-c499611fc58d"}, { Terminator.IdentifierTerminator, "8efce527-7ba0-435f-a58f-b048a7a0e505"}, { new Terminator(KeywordType.Not), "dd22d779-6929-400f-832a-6d4e7eda69e6"}, { new Terminator(OperatorType.Minus), "28fb603c-e1e9-4bfa-ab9b-f8887e1996c0"},}, new Dictionary<Terminator, ReduceInformation>{ }, "bcbae121-f455-43db-ad52-0d675a786634") },
		{ "07a78aea-a3f7-4f15-87c9-3c42b7914284", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, }, "07a78aea-a3f7-4f15-87c9-3c42b7914284") },
		{ "af07aa24-5fd4-4118-acb1-c007eae6ed21", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "a20d51ab-d41b-4d08-9a84-225578db0f89"}, { new NonTerminator(NonTerminatorType.Expression), "10204ac0-c523-4b6c-890d-d508a8e83f48"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "72d6e812-1720-408a-ac1c-5ac20fa267d1"}, { new NonTerminator(NonTerminatorType.Term), "0b3b44d2-8458-48d1-b7c4-2329d479b56c"}, { new NonTerminator(NonTerminatorType.Factor), "2f595e0e-c29e-494d-a5bb-a570db3254fd"}, { Terminator.NumberTerminator, "c86f637c-4e17-4d3d-bfd7-f1fc5abdf7f4"}, { new NonTerminator(NonTerminatorType.Variable), "fa555a17-89d0-4cf7-989a-0b0775ade6d4"}, { new Terminator(DelimiterType.LeftParenthesis), "8fab798a-03a8-45b7-8380-7b7eaee651fa"}, { Terminator.IdentifierTerminator, "513b9c84-fc68-4334-976d-7547b1ec521f"}, { new Terminator(KeywordType.Not), "928a980c-85a8-4cf5-a5d1-117516baef4e"}, { new Terminator(OperatorType.Minus), "e2c085de-e99b-4f3a-b869-c3ca5f5dcf75"},}, new Dictionary<Terminator, ReduceInformation>{ }, "af07aa24-5fd4-4118-acb1-c007eae6ed21") },
		{ "18804690-9067-487a-9ea1-75c6519285ba", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "18804690-9067-487a-9ea1-75c6519285ba") },
		{ "708740cb-62db-4187-898f-e66eb17abe2c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "708740cb-62db-4187-898f-e66eb17abe2c") },
		{ "6b1deb83-9d0c-429d-8a32-c51d9db7cd8d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ElsePart), "c31614ec-b50c-49b7-ad94-d6dd716d918a"}, { new Terminator(KeywordType.Else), "8ca0d603-8db8-472b-9929-111f27f6f615"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(0, new NonTerminator(NonTerminatorType.ElsePart))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(0, new NonTerminator(NonTerminatorType.ElsePart))}, }, "6b1deb83-9d0c-429d-8a32-c51d9db7cd8d") },
		{ "c5d258c5-490b-4c90-8b5b-8242a6dbfc6d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(OperatorType.Assign), "2cabd519-a4b4-40c1-b999-f35a12a71c5a"},}, new Dictionary<Terminator, ReduceInformation>{ }, "c5d258c5-490b-4c90-8b5b-8242a6dbfc6d") },
		{ "9a2b8a88-a48e-46e2-af69-f3561ad39bb7", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(OperatorType.Assign), "8c9a5a6d-24b1-4ebc-9159-406925920da8"}, { new NonTerminator(NonTerminatorType.IdVarPart), "48402e50-8efc-4ee1-b0ac-b318b198cdc6"}, { new Terminator(DelimiterType.LeftSquareBracket), "ad1a1d2d-24d0-4c6e-8654-b1025dbada2f"}, { new Terminator(DelimiterType.LeftParenthesis), "ea8355f8-a135-4d5d-8221-9125bef12329"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(OperatorType.Assign), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.ProcedureCall))}, { new Terminator(KeywordType.Else), new ReduceInformation(1, new NonTerminator(NonTerminatorType.ProcedureCall))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.ProcedureCall))}, }, "9a2b8a88-a48e-46e2-af69-f3561ad39bb7") },
		{ "e9f86400-0193-40f4-a076-09e7b9b028e9", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(KeywordType.Else), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Statement))}, }, "e9f86400-0193-40f4-a076-09e7b9b028e9") },
		{ "48fe99c9-db82-494e-b751-8bda645590d0", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(KeywordType.Else), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Statement))}, }, "48fe99c9-db82-494e-b751-8bda645590d0") },
		{ "79db3815-bc73-4868-ad20-1fc22d7453b9", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "d80554bd-0e20-4fb6-a6e8-35ed8a788a02"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "9372190c-03b2-4dd8-b6d0-ed9a16c071cc"}, { new NonTerminator(NonTerminatorType.Term), "4101b78d-0b68-4d0a-819e-ec5221d8ad24"}, { new NonTerminator(NonTerminatorType.Factor), "87546012-c006-4323-84d2-09d75a8399c5"}, { Terminator.NumberTerminator, "fa97211a-80b7-4975-9eee-548312d9b847"}, { new NonTerminator(NonTerminatorType.Variable), "18fb7818-280d-424d-acd7-c30c11393bbf"}, { new Terminator(DelimiterType.LeftParenthesis), "b9e87be5-24b1-455e-8377-274fab33e19a"}, { Terminator.IdentifierTerminator, "2568b015-15cf-42e5-bacf-64d5d56c3b0e"}, { new Terminator(KeywordType.Not), "bb6682cd-c5f2-49f5-a560-001ce41b69a5"}, { new Terminator(OperatorType.Minus), "0ebd6126-1360-4d5b-ba9c-0b49e8d814fa"},}, new Dictionary<Terminator, ReduceInformation>{ }, "79db3815-bc73-4868-ad20-1fc22d7453b9") },
		{ "ec4898d0-5ba7-47f1-885e-07725f2e43dc", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { Terminator.IdentifierTerminator, "4732a6c9-d6bb-442c-864d-8980e73ff547"},}, new Dictionary<Terminator, ReduceInformation>{ }, "ec4898d0-5ba7-47f1-885e-07725f2e43dc") },
		{ "65fb1933-38c9-42db-9907-36fe3c05573a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.StatementList), "cc957d7d-92b3-430f-b18b-5a2d4b5b5b14"}, { new NonTerminator(NonTerminatorType.Statement), "382f5704-ba25-4f9d-a658-af06ba9c8ecc"}, { new NonTerminator(NonTerminatorType.Variable), "ccb9ddb8-4ca9-4498-906e-81b654135662"}, { Terminator.IdentifierTerminator, "764301e0-1e52-4941-80fc-f768fabb848a"}, { new NonTerminator(NonTerminatorType.ProcedureCall), "d19e5b10-7ef3-4981-8c0b-deca4bfff866"}, { new NonTerminator(NonTerminatorType.CompoundStatement), "67b6f07f-8024-4a8f-8f06-3e29ee8141d8"}, { new Terminator(KeywordType.If), "1139d083-a185-4a24-a86e-2dfa3d0514e2"}, { new Terminator(KeywordType.For), "8337e624-bbce-4b9a-8231-b6b264eefd7f"}, { new Terminator(KeywordType.Begin), "5c23381d-aee0-4773-ab73-20aa2a344931"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, }, "65fb1933-38c9-42db-9907-36fe3c05573a") },
		{ "1bd301f5-2851-43b5-99aa-2b2e9c756746", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.AddOperator), "573debc2-b615-48ab-bfa9-ac1489967f6d"}, { new Terminator(OperatorType.Plus), "493d6d76-ed65-4308-aee5-65c5fe502546"}, { new Terminator(OperatorType.Minus), "c41b579b-0a72-411a-b9ed-f3ca30f5a615"}, { new Terminator(KeywordType.Or), "22878828-eed7-470f-a07f-595bb2eb7e2a"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Expression))}, }, "1bd301f5-2851-43b5-99aa-2b2e9c756746") },
		{ "dda70d13-8e10-4723-a967-73c2d4560abf", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "ce757e88-c309-4c8f-b512-fde449379b78"}, { new Terminator(OperatorType.Multiply), "fd20e81d-52b7-4920-adab-e2a1e49a6b6a"}, { new Terminator(OperatorType.Divide), "ef7280a1-85cf-4590-ab87-68bdd867d9e5"}, { new Terminator(KeywordType.Divide), "9192ebb2-aa8a-414d-ac9e-7206883933e6"}, { new Terminator(KeywordType.Mod), "ae32e061-59db-49fd-addd-2d5c3e9014a3"}, { new Terminator(KeywordType.And), "5c4b371b-c686-4545-b641-4c196ca6b25f"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "dda70d13-8e10-4723-a967-73c2d4560abf") },
		{ "44bb095c-dbed-40a4-a4e5-ca65d38ca2ac", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, }, "44bb095c-dbed-40a4-a4e5-ca65d38ca2ac") },
		{ "6abd39fc-1d52-41a2-ae4b-43a6b52f2537", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "6abd39fc-1d52-41a2-ae4b-43a6b52f2537") },
		{ "7b2c56ec-cb0f-41f6-a54b-d73c18298449", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "7b2c56ec-cb0f-41f6-a54b-d73c18298449") },
		{ "3e5d0461-383e-4afd-8bd2-4c3ac4afbf0b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "f11a3033-dc94-4fc8-bbae-61d3562ccd16"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "38eec331-d4ae-4b06-a659-98475fdb5aa0"}, { new NonTerminator(NonTerminatorType.Term), "5a5a9a53-cfe3-4e81-8fcf-2e6d11b16b0e"}, { new NonTerminator(NonTerminatorType.Factor), "175e095a-0963-472f-b84a-ece518bd2cf2"}, { Terminator.NumberTerminator, "3861f05c-791c-4f64-b291-d985540c7651"}, { new NonTerminator(NonTerminatorType.Variable), "c693eb95-cdcd-4c7d-8717-3ead45ef8bfc"}, { new Terminator(DelimiterType.LeftParenthesis), "efc9be5f-b54d-4b96-8aec-f3540a407dc8"}, { Terminator.IdentifierTerminator, "a8ce470f-1537-428f-88c2-9270bfed81b6"}, { new Terminator(KeywordType.Not), "addb6420-8367-48a9-9f28-327cf2a88028"}, { new Terminator(OperatorType.Minus), "75101aaf-bd06-4db5-809c-9e8b4533dc5d"},}, new Dictionary<Terminator, ReduceInformation>{ }, "3e5d0461-383e-4afd-8bd2-4c3ac4afbf0b") },
		{ "068b589e-cf17-40e4-baae-3f3b18ddb76e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.LeftParenthesis), "fd15c449-ac45-4bad-b4bb-a0f2f4ac7626"}, { new NonTerminator(NonTerminatorType.IdVarPart), "91afac8d-1047-4a2e-aed9-b3273bc6894f"}, { new Terminator(DelimiterType.LeftSquareBracket), "988ddf5d-71fe-4f8f-b49f-a24ad209479f"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "068b589e-cf17-40e4-baae-3f3b18ddb76e") },
		{ "f4285e75-83ea-4c5d-8403-e01bf0d9bbfb", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "0bbee844-15b8-463a-b72c-d1b393bba256"}, { Terminator.NumberTerminator, "6abd39fc-1d52-41a2-ae4b-43a6b52f2537"}, { new NonTerminator(NonTerminatorType.Variable), "7b2c56ec-cb0f-41f6-a54b-d73c18298449"}, { new Terminator(DelimiterType.LeftParenthesis), "3e5d0461-383e-4afd-8bd2-4c3ac4afbf0b"}, { Terminator.IdentifierTerminator, "068b589e-cf17-40e4-baae-3f3b18ddb76e"}, { new Terminator(KeywordType.Not), "f4285e75-83ea-4c5d-8403-e01bf0d9bbfb"}, { new Terminator(OperatorType.Minus), "a29a7e8a-8e5a-458a-8b41-cf17752ff1b1"},}, new Dictionary<Terminator, ReduceInformation>{ }, "f4285e75-83ea-4c5d-8403-e01bf0d9bbfb") },
		{ "a29a7e8a-8e5a-458a-8b41-cf17752ff1b1", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "a103d08c-8e5d-4cff-a781-718fd3d6e0fd"}, { Terminator.NumberTerminator, "6abd39fc-1d52-41a2-ae4b-43a6b52f2537"}, { new NonTerminator(NonTerminatorType.Variable), "7b2c56ec-cb0f-41f6-a54b-d73c18298449"}, { new Terminator(DelimiterType.LeftParenthesis), "3e5d0461-383e-4afd-8bd2-4c3ac4afbf0b"}, { Terminator.IdentifierTerminator, "068b589e-cf17-40e4-baae-3f3b18ddb76e"}, { new Terminator(KeywordType.Not), "f4285e75-83ea-4c5d-8403-e01bf0d9bbfb"}, { new Terminator(OperatorType.Minus), "a29a7e8a-8e5a-458a-8b41-cf17752ff1b1"},}, new Dictionary<Terminator, ReduceInformation>{ }, "a29a7e8a-8e5a-458a-8b41-cf17752ff1b1") },
		{ "2d37c287-549e-42c5-8326-211797dd6131", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "abc7c9b1-5843-493b-a7fc-ebb63e19a9b5"}, { new Terminator(OperatorType.Multiply), "fd20e81d-52b7-4920-adab-e2a1e49a6b6a"}, { new Terminator(OperatorType.Divide), "ef7280a1-85cf-4590-ab87-68bdd867d9e5"}, { new Terminator(KeywordType.Divide), "9192ebb2-aa8a-414d-ac9e-7206883933e6"}, { new Terminator(KeywordType.Mod), "ae32e061-59db-49fd-addd-2d5c3e9014a3"}, { new Terminator(KeywordType.And), "5c4b371b-c686-4545-b641-4c196ca6b25f"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "2d37c287-549e-42c5-8326-211797dd6131") },
		{ "bca1aaa9-0dda-477e-a6b4-f2e36b5a2d3f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, }, "bca1aaa9-0dda-477e-a6b4-f2e36b5a2d3f") },
		{ "eb94136a-d85e-4562-aca7-5f9c8279483a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "eb94136a-d85e-4562-aca7-5f9c8279483a") },
		{ "6c1cf854-f573-4675-95fc-d0289e4e8ca2", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.SimpleExpression), "e0733e10-922b-4841-9464-447f3c5afb17"}, { new NonTerminator(NonTerminatorType.Term), "6092fbdf-04e0-4ad1-88e7-787aac644669"}, { new NonTerminator(NonTerminatorType.Factor), "42bb7db9-5be9-4ac8-89ed-01578a50d2fc"}, { Terminator.NumberTerminator, "2a5a634a-a406-4e54-a148-6378d751f2c4"}, { new NonTerminator(NonTerminatorType.Variable), "bfc1ac1f-9a68-43c1-a75d-612f181a96d2"}, { new Terminator(DelimiterType.LeftParenthesis), "f6d9e32c-49ee-436c-ba97-fcdd527d9fdb"}, { Terminator.IdentifierTerminator, "77dacef1-16f6-4ede-93f5-3f7c1a45da85"}, { new Terminator(KeywordType.Not), "8763ccfb-4769-4111-8f72-0408293e631a"}, { new Terminator(OperatorType.Minus), "6abdc924-448f-4e57-8b91-afb7c5e8026c"},}, new Dictionary<Terminator, ReduceInformation>{ }, "6c1cf854-f573-4675-95fc-d0289e4e8ca2") },
		{ "f909a793-4d7a-48c0-b17f-90aec0056eef", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Term), "560cde6a-a7d2-42d0-8919-35ede21bbf77"}, { new NonTerminator(NonTerminatorType.Factor), "175e095a-0963-472f-b84a-ece518bd2cf2"}, { Terminator.NumberTerminator, "3861f05c-791c-4f64-b291-d985540c7651"}, { new NonTerminator(NonTerminatorType.Variable), "c693eb95-cdcd-4c7d-8717-3ead45ef8bfc"}, { new Terminator(DelimiterType.LeftParenthesis), "efc9be5f-b54d-4b96-8aec-f3540a407dc8"}, { Terminator.IdentifierTerminator, "a8ce470f-1537-428f-88c2-9270bfed81b6"}, { new Terminator(KeywordType.Not), "addb6420-8367-48a9-9f28-327cf2a88028"}, { new Terminator(OperatorType.Minus), "75101aaf-bd06-4db5-809c-9e8b4533dc5d"},}, new Dictionary<Terminator, ReduceInformation>{ }, "f909a793-4d7a-48c0-b17f-90aec0056eef") },
		{ "a67b3d92-1e95-452c-aeb4-95fe5c85f9bc", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "60557bde-770c-4a18-be29-61dccb476f41"}, { Terminator.NumberTerminator, "3861f05c-791c-4f64-b291-d985540c7651"}, { new NonTerminator(NonTerminatorType.Variable), "c693eb95-cdcd-4c7d-8717-3ead45ef8bfc"}, { new Terminator(DelimiterType.LeftParenthesis), "efc9be5f-b54d-4b96-8aec-f3540a407dc8"}, { Terminator.IdentifierTerminator, "a8ce470f-1537-428f-88c2-9270bfed81b6"}, { new Terminator(KeywordType.Not), "addb6420-8367-48a9-9f28-327cf2a88028"}, { new Terminator(OperatorType.Minus), "75101aaf-bd06-4db5-809c-9e8b4533dc5d"},}, new Dictionary<Terminator, ReduceInformation>{ }, "a67b3d92-1e95-452c-aeb4-95fe5c85f9bc") },
		{ "c2dadcf2-f088-4a0c-bafa-8bbd3f52ed13", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "d75e0d15-0a0e-4d7e-b9e5-ebd5c6359643"},}, new Dictionary<Terminator, ReduceInformation>{ }, "c2dadcf2-f088-4a0c-bafa-8bbd3f52ed13") },
		{ "5eea440c-8df5-4a70-8341-7a02a72fd8a0", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "1a6d2be9-bf26-44cc-97e4-6060948a8ddd"}, { new NonTerminator(NonTerminatorType.Expression), "a582ab7f-9aa1-4130-9df3-2ba9dd0fe6f3"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "c3098ddd-d7d7-4d8b-b9ab-1ebedac5467f"}, { new NonTerminator(NonTerminatorType.Term), "c83de3f5-4130-4b2e-8715-e72b2e35275e"}, { new NonTerminator(NonTerminatorType.Factor), "9c184da0-a0b4-4ae3-9b1e-93b0fe09fbf1"}, { Terminator.NumberTerminator, "eaf4d875-7318-4f6e-b25b-829d30a888b8"}, { new NonTerminator(NonTerminatorType.Variable), "eb827526-1aa2-47ab-8b01-b1b0792e4297"}, { new Terminator(DelimiterType.LeftParenthesis), "2894d0f9-3faf-4be5-b2b6-c499611fc58d"}, { Terminator.IdentifierTerminator, "8efce527-7ba0-435f-a58f-b048a7a0e505"}, { new Terminator(KeywordType.Not), "dd22d779-6929-400f-832a-6d4e7eda69e6"}, { new Terminator(OperatorType.Minus), "28fb603c-e1e9-4bfa-ab9b-f8887e1996c0"},}, new Dictionary<Terminator, ReduceInformation>{ }, "5eea440c-8df5-4a70-8341-7a02a72fd8a0") },
		{ "c413807e-fe57-41d5-9d0a-c05a17f5f476", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, }, "c413807e-fe57-41d5-9d0a-c05a17f5f476") },
		{ "1185a1ac-c815-461a-96a7-256441a0df65", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "a21bab7a-595c-441e-a87c-8e8407b7e04a"}, { new NonTerminator(NonTerminatorType.Expression), "10204ac0-c523-4b6c-890d-d508a8e83f48"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "72d6e812-1720-408a-ac1c-5ac20fa267d1"}, { new NonTerminator(NonTerminatorType.Term), "0b3b44d2-8458-48d1-b7c4-2329d479b56c"}, { new NonTerminator(NonTerminatorType.Factor), "2f595e0e-c29e-494d-a5bb-a570db3254fd"}, { Terminator.NumberTerminator, "c86f637c-4e17-4d3d-bfd7-f1fc5abdf7f4"}, { new NonTerminator(NonTerminatorType.Variable), "fa555a17-89d0-4cf7-989a-0b0775ade6d4"}, { new Terminator(DelimiterType.LeftParenthesis), "8fab798a-03a8-45b7-8380-7b7eaee651fa"}, { Terminator.IdentifierTerminator, "513b9c84-fc68-4334-976d-7547b1ec521f"}, { new Terminator(KeywordType.Not), "928a980c-85a8-4cf5-a5d1-117516baef4e"}, { new Terminator(OperatorType.Minus), "e2c085de-e99b-4f3a-b869-c3ca5f5dcf75"},}, new Dictionary<Terminator, ReduceInformation>{ }, "1185a1ac-c815-461a-96a7-256441a0df65") },
		{ "5cfb5049-bea0-478f-8923-d9a46e76ad4a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "5cfb5049-bea0-478f-8923-d9a46e76ad4a") },
		{ "99d45867-6a29-4bd7-9323-016a9a6b8757", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "99d45867-6a29-4bd7-9323-016a9a6b8757") },
		{ "ae2cf852-96b8-453d-adb2-47f0a326e8a0", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "c7a1b3aa-fefe-40e4-ab25-d9d38ea14dbb"}, { new Terminator(DelimiterType.Comma), "dd4993c2-1db9-4e5a-a3f2-c4660ec4e365"},}, new Dictionary<Terminator, ReduceInformation>{ }, "ae2cf852-96b8-453d-adb2-47f0a326e8a0") },
		{ "23e6496a-387e-45e7-9343-1727af61d29a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightSquareBracket), "d41e795a-82db-45e2-bf39-67e8469cb130"}, { new Terminator(DelimiterType.Comma), "ecf84275-ae5c-4a59-96fb-d2ef13a595a8"},}, new Dictionary<Terminator, ReduceInformation>{ }, "23e6496a-387e-45e7-9343-1727af61d29a") },
		{ "8f2d1e59-ae2a-4a98-b7a5-ec5d38c46abd", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(KeywordType.To), "ed8639a3-5285-46a5-8a2d-72a7c3e2432d"},}, new Dictionary<Terminator, ReduceInformation>{ }, "8f2d1e59-ae2a-4a98-b7a5-ec5d38c46abd") },
		{ "cd7ff125-4a37-4908-ae74-cd22185c30cb", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.RelationOperator), "d4c5b403-de15-4e78-88e0-591c0adc9c3a"}, { new Terminator(OperatorType.Equal), "6112dd54-ce34-4dd2-be4c-b06dd725bbd8"}, { new Terminator(OperatorType.NotEqual), "1338afe1-da9e-4380-8a28-7f2d82ef2d68"}, { new Terminator(OperatorType.Less), "fc9b446b-c4b5-4720-bc46-49344508202f"}, { new Terminator(OperatorType.LessEqual), "39ac36f9-efe7-4a1b-ba37-a9729fbf341a"}, { new Terminator(OperatorType.Greater), "fc85ac1b-5a0f-405f-a8f0-a1e20d44351d"}, { new Terminator(OperatorType.GreaterEqual), "f0598e66-3db3-44bd-8b9d-cd6c038df056"}, { new NonTerminator(NonTerminatorType.AddOperator), "8fdee538-6af7-4ee3-9dff-debce1b8d1dd"}, { new Terminator(OperatorType.Plus), "493d6d76-ed65-4308-aee5-65c5fe502546"}, { new Terminator(OperatorType.Minus), "c41b579b-0a72-411a-b9ed-f3ca30f5a615"}, { new Terminator(KeywordType.Or), "22878828-eed7-470f-a07f-595bb2eb7e2a"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Expression))}, }, "cd7ff125-4a37-4908-ae74-cd22185c30cb") },
		{ "1b085cbe-da51-490b-a30b-b52e6f8a3897", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "89fb02c8-5527-435b-a3fa-14b3751e1700"}, { new Terminator(OperatorType.Multiply), "fd20e81d-52b7-4920-adab-e2a1e49a6b6a"}, { new Terminator(OperatorType.Divide), "ef7280a1-85cf-4590-ab87-68bdd867d9e5"}, { new Terminator(KeywordType.Divide), "9192ebb2-aa8a-414d-ac9e-7206883933e6"}, { new Terminator(KeywordType.Mod), "ae32e061-59db-49fd-addd-2d5c3e9014a3"}, { new Terminator(KeywordType.And), "5c4b371b-c686-4545-b641-4c196ca6b25f"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "1b085cbe-da51-490b-a30b-b52e6f8a3897") },
		{ "e8c6655e-9ea9-40f9-99d2-7d55a1cdf5dd", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, }, "e8c6655e-9ea9-40f9-99d2-7d55a1cdf5dd") },
		{ "a3cb4c23-4b02-4dd3-a2f3-1c50b7a1a951", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "a3cb4c23-4b02-4dd3-a2f3-1c50b7a1a951") },
		{ "138c0386-8ea0-455a-a824-c6b28e3537c9", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "138c0386-8ea0-455a-a824-c6b28e3537c9") },
		{ "5b8d9c0d-8dbc-4eb2-bd3f-537c3984b48e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "7755b452-3925-4c65-a989-e15df6f8f663"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "38eec331-d4ae-4b06-a659-98475fdb5aa0"}, { new NonTerminator(NonTerminatorType.Term), "5a5a9a53-cfe3-4e81-8fcf-2e6d11b16b0e"}, { new NonTerminator(NonTerminatorType.Factor), "175e095a-0963-472f-b84a-ece518bd2cf2"}, { Terminator.NumberTerminator, "3861f05c-791c-4f64-b291-d985540c7651"}, { new NonTerminator(NonTerminatorType.Variable), "c693eb95-cdcd-4c7d-8717-3ead45ef8bfc"}, { new Terminator(DelimiterType.LeftParenthesis), "efc9be5f-b54d-4b96-8aec-f3540a407dc8"}, { Terminator.IdentifierTerminator, "a8ce470f-1537-428f-88c2-9270bfed81b6"}, { new Terminator(KeywordType.Not), "addb6420-8367-48a9-9f28-327cf2a88028"}, { new Terminator(OperatorType.Minus), "75101aaf-bd06-4db5-809c-9e8b4533dc5d"},}, new Dictionary<Terminator, ReduceInformation>{ }, "5b8d9c0d-8dbc-4eb2-bd3f-537c3984b48e") },
		{ "8e45f22e-454f-47f0-8ef0-2b3c0fa1878b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.LeftParenthesis), "1d9aa238-31cf-4e2f-a613-73680ee94717"}, { new NonTerminator(NonTerminatorType.IdVarPart), "b126e8cd-fd9c-4fcd-9efb-0d577e651425"}, { new Terminator(DelimiterType.LeftSquareBracket), "49a0716d-d548-4ba7-bdfe-68c7fc3a5830"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Equal), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Less), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Greater), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "8e45f22e-454f-47f0-8ef0-2b3c0fa1878b") },
		{ "14e584b7-b902-49cf-95c1-16d6d0e40ac1", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "f1113cac-5a3d-4575-b4f9-e07883c85a0d"}, { Terminator.NumberTerminator, "a3cb4c23-4b02-4dd3-a2f3-1c50b7a1a951"}, { new NonTerminator(NonTerminatorType.Variable), "138c0386-8ea0-455a-a824-c6b28e3537c9"}, { new Terminator(DelimiterType.LeftParenthesis), "5b8d9c0d-8dbc-4eb2-bd3f-537c3984b48e"}, { Terminator.IdentifierTerminator, "8e45f22e-454f-47f0-8ef0-2b3c0fa1878b"}, { new Terminator(KeywordType.Not), "14e584b7-b902-49cf-95c1-16d6d0e40ac1"}, { new Terminator(OperatorType.Minus), "b723344c-19d8-4ecf-a0aa-eb8cc3782aff"},}, new Dictionary<Terminator, ReduceInformation>{ }, "14e584b7-b902-49cf-95c1-16d6d0e40ac1") },
		{ "b723344c-19d8-4ecf-a0aa-eb8cc3782aff", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "c31fed74-c9a5-4008-a871-1830d016ab4d"}, { Terminator.NumberTerminator, "a3cb4c23-4b02-4dd3-a2f3-1c50b7a1a951"}, { new NonTerminator(NonTerminatorType.Variable), "138c0386-8ea0-455a-a824-c6b28e3537c9"}, { new Terminator(DelimiterType.LeftParenthesis), "5b8d9c0d-8dbc-4eb2-bd3f-537c3984b48e"}, { Terminator.IdentifierTerminator, "8e45f22e-454f-47f0-8ef0-2b3c0fa1878b"}, { new Terminator(KeywordType.Not), "14e584b7-b902-49cf-95c1-16d6d0e40ac1"}, { new Terminator(OperatorType.Minus), "b723344c-19d8-4ecf-a0aa-eb8cc3782aff"},}, new Dictionary<Terminator, ReduceInformation>{ }, "b723344c-19d8-4ecf-a0aa-eb8cc3782aff") },
		{ "63a3d7e2-f9f7-43d7-8941-6fbb0db589b5", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SubprogramBody))}, }, "63a3d7e2-f9f7-43d7-8941-6fbb0db589b5") },
		{ "fe33641c-ef48-4618-a1af-f852cefa2062", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.StatementList), "8dbceb0c-4b60-4cdd-bb30-e3a72e0d61b8"}, { new NonTerminator(NonTerminatorType.Statement), "382f5704-ba25-4f9d-a658-af06ba9c8ecc"}, { new NonTerminator(NonTerminatorType.Variable), "ccb9ddb8-4ca9-4498-906e-81b654135662"}, { Terminator.IdentifierTerminator, "764301e0-1e52-4941-80fc-f768fabb848a"}, { new NonTerminator(NonTerminatorType.ProcedureCall), "d19e5b10-7ef3-4981-8c0b-deca4bfff866"}, { new NonTerminator(NonTerminatorType.CompoundStatement), "67b6f07f-8024-4a8f-8f06-3e29ee8141d8"}, { new Terminator(KeywordType.If), "1139d083-a185-4a24-a86e-2dfa3d0514e2"}, { new Terminator(KeywordType.For), "8337e624-bbce-4b9a-8231-b6b264eefd7f"}, { new Terminator(KeywordType.Begin), "5c23381d-aee0-4773-ab73-20aa2a344931"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, }, "fe33641c-ef48-4618-a1af-f852cefa2062") },
		{ "194ca5b3-e196-43f8-b1b5-fe180e9d3da2", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.Semicolon), "06d47b85-52f7-4599-bdb2-57773137db6f"},}, new Dictionary<Terminator, ReduceInformation>{ }, "194ca5b3-e196-43f8-b1b5-fe180e9d3da2") },
		{ "58fe0694-747e-4b48-8ac1-bd58f454f186", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { Terminator.IdentifierTerminator, "96728733-8b22-4516-b05e-bdff14c767c6"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Begin), new ReduceInformation(3, new NonTerminator(NonTerminatorType.ConstDeclarations))}, { new Terminator(KeywordType.Var), new ReduceInformation(3, new NonTerminator(NonTerminatorType.ConstDeclarations))}, }, "58fe0694-747e-4b48-8ac1-bd58f454f186") },
		{ "5baa3173-72cb-47f6-9101-5a6016508ff6", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.FormalParameter))}, }, "5baa3173-72cb-47f6-9101-5a6016508ff6") },
		{ "1d3a1a40-39ba-4bb3-9015-595ae01650de", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Parameter), "09dc4e35-539c-448a-a588-f8628761e4dd"}, { new NonTerminator(NonTerminatorType.VarParameter), "1c54abd3-ad5a-460b-b84c-508c30c2925b"}, { new NonTerminator(NonTerminatorType.ValueParameter), "7d1b8c5c-7295-4e38-9399-c7e99513b3af"}, { new Terminator(KeywordType.Var), "eb3fbdf4-847d-4499-9851-d06d9fc995b9"}, { new NonTerminator(NonTerminatorType.IdentifierList), "8129be83-68fb-4cb1-b760-6daf04d786a9"}, { Terminator.IdentifierTerminator, "3ed14f62-8e9f-4c1a-8e1e-5d522192093b"},}, new Dictionary<Terminator, ReduceInformation>{ }, "1d3a1a40-39ba-4bb3-9015-595ae01650de") },
		{ "acb91407-eb13-42fa-8719-77d9a4a905d9", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(2, new NonTerminator(NonTerminatorType.VarParameter))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.VarParameter))}, }, "acb91407-eb13-42fa-8719-77d9a4a905d9") },
		{ "1cf3f054-1f43-4c28-9181-45664b9775b4", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.BasicType), "0d6b6db7-375e-459a-a4f5-8d900ae56699"}, { new Terminator(KeywordType.Integer), "7c1ea2ce-ab74-4a53-9781-b5f292c9774d"}, { new Terminator(KeywordType.Real), "319df1bd-d7b8-480e-8ab0-434607cd08e9"}, { new Terminator(KeywordType.Boolean), "c19a1a5c-46f4-4c72-935c-acd21cf32ca7"}, { new Terminator(KeywordType.Character), "0fedfaad-a12e-4064-a450-831c5fb93533"},}, new Dictionary<Terminator, ReduceInformation>{ }, "1cf3f054-1f43-4c28-9181-45664b9775b4") },
		{ "bcbc127a-aa3f-4d7d-9df6-0dd240787f8e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(5, new NonTerminator(NonTerminatorType.SubprogramHead))}, }, "bcbc127a-aa3f-4d7d-9df6-0dd240787f8e") },
		{ "d591c2ea-49de-49d4-8026-f7cbb1ff0004", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Colon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.FormalParameter))}, }, "d591c2ea-49de-49d4-8026-f7cbb1ff0004") },
		{ "3bb5f984-8ec0-4021-b6a4-5fedcd5b64d2", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(KeywordType.Of), "dbebe90b-0d37-43df-a8b8-477db9a7d9dc"},}, new Dictionary<Terminator, ReduceInformation>{ }, "3bb5f984-8ec0-4021-b6a4-5fedcd5b64d2") },
		{ "a0a8f3b7-6141-4883-83c9-fc7585981ef6", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { Terminator.NumberTerminator, "f7a93f8c-d074-4d4d-a5ad-ebae9f2b1a32"},}, new Dictionary<Terminator, ReduceInformation>{ }, "a0a8f3b7-6141-4883-83c9-fc7585981ef6") },
		{ "6ac877b0-965b-4a98-8109-50fadbccdba4", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { Terminator.NumberTerminator, "c6ab978e-3d10-4f6a-a6a7-568eaee5ba44"},}, new Dictionary<Terminator, ReduceInformation>{ }, "6ac877b0-965b-4a98-8109-50fadbccdba4") },
		{ "c7e82e7f-a75b-4f20-90d7-5af1e60eeeae", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.AddOperator), "7770ad47-2dbd-48a1-8fc8-65f5f7d27956"}, { new Terminator(OperatorType.Plus), "493d6d76-ed65-4308-aee5-65c5fe502546"}, { new Terminator(OperatorType.Minus), "c41b579b-0a72-411a-b9ed-f3ca30f5a615"}, { new Terminator(KeywordType.Or), "22878828-eed7-470f-a07f-595bb2eb7e2a"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Expression))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Expression))}, }, "c7e82e7f-a75b-4f20-90d7-5af1e60eeeae") },
		{ "e61b6949-0204-4809-a7a8-c14193efa53c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "2cd1bf85-bbe1-47c2-ac10-a540522bdbce"}, { new Terminator(OperatorType.Multiply), "fd20e81d-52b7-4920-adab-e2a1e49a6b6a"}, { new Terminator(OperatorType.Divide), "ef7280a1-85cf-4590-ab87-68bdd867d9e5"}, { new Terminator(KeywordType.Divide), "9192ebb2-aa8a-414d-ac9e-7206883933e6"}, { new Terminator(KeywordType.Mod), "ae32e061-59db-49fd-addd-2d5c3e9014a3"}, { new Terminator(KeywordType.And), "5c4b371b-c686-4545-b641-4c196ca6b25f"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "e61b6949-0204-4809-a7a8-c14193efa53c") },
		{ "19cfe0eb-0189-4bfa-83ef-442a36cd1138", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, }, "19cfe0eb-0189-4bfa-83ef-442a36cd1138") },
		{ "ff4e43b4-9264-41ed-8fd4-fe02b4673621", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "ff4e43b4-9264-41ed-8fd4-fe02b4673621") },
		{ "70dc3750-b84d-4199-808f-50c3ab8858e0", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "70dc3750-b84d-4199-808f-50c3ab8858e0") },
		{ "be22425a-a8f1-4ac2-b180-9c579b063d86", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "1d626d30-a2b4-4eaf-8272-f008a108d644"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "38eec331-d4ae-4b06-a659-98475fdb5aa0"}, { new NonTerminator(NonTerminatorType.Term), "5a5a9a53-cfe3-4e81-8fcf-2e6d11b16b0e"}, { new NonTerminator(NonTerminatorType.Factor), "175e095a-0963-472f-b84a-ece518bd2cf2"}, { Terminator.NumberTerminator, "3861f05c-791c-4f64-b291-d985540c7651"}, { new NonTerminator(NonTerminatorType.Variable), "c693eb95-cdcd-4c7d-8717-3ead45ef8bfc"}, { new Terminator(DelimiterType.LeftParenthesis), "efc9be5f-b54d-4b96-8aec-f3540a407dc8"}, { Terminator.IdentifierTerminator, "a8ce470f-1537-428f-88c2-9270bfed81b6"}, { new Terminator(KeywordType.Not), "addb6420-8367-48a9-9f28-327cf2a88028"}, { new Terminator(OperatorType.Minus), "75101aaf-bd06-4db5-809c-9e8b4533dc5d"},}, new Dictionary<Terminator, ReduceInformation>{ }, "be22425a-a8f1-4ac2-b180-9c579b063d86") },
		{ "75199901-29ad-4fde-8e16-2a9ce7439b9d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.LeftParenthesis), "df220402-c4e0-4ed1-b602-25a79ac551f4"}, { new NonTerminator(NonTerminatorType.IdVarPart), "c387a2bb-8d76-420f-b6ce-1a2ba965b2ed"}, { new Terminator(DelimiterType.LeftSquareBracket), "fe415909-e195-4f24-b56a-40b7fad976fa"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "75199901-29ad-4fde-8e16-2a9ce7439b9d") },
		{ "ffcfe34e-f488-4c19-b03c-f14a0c709964", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "494e772e-87f2-41ec-a1ea-bde8b1aa79e3"}, { Terminator.NumberTerminator, "ff4e43b4-9264-41ed-8fd4-fe02b4673621"}, { new NonTerminator(NonTerminatorType.Variable), "70dc3750-b84d-4199-808f-50c3ab8858e0"}, { new Terminator(DelimiterType.LeftParenthesis), "be22425a-a8f1-4ac2-b180-9c579b063d86"}, { Terminator.IdentifierTerminator, "75199901-29ad-4fde-8e16-2a9ce7439b9d"}, { new Terminator(KeywordType.Not), "ffcfe34e-f488-4c19-b03c-f14a0c709964"}, { new Terminator(OperatorType.Minus), "be7ba5b3-c18b-4687-82d8-69e621837916"},}, new Dictionary<Terminator, ReduceInformation>{ }, "ffcfe34e-f488-4c19-b03c-f14a0c709964") },
		{ "be7ba5b3-c18b-4687-82d8-69e621837916", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "69490ee7-ab21-4ae8-929e-b13c51bf25d3"}, { Terminator.NumberTerminator, "ff4e43b4-9264-41ed-8fd4-fe02b4673621"}, { new NonTerminator(NonTerminatorType.Variable), "70dc3750-b84d-4199-808f-50c3ab8858e0"}, { new Terminator(DelimiterType.LeftParenthesis), "be22425a-a8f1-4ac2-b180-9c579b063d86"}, { Terminator.IdentifierTerminator, "75199901-29ad-4fde-8e16-2a9ce7439b9d"}, { new Terminator(KeywordType.Not), "ffcfe34e-f488-4c19-b03c-f14a0c709964"}, { new Terminator(OperatorType.Minus), "be7ba5b3-c18b-4687-82d8-69e621837916"},}, new Dictionary<Terminator, ReduceInformation>{ }, "be7ba5b3-c18b-4687-82d8-69e621837916") },
		{ "a5d072d8-4dbe-401f-bffd-52987b4304f9", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "7d5842f8-4715-4558-b43b-bdcfd821ece2"}, { new Terminator(OperatorType.Multiply), "fd20e81d-52b7-4920-adab-e2a1e49a6b6a"}, { new Terminator(OperatorType.Divide), "ef7280a1-85cf-4590-ab87-68bdd867d9e5"}, { new Terminator(KeywordType.Divide), "9192ebb2-aa8a-414d-ac9e-7206883933e6"}, { new Terminator(KeywordType.Mod), "ae32e061-59db-49fd-addd-2d5c3e9014a3"}, { new Terminator(KeywordType.And), "5c4b371b-c686-4545-b641-4c196ca6b25f"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "a5d072d8-4dbe-401f-bffd-52987b4304f9") },
		{ "5903b318-099a-4afc-b2d0-c33cbd915a07", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, }, "5903b318-099a-4afc-b2d0-c33cbd915a07") },
		{ "4cc3390c-dc0a-4bdd-bffb-90e9c1e979e5", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "4cc3390c-dc0a-4bdd-bffb-90e9c1e979e5") },
		{ "a0b14868-8772-40ad-a8bb-28cd4a33e54b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "7fbabe78-2047-47c6-bf30-503e7d86fd9b"}, { new Terminator(DelimiterType.Comma), "dd4993c2-1db9-4e5a-a3f2-c4660ec4e365"},}, new Dictionary<Terminator, ReduceInformation>{ }, "a0b14868-8772-40ad-a8bb-28cd4a33e54b") },
		{ "2339d6bf-8732-4a03-aa14-4f0f2c574920", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightSquareBracket), "1aeba5d6-3fe4-449d-9e7a-8b55f2e37f41"}, { new Terminator(DelimiterType.Comma), "ecf84275-ae5c-4a59-96fb-d2ef13a595a8"},}, new Dictionary<Terminator, ReduceInformation>{ }, "2339d6bf-8732-4a03-aa14-4f0f2c574920") },
		{ "fd16c929-4cb4-4445-a854-68564b63eb12", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(3, new NonTerminator(NonTerminatorType.ExpressionList))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.ExpressionList))}, }, "fd16c929-4cb4-4445-a854-68564b63eb12") },
		{ "6cd3b153-5fda-4872-9c55-04ff10ff0ecc", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.AddOperator), "34927101-5f58-4602-8f84-22bf65fa02a3"}, { new Terminator(OperatorType.Plus), "493d6d76-ed65-4308-aee5-65c5fe502546"}, { new Terminator(OperatorType.Minus), "c41b579b-0a72-411a-b9ed-f3ca30f5a615"}, { new Terminator(KeywordType.Or), "22878828-eed7-470f-a07f-595bb2eb7e2a"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Expression))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Expression))}, }, "6cd3b153-5fda-4872-9c55-04ff10ff0ecc") },
		{ "1ecca4ed-8722-4b0d-afb1-365c050b9b25", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "1fcec038-a6e7-4e28-833e-6bb9424f7ca9"}, { new Terminator(OperatorType.Multiply), "fd20e81d-52b7-4920-adab-e2a1e49a6b6a"}, { new Terminator(OperatorType.Divide), "ef7280a1-85cf-4590-ab87-68bdd867d9e5"}, { new Terminator(KeywordType.Divide), "9192ebb2-aa8a-414d-ac9e-7206883933e6"}, { new Terminator(KeywordType.Mod), "ae32e061-59db-49fd-addd-2d5c3e9014a3"}, { new Terminator(KeywordType.And), "5c4b371b-c686-4545-b641-4c196ca6b25f"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "1ecca4ed-8722-4b0d-afb1-365c050b9b25") },
		{ "13f24f9d-eafa-49d3-981d-e9708deefa70", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, }, "13f24f9d-eafa-49d3-981d-e9708deefa70") },
		{ "c36a5a82-356a-4f27-9507-4853647330f8", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "c36a5a82-356a-4f27-9507-4853647330f8") },
		{ "25d3aea4-fc9e-41ac-a0ba-db8ef11cee57", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "25d3aea4-fc9e-41ac-a0ba-db8ef11cee57") },
		{ "801568bc-0c2a-423f-9fd4-7f216d41b5c8", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "0d8e7b5f-ce90-4222-900e-26a12528e9d6"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "38eec331-d4ae-4b06-a659-98475fdb5aa0"}, { new NonTerminator(NonTerminatorType.Term), "5a5a9a53-cfe3-4e81-8fcf-2e6d11b16b0e"}, { new NonTerminator(NonTerminatorType.Factor), "175e095a-0963-472f-b84a-ece518bd2cf2"}, { Terminator.NumberTerminator, "3861f05c-791c-4f64-b291-d985540c7651"}, { new NonTerminator(NonTerminatorType.Variable), "c693eb95-cdcd-4c7d-8717-3ead45ef8bfc"}, { new Terminator(DelimiterType.LeftParenthesis), "efc9be5f-b54d-4b96-8aec-f3540a407dc8"}, { Terminator.IdentifierTerminator, "a8ce470f-1537-428f-88c2-9270bfed81b6"}, { new Terminator(KeywordType.Not), "addb6420-8367-48a9-9f28-327cf2a88028"}, { new Terminator(OperatorType.Minus), "75101aaf-bd06-4db5-809c-9e8b4533dc5d"},}, new Dictionary<Terminator, ReduceInformation>{ }, "801568bc-0c2a-423f-9fd4-7f216d41b5c8") },
		{ "62d908d5-bc3f-41cd-9d71-fff26a1343c1", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.LeftParenthesis), "2535e225-0b06-4a60-93e1-85292149e8ad"}, { new NonTerminator(NonTerminatorType.IdVarPart), "fb68fd49-5145-4ef1-acf5-8ce1675a1b8c"}, { new Terminator(DelimiterType.LeftSquareBracket), "53f9b5b9-3450-4e3b-b74c-a5e432c28c25"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "62d908d5-bc3f-41cd-9d71-fff26a1343c1") },
		{ "2269c88f-87d2-4c54-9270-380d70d9ec35", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "c26cd85c-b279-40bf-b747-cdcb11b5a046"}, { Terminator.NumberTerminator, "c36a5a82-356a-4f27-9507-4853647330f8"}, { new NonTerminator(NonTerminatorType.Variable), "25d3aea4-fc9e-41ac-a0ba-db8ef11cee57"}, { new Terminator(DelimiterType.LeftParenthesis), "801568bc-0c2a-423f-9fd4-7f216d41b5c8"}, { Terminator.IdentifierTerminator, "62d908d5-bc3f-41cd-9d71-fff26a1343c1"}, { new Terminator(KeywordType.Not), "2269c88f-87d2-4c54-9270-380d70d9ec35"}, { new Terminator(OperatorType.Minus), "3c697023-0a28-43a8-be5b-ef957bbf16a6"},}, new Dictionary<Terminator, ReduceInformation>{ }, "2269c88f-87d2-4c54-9270-380d70d9ec35") },
		{ "3c697023-0a28-43a8-be5b-ef957bbf16a6", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "931e83e9-1736-4ae4-885d-edbfb0a4277f"}, { Terminator.NumberTerminator, "c36a5a82-356a-4f27-9507-4853647330f8"}, { new NonTerminator(NonTerminatorType.Variable), "25d3aea4-fc9e-41ac-a0ba-db8ef11cee57"}, { new Terminator(DelimiterType.LeftParenthesis), "801568bc-0c2a-423f-9fd4-7f216d41b5c8"}, { Terminator.IdentifierTerminator, "62d908d5-bc3f-41cd-9d71-fff26a1343c1"}, { new Terminator(KeywordType.Not), "2269c88f-87d2-4c54-9270-380d70d9ec35"}, { new Terminator(OperatorType.Minus), "3c697023-0a28-43a8-be5b-ef957bbf16a6"},}, new Dictionary<Terminator, ReduceInformation>{ }, "3c697023-0a28-43a8-be5b-ef957bbf16a6") },
		{ "4f68cbaf-b771-4f09-83ab-8d7283eb0c3c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "05f80b0f-765b-49de-8b00-897364786373"}, { new Terminator(OperatorType.Multiply), "fd20e81d-52b7-4920-adab-e2a1e49a6b6a"}, { new Terminator(OperatorType.Divide), "ef7280a1-85cf-4590-ab87-68bdd867d9e5"}, { new Terminator(KeywordType.Divide), "9192ebb2-aa8a-414d-ac9e-7206883933e6"}, { new Terminator(KeywordType.Mod), "ae32e061-59db-49fd-addd-2d5c3e9014a3"}, { new Terminator(KeywordType.And), "5c4b371b-c686-4545-b641-4c196ca6b25f"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "4f68cbaf-b771-4f09-83ab-8d7283eb0c3c") },
		{ "787200ba-571b-433b-9e83-c4ed871f52c5", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, }, "787200ba-571b-433b-9e83-c4ed871f52c5") },
		{ "0c210077-0859-488b-9991-5fcac82cce1a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "0c210077-0859-488b-9991-5fcac82cce1a") },
		{ "06846c83-218a-4305-b749-00df4561ab21", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "6d1d3243-8773-4a86-ba5d-0157c8af19b9"}, { new Terminator(DelimiterType.Comma), "dd4993c2-1db9-4e5a-a3f2-c4660ec4e365"},}, new Dictionary<Terminator, ReduceInformation>{ }, "06846c83-218a-4305-b749-00df4561ab21") },
		{ "78bebc29-c88c-4d86-bd44-d5f34b99570b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightSquareBracket), "5d6ffae2-17f9-4a7a-b8fa-4a2f394dfab2"}, { new Terminator(DelimiterType.Comma), "ecf84275-ae5c-4a59-96fb-d2ef13a595a8"},}, new Dictionary<Terminator, ReduceInformation>{ }, "78bebc29-c88c-4d86-bd44-d5f34b99570b") },
		{ "43891951-79a3-429b-b599-3d57bf0975f7", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.ExpressionList))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.ExpressionList))}, }, "43891951-79a3-429b-b599-3d57bf0975f7") },
		{ "069f86f9-ca28-44c5-8f86-a470b4d1441c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.AddOperator), "31793d6b-e65f-48d3-b3b3-42f95c8eaa5f"}, { new Terminator(OperatorType.Plus), "493d6d76-ed65-4308-aee5-65c5fe502546"}, { new Terminator(OperatorType.Minus), "c41b579b-0a72-411a-b9ed-f3ca30f5a615"}, { new Terminator(KeywordType.Or), "22878828-eed7-470f-a07f-595bb2eb7e2a"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Expression))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Expression))}, }, "069f86f9-ca28-44c5-8f86-a470b4d1441c") },
		{ "687ca7fc-755d-4d28-826e-19f6f2e5359f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "5cf7b429-f628-4955-90cd-8b9540a53fe4"}, { new Terminator(OperatorType.Multiply), "fd20e81d-52b7-4920-adab-e2a1e49a6b6a"}, { new Terminator(OperatorType.Divide), "ef7280a1-85cf-4590-ab87-68bdd867d9e5"}, { new Terminator(KeywordType.Divide), "9192ebb2-aa8a-414d-ac9e-7206883933e6"}, { new Terminator(KeywordType.Mod), "ae32e061-59db-49fd-addd-2d5c3e9014a3"}, { new Terminator(KeywordType.And), "5c4b371b-c686-4545-b641-4c196ca6b25f"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "687ca7fc-755d-4d28-826e-19f6f2e5359f") },
		{ "a6ee60f9-e7a6-4693-a4e8-cd41934b1727", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, }, "a6ee60f9-e7a6-4693-a4e8-cd41934b1727") },
		{ "a7df07cd-c0da-4e90-8871-a13944fae98c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "a7df07cd-c0da-4e90-8871-a13944fae98c") },
		{ "265ce997-0847-47ff-84ad-380765fb1439", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "265ce997-0847-47ff-84ad-380765fb1439") },
		{ "1e10eeed-1b88-45a5-a914-bbcf404cda86", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "024c3624-c4cf-4acd-a3e3-890352daf1e9"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "38eec331-d4ae-4b06-a659-98475fdb5aa0"}, { new NonTerminator(NonTerminatorType.Term), "5a5a9a53-cfe3-4e81-8fcf-2e6d11b16b0e"}, { new NonTerminator(NonTerminatorType.Factor), "175e095a-0963-472f-b84a-ece518bd2cf2"}, { Terminator.NumberTerminator, "3861f05c-791c-4f64-b291-d985540c7651"}, { new NonTerminator(NonTerminatorType.Variable), "c693eb95-cdcd-4c7d-8717-3ead45ef8bfc"}, { new Terminator(DelimiterType.LeftParenthesis), "efc9be5f-b54d-4b96-8aec-f3540a407dc8"}, { Terminator.IdentifierTerminator, "a8ce470f-1537-428f-88c2-9270bfed81b6"}, { new Terminator(KeywordType.Not), "addb6420-8367-48a9-9f28-327cf2a88028"}, { new Terminator(OperatorType.Minus), "75101aaf-bd06-4db5-809c-9e8b4533dc5d"},}, new Dictionary<Terminator, ReduceInformation>{ }, "1e10eeed-1b88-45a5-a914-bbcf404cda86") },
		{ "55db5714-9404-40ae-a5cc-d7cd7968f2be", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.LeftParenthesis), "e141d1ca-269a-4f77-a953-63106ee7ac34"}, { new NonTerminator(NonTerminatorType.IdVarPart), "c1e2fed5-d017-4fd1-8c5e-d077a3b6d161"}, { new Terminator(DelimiterType.LeftSquareBracket), "34299268-f9b2-435f-b210-7395b25c72b8"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "55db5714-9404-40ae-a5cc-d7cd7968f2be") },
		{ "cba1e33d-813e-4f0e-9bda-ba7f1020c426", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "e9db6c03-fe83-436f-beef-6379906c15e0"}, { Terminator.NumberTerminator, "a7df07cd-c0da-4e90-8871-a13944fae98c"}, { new NonTerminator(NonTerminatorType.Variable), "265ce997-0847-47ff-84ad-380765fb1439"}, { new Terminator(DelimiterType.LeftParenthesis), "1e10eeed-1b88-45a5-a914-bbcf404cda86"}, { Terminator.IdentifierTerminator, "55db5714-9404-40ae-a5cc-d7cd7968f2be"}, { new Terminator(KeywordType.Not), "cba1e33d-813e-4f0e-9bda-ba7f1020c426"}, { new Terminator(OperatorType.Minus), "4689d95f-7bbb-41de-9f96-ba34da344607"},}, new Dictionary<Terminator, ReduceInformation>{ }, "cba1e33d-813e-4f0e-9bda-ba7f1020c426") },
		{ "4689d95f-7bbb-41de-9f96-ba34da344607", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "88b33a61-8b80-4e73-b675-c9027cbafac2"}, { Terminator.NumberTerminator, "a7df07cd-c0da-4e90-8871-a13944fae98c"}, { new NonTerminator(NonTerminatorType.Variable), "265ce997-0847-47ff-84ad-380765fb1439"}, { new Terminator(DelimiterType.LeftParenthesis), "1e10eeed-1b88-45a5-a914-bbcf404cda86"}, { Terminator.IdentifierTerminator, "55db5714-9404-40ae-a5cc-d7cd7968f2be"}, { new Terminator(KeywordType.Not), "cba1e33d-813e-4f0e-9bda-ba7f1020c426"}, { new Terminator(OperatorType.Minus), "4689d95f-7bbb-41de-9f96-ba34da344607"},}, new Dictionary<Terminator, ReduceInformation>{ }, "4689d95f-7bbb-41de-9f96-ba34da344607") },
		{ "ef1bad8e-e7b6-4bc6-b0fb-242cff06ea74", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "7964d212-7d90-4920-b7c1-abae26705284"}, { new Terminator(OperatorType.Multiply), "fd20e81d-52b7-4920-adab-e2a1e49a6b6a"}, { new Terminator(OperatorType.Divide), "ef7280a1-85cf-4590-ab87-68bdd867d9e5"}, { new Terminator(KeywordType.Divide), "9192ebb2-aa8a-414d-ac9e-7206883933e6"}, { new Terminator(KeywordType.Mod), "ae32e061-59db-49fd-addd-2d5c3e9014a3"}, { new Terminator(KeywordType.And), "5c4b371b-c686-4545-b641-4c196ca6b25f"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "ef1bad8e-e7b6-4bc6-b0fb-242cff06ea74") },
		{ "2585b2b3-825d-4918-a65f-6bf0e2b91a04", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, }, "2585b2b3-825d-4918-a65f-6bf0e2b91a04") },
		{ "7e010031-8e0f-4159-a5ca-a4d6a43f32c4", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "7e010031-8e0f-4159-a5ca-a4d6a43f32c4") },
		{ "872686a0-a439-401f-aef4-fd55a5e4de9c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "2b0118c3-48cd-489e-a7a6-623a05d1c646"}, { new Terminator(DelimiterType.Comma), "dd4993c2-1db9-4e5a-a3f2-c4660ec4e365"},}, new Dictionary<Terminator, ReduceInformation>{ }, "872686a0-a439-401f-aef4-fd55a5e4de9c") },
		{ "a20d51ab-d41b-4d08-9a84-225578db0f89", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightSquareBracket), "b925d90f-939b-4806-bbc7-e9aca4c3d1ab"}, { new Terminator(DelimiterType.Comma), "ecf84275-ae5c-4a59-96fb-d2ef13a595a8"},}, new Dictionary<Terminator, ReduceInformation>{ }, "a20d51ab-d41b-4d08-9a84-225578db0f89") },
		{ "c31614ec-b50c-49b7-ad94-d6dd716d918a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(5, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(5, new NonTerminator(NonTerminatorType.Statement))}, }, "c31614ec-b50c-49b7-ad94-d6dd716d918a") },
		{ "8ca0d603-8db8-472b-9929-111f27f6f615", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Statement), "d6366716-ff5a-4a53-89fa-d83fdf929ec9"}, { new NonTerminator(NonTerminatorType.Variable), "ccb9ddb8-4ca9-4498-906e-81b654135662"}, { Terminator.IdentifierTerminator, "764301e0-1e52-4941-80fc-f768fabb848a"}, { new NonTerminator(NonTerminatorType.ProcedureCall), "d19e5b10-7ef3-4981-8c0b-deca4bfff866"}, { new NonTerminator(NonTerminatorType.CompoundStatement), "67b6f07f-8024-4a8f-8f06-3e29ee8141d8"}, { new Terminator(KeywordType.If), "1139d083-a185-4a24-a86e-2dfa3d0514e2"}, { new Terminator(KeywordType.For), "8337e624-bbce-4b9a-8231-b6b264eefd7f"}, { new Terminator(KeywordType.Begin), "5c23381d-aee0-4773-ab73-20aa2a344931"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, }, "8ca0d603-8db8-472b-9929-111f27f6f615") },
		{ "2cabd519-a4b4-40c1-b999-f35a12a71c5a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "50923814-6ff6-49a1-8a51-b0f1d6d43539"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "d4f48672-6fe9-4839-a022-03ad51a13b21"}, { new NonTerminator(NonTerminatorType.Term), "cffb0ebf-158b-40bc-a554-a29574d03375"}, { new NonTerminator(NonTerminatorType.Factor), "a697a4fa-5d3d-45c4-a1da-f2783855186e"}, { Terminator.NumberTerminator, "d0368978-0f30-41f5-a0cc-b837ed474739"}, { new NonTerminator(NonTerminatorType.Variable), "72921583-cb29-44a3-a883-d86b10c630e4"}, { new Terminator(DelimiterType.LeftParenthesis), "df70bbe0-47c7-4421-80d2-ad8e2c4dac96"}, { Terminator.IdentifierTerminator, "04758ff4-52e2-493b-a02d-951bd58ede9b"}, { new Terminator(KeywordType.Not), "a50cf991-5964-4e91-a4d2-bdfaebd4d461"}, { new Terminator(OperatorType.Minus), "c31ada0c-d881-4599-a8a5-a079e69b52dd"},}, new Dictionary<Terminator, ReduceInformation>{ }, "2cabd519-a4b4-40c1-b999-f35a12a71c5a") },
		{ "8c9a5a6d-24b1-4ebc-9159-406925920da8", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "998f3c7b-1c09-443d-9b98-406c382b7329"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "d4f48672-6fe9-4839-a022-03ad51a13b21"}, { new NonTerminator(NonTerminatorType.Term), "cffb0ebf-158b-40bc-a554-a29574d03375"}, { new NonTerminator(NonTerminatorType.Factor), "a697a4fa-5d3d-45c4-a1da-f2783855186e"}, { Terminator.NumberTerminator, "d0368978-0f30-41f5-a0cc-b837ed474739"}, { new NonTerminator(NonTerminatorType.Variable), "72921583-cb29-44a3-a883-d86b10c630e4"}, { new Terminator(DelimiterType.LeftParenthesis), "df70bbe0-47c7-4421-80d2-ad8e2c4dac96"}, { Terminator.IdentifierTerminator, "04758ff4-52e2-493b-a02d-951bd58ede9b"}, { new Terminator(KeywordType.Not), "a50cf991-5964-4e91-a4d2-bdfaebd4d461"}, { new Terminator(OperatorType.Minus), "c31ada0c-d881-4599-a8a5-a079e69b52dd"},}, new Dictionary<Terminator, ReduceInformation>{ }, "8c9a5a6d-24b1-4ebc-9159-406925920da8") },
		{ "ea8355f8-a135-4d5d-8221-9125bef12329", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "43b8d4ad-d76d-4791-8fb1-9393ffb6d7dc"}, { new NonTerminator(NonTerminatorType.Expression), "a582ab7f-9aa1-4130-9df3-2ba9dd0fe6f3"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "c3098ddd-d7d7-4d8b-b9ab-1ebedac5467f"}, { new NonTerminator(NonTerminatorType.Term), "c83de3f5-4130-4b2e-8715-e72b2e35275e"}, { new NonTerminator(NonTerminatorType.Factor), "9c184da0-a0b4-4ae3-9b1e-93b0fe09fbf1"}, { Terminator.NumberTerminator, "eaf4d875-7318-4f6e-b25b-829d30a888b8"}, { new NonTerminator(NonTerminatorType.Variable), "eb827526-1aa2-47ab-8b01-b1b0792e4297"}, { new Terminator(DelimiterType.LeftParenthesis), "2894d0f9-3faf-4be5-b2b6-c499611fc58d"}, { Terminator.IdentifierTerminator, "8efce527-7ba0-435f-a58f-b048a7a0e505"}, { new Terminator(KeywordType.Not), "dd22d779-6929-400f-832a-6d4e7eda69e6"}, { new Terminator(OperatorType.Minus), "28fb603c-e1e9-4bfa-ab9b-f8887e1996c0"},}, new Dictionary<Terminator, ReduceInformation>{ }, "ea8355f8-a135-4d5d-8221-9125bef12329") },
		{ "d80554bd-0e20-4fb6-a6e8-35ed8a788a02", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(KeywordType.Then), "481574c9-ea10-4832-afb6-16ef86a76e6e"},}, new Dictionary<Terminator, ReduceInformation>{ }, "d80554bd-0e20-4fb6-a6e8-35ed8a788a02") },
		{ "4732a6c9-d6bb-442c-864d-8980e73ff547", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(OperatorType.Assign), "0a1fdbf2-0439-42f4-9035-5dd6f25dd036"},}, new Dictionary<Terminator, ReduceInformation>{ }, "4732a6c9-d6bb-442c-864d-8980e73ff547") },
		{ "cc957d7d-92b3-430f-b18b-5a2d4b5b5b14", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(KeywordType.End), "7c27c7c7-ae11-4790-89df-1ce0959cb860"}, { new Terminator(DelimiterType.Semicolon), "f83f97d0-e0c9-462f-b6fa-0bc1226d63fe"},}, new Dictionary<Terminator, ReduceInformation>{ }, "cc957d7d-92b3-430f-b18b-5a2d4b5b5b14") },
		{ "573debc2-b615-48ab-bfa9-ac1489967f6d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Term), "3f10d9a1-f758-442e-be9d-821751ba8f4d"}, { new NonTerminator(NonTerminatorType.Factor), "44bb095c-dbed-40a4-a4e5-ca65d38ca2ac"}, { Terminator.NumberTerminator, "6abd39fc-1d52-41a2-ae4b-43a6b52f2537"}, { new NonTerminator(NonTerminatorType.Variable), "7b2c56ec-cb0f-41f6-a54b-d73c18298449"}, { new Terminator(DelimiterType.LeftParenthesis), "3e5d0461-383e-4afd-8bd2-4c3ac4afbf0b"}, { Terminator.IdentifierTerminator, "068b589e-cf17-40e4-baae-3f3b18ddb76e"}, { new Terminator(KeywordType.Not), "f4285e75-83ea-4c5d-8403-e01bf0d9bbfb"}, { new Terminator(OperatorType.Minus), "a29a7e8a-8e5a-458a-8b41-cf17752ff1b1"},}, new Dictionary<Terminator, ReduceInformation>{ }, "573debc2-b615-48ab-bfa9-ac1489967f6d") },
		{ "ce757e88-c309-4c8f-b512-fde449379b78", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "a40d046b-a706-4a39-84f2-3a28a038e558"}, { Terminator.NumberTerminator, "6abd39fc-1d52-41a2-ae4b-43a6b52f2537"}, { new NonTerminator(NonTerminatorType.Variable), "7b2c56ec-cb0f-41f6-a54b-d73c18298449"}, { new Terminator(DelimiterType.LeftParenthesis), "3e5d0461-383e-4afd-8bd2-4c3ac4afbf0b"}, { Terminator.IdentifierTerminator, "068b589e-cf17-40e4-baae-3f3b18ddb76e"}, { new Terminator(KeywordType.Not), "f4285e75-83ea-4c5d-8403-e01bf0d9bbfb"}, { new Terminator(OperatorType.Minus), "a29a7e8a-8e5a-458a-8b41-cf17752ff1b1"},}, new Dictionary<Terminator, ReduceInformation>{ }, "ce757e88-c309-4c8f-b512-fde449379b78") },
		{ "f11a3033-dc94-4fc8-bbae-61d3562ccd16", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "759bbbce-3ad6-4cde-912d-3eb3d0efa050"},}, new Dictionary<Terminator, ReduceInformation>{ }, "f11a3033-dc94-4fc8-bbae-61d3562ccd16") },
		{ "fd15c449-ac45-4bad-b4bb-a0f2f4ac7626", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "52b527f4-8de6-4d44-924a-4a1b7847187a"}, { new NonTerminator(NonTerminatorType.Expression), "a582ab7f-9aa1-4130-9df3-2ba9dd0fe6f3"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "c3098ddd-d7d7-4d8b-b9ab-1ebedac5467f"}, { new NonTerminator(NonTerminatorType.Term), "c83de3f5-4130-4b2e-8715-e72b2e35275e"}, { new NonTerminator(NonTerminatorType.Factor), "9c184da0-a0b4-4ae3-9b1e-93b0fe09fbf1"}, { Terminator.NumberTerminator, "eaf4d875-7318-4f6e-b25b-829d30a888b8"}, { new NonTerminator(NonTerminatorType.Variable), "eb827526-1aa2-47ab-8b01-b1b0792e4297"}, { new Terminator(DelimiterType.LeftParenthesis), "2894d0f9-3faf-4be5-b2b6-c499611fc58d"}, { Terminator.IdentifierTerminator, "8efce527-7ba0-435f-a58f-b048a7a0e505"}, { new Terminator(KeywordType.Not), "dd22d779-6929-400f-832a-6d4e7eda69e6"}, { new Terminator(OperatorType.Minus), "28fb603c-e1e9-4bfa-ab9b-f8887e1996c0"},}, new Dictionary<Terminator, ReduceInformation>{ }, "fd15c449-ac45-4bad-b4bb-a0f2f4ac7626") },
		{ "91afac8d-1047-4a2e-aed9-b3273bc6894f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, }, "91afac8d-1047-4a2e-aed9-b3273bc6894f") },
		{ "988ddf5d-71fe-4f8f-b49f-a24ad209479f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "9ebd33e8-8901-4932-873d-e225a974d882"}, { new NonTerminator(NonTerminatorType.Expression), "10204ac0-c523-4b6c-890d-d508a8e83f48"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "72d6e812-1720-408a-ac1c-5ac20fa267d1"}, { new NonTerminator(NonTerminatorType.Term), "0b3b44d2-8458-48d1-b7c4-2329d479b56c"}, { new NonTerminator(NonTerminatorType.Factor), "2f595e0e-c29e-494d-a5bb-a570db3254fd"}, { Terminator.NumberTerminator, "c86f637c-4e17-4d3d-bfd7-f1fc5abdf7f4"}, { new NonTerminator(NonTerminatorType.Variable), "fa555a17-89d0-4cf7-989a-0b0775ade6d4"}, { new Terminator(DelimiterType.LeftParenthesis), "8fab798a-03a8-45b7-8380-7b7eaee651fa"}, { Terminator.IdentifierTerminator, "513b9c84-fc68-4334-976d-7547b1ec521f"}, { new Terminator(KeywordType.Not), "928a980c-85a8-4cf5-a5d1-117516baef4e"}, { new Terminator(OperatorType.Minus), "e2c085de-e99b-4f3a-b869-c3ca5f5dcf75"},}, new Dictionary<Terminator, ReduceInformation>{ }, "988ddf5d-71fe-4f8f-b49f-a24ad209479f") },
		{ "0bbee844-15b8-463a-b72c-d1b393bba256", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "0bbee844-15b8-463a-b72c-d1b393bba256") },
		{ "a103d08c-8e5d-4cff-a781-718fd3d6e0fd", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "a103d08c-8e5d-4cff-a781-718fd3d6e0fd") },
		{ "e0733e10-922b-4841-9464-447f3c5afb17", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.AddOperator), "d6624c6b-a187-4803-84fe-c3a8f6e434a2"}, { new Terminator(OperatorType.Plus), "493d6d76-ed65-4308-aee5-65c5fe502546"}, { new Terminator(OperatorType.Minus), "c41b579b-0a72-411a-b9ed-f3ca30f5a615"}, { new Terminator(KeywordType.Or), "22878828-eed7-470f-a07f-595bb2eb7e2a"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Expression))}, }, "e0733e10-922b-4841-9464-447f3c5afb17") },
		{ "6092fbdf-04e0-4ad1-88e7-787aac644669", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "19067c18-27f1-4ad5-90c4-194d67b7315e"}, { new Terminator(OperatorType.Multiply), "fd20e81d-52b7-4920-adab-e2a1e49a6b6a"}, { new Terminator(OperatorType.Divide), "ef7280a1-85cf-4590-ab87-68bdd867d9e5"}, { new Terminator(KeywordType.Divide), "9192ebb2-aa8a-414d-ac9e-7206883933e6"}, { new Terminator(KeywordType.Mod), "ae32e061-59db-49fd-addd-2d5c3e9014a3"}, { new Terminator(KeywordType.And), "5c4b371b-c686-4545-b641-4c196ca6b25f"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "6092fbdf-04e0-4ad1-88e7-787aac644669") },
		{ "42bb7db9-5be9-4ac8-89ed-01578a50d2fc", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, }, "42bb7db9-5be9-4ac8-89ed-01578a50d2fc") },
		{ "2a5a634a-a406-4e54-a148-6378d751f2c4", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "2a5a634a-a406-4e54-a148-6378d751f2c4") },
		{ "bfc1ac1f-9a68-43c1-a75d-612f181a96d2", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "bfc1ac1f-9a68-43c1-a75d-612f181a96d2") },
		{ "f6d9e32c-49ee-436c-ba97-fcdd527d9fdb", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "dfaee9e5-df35-4f62-b3c7-f89c72915cbf"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "38eec331-d4ae-4b06-a659-98475fdb5aa0"}, { new NonTerminator(NonTerminatorType.Term), "5a5a9a53-cfe3-4e81-8fcf-2e6d11b16b0e"}, { new NonTerminator(NonTerminatorType.Factor), "175e095a-0963-472f-b84a-ece518bd2cf2"}, { Terminator.NumberTerminator, "3861f05c-791c-4f64-b291-d985540c7651"}, { new NonTerminator(NonTerminatorType.Variable), "c693eb95-cdcd-4c7d-8717-3ead45ef8bfc"}, { new Terminator(DelimiterType.LeftParenthesis), "efc9be5f-b54d-4b96-8aec-f3540a407dc8"}, { Terminator.IdentifierTerminator, "a8ce470f-1537-428f-88c2-9270bfed81b6"}, { new Terminator(KeywordType.Not), "addb6420-8367-48a9-9f28-327cf2a88028"}, { new Terminator(OperatorType.Minus), "75101aaf-bd06-4db5-809c-9e8b4533dc5d"},}, new Dictionary<Terminator, ReduceInformation>{ }, "f6d9e32c-49ee-436c-ba97-fcdd527d9fdb") },
		{ "77dacef1-16f6-4ede-93f5-3f7c1a45da85", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.LeftParenthesis), "da7faa6b-7d3d-468d-af84-308d59ef8647"}, { new NonTerminator(NonTerminatorType.IdVarPart), "e2457739-4ca7-49ec-b8d8-8632118b69c3"}, { new Terminator(DelimiterType.LeftSquareBracket), "fc61abfc-0581-472d-9982-39b508a944be"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "77dacef1-16f6-4ede-93f5-3f7c1a45da85") },
		{ "8763ccfb-4769-4111-8f72-0408293e631a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "de72cbd1-d6eb-4f5e-92d1-14feb980cc14"}, { Terminator.NumberTerminator, "2a5a634a-a406-4e54-a148-6378d751f2c4"}, { new NonTerminator(NonTerminatorType.Variable), "bfc1ac1f-9a68-43c1-a75d-612f181a96d2"}, { new Terminator(DelimiterType.LeftParenthesis), "f6d9e32c-49ee-436c-ba97-fcdd527d9fdb"}, { Terminator.IdentifierTerminator, "77dacef1-16f6-4ede-93f5-3f7c1a45da85"}, { new Terminator(KeywordType.Not), "8763ccfb-4769-4111-8f72-0408293e631a"}, { new Terminator(OperatorType.Minus), "6abdc924-448f-4e57-8b91-afb7c5e8026c"},}, new Dictionary<Terminator, ReduceInformation>{ }, "8763ccfb-4769-4111-8f72-0408293e631a") },
		{ "6abdc924-448f-4e57-8b91-afb7c5e8026c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "2fa813ed-0199-4b84-81ce-c5ff2a0e42db"}, { Terminator.NumberTerminator, "2a5a634a-a406-4e54-a148-6378d751f2c4"}, { new NonTerminator(NonTerminatorType.Variable), "bfc1ac1f-9a68-43c1-a75d-612f181a96d2"}, { new Terminator(DelimiterType.LeftParenthesis), "f6d9e32c-49ee-436c-ba97-fcdd527d9fdb"}, { Terminator.IdentifierTerminator, "77dacef1-16f6-4ede-93f5-3f7c1a45da85"}, { new Terminator(KeywordType.Not), "8763ccfb-4769-4111-8f72-0408293e631a"}, { new Terminator(OperatorType.Minus), "6abdc924-448f-4e57-8b91-afb7c5e8026c"},}, new Dictionary<Terminator, ReduceInformation>{ }, "6abdc924-448f-4e57-8b91-afb7c5e8026c") },
		{ "560cde6a-a7d2-42d0-8919-35ede21bbf77", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "a67b3d92-1e95-452c-aeb4-95fe5c85f9bc"}, { new Terminator(OperatorType.Multiply), "fd20e81d-52b7-4920-adab-e2a1e49a6b6a"}, { new Terminator(OperatorType.Divide), "ef7280a1-85cf-4590-ab87-68bdd867d9e5"}, { new Terminator(KeywordType.Divide), "9192ebb2-aa8a-414d-ac9e-7206883933e6"}, { new Terminator(KeywordType.Mod), "ae32e061-59db-49fd-addd-2d5c3e9014a3"}, { new Terminator(KeywordType.And), "5c4b371b-c686-4545-b641-4c196ca6b25f"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "560cde6a-a7d2-42d0-8919-35ede21bbf77") },
		{ "60557bde-770c-4a18-be29-61dccb476f41", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, }, "60557bde-770c-4a18-be29-61dccb476f41") },
		{ "d75e0d15-0a0e-4d7e-b9e5-ebd5c6359643", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "d75e0d15-0a0e-4d7e-b9e5-ebd5c6359643") },
		{ "1a6d2be9-bf26-44cc-97e4-6060948a8ddd", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "102faf96-c009-4341-84ed-b1dd5106bdaf"}, { new Terminator(DelimiterType.Comma), "dd4993c2-1db9-4e5a-a3f2-c4660ec4e365"},}, new Dictionary<Terminator, ReduceInformation>{ }, "1a6d2be9-bf26-44cc-97e4-6060948a8ddd") },
		{ "a21bab7a-595c-441e-a87c-8e8407b7e04a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightSquareBracket), "efebc401-8ed3-4811-b223-54872f5fde26"}, { new Terminator(DelimiterType.Comma), "ecf84275-ae5c-4a59-96fb-d2ef13a595a8"},}, new Dictionary<Terminator, ReduceInformation>{ }, "a21bab7a-595c-441e-a87c-8e8407b7e04a") },
		{ "c7a1b3aa-fefe-40e4-ab25-d9d38ea14dbb", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, }, "c7a1b3aa-fefe-40e4-ab25-d9d38ea14dbb") },
		{ "d41e795a-82db-45e2-bf39-67e8469cb130", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "d41e795a-82db-45e2-bf39-67e8469cb130") },
		{ "ed8639a3-5285-46a5-8a2d-72a7c3e2432d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "216a7454-691e-41bb-9366-39c9eab4bfb9"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "7326cbb1-45a6-48f7-9156-959b6c65b1ed"}, { new NonTerminator(NonTerminatorType.Term), "e9a83490-e674-4a70-a544-af4b50adfbb5"}, { new NonTerminator(NonTerminatorType.Factor), "dcaea83e-fb38-455b-b282-8c83dfe32e9c"}, { Terminator.NumberTerminator, "57ef29e2-0bea-430d-9c00-5c6811158257"}, { new NonTerminator(NonTerminatorType.Variable), "6cc38689-9575-499b-8718-f71a97c1c393"}, { new Terminator(DelimiterType.LeftParenthesis), "99cef7cc-a6db-4888-bd5c-3276c85ed9e0"}, { Terminator.IdentifierTerminator, "eb0a685b-db40-4c1f-b8d2-c41f7cf7103f"}, { new Terminator(KeywordType.Not), "fd0bf9cf-2e29-4301-82ea-99b47903069c"}, { new Terminator(OperatorType.Minus), "cc266588-7a02-4ee7-9409-719f30519c17"},}, new Dictionary<Terminator, ReduceInformation>{ }, "ed8639a3-5285-46a5-8a2d-72a7c3e2432d") },
		{ "d4c5b403-de15-4e78-88e0-591c0adc9c3a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.SimpleExpression), "3a0f9bf3-2011-42ce-82a2-31701fcd65dc"}, { new NonTerminator(NonTerminatorType.Term), "597d7e8b-4d0f-49c4-a707-7e0b7a4bf952"}, { new NonTerminator(NonTerminatorType.Factor), "11b69276-2a19-4474-87e1-22741146fc72"}, { Terminator.NumberTerminator, "219b97c4-d730-4e29-8186-27753ab58d87"}, { new NonTerminator(NonTerminatorType.Variable), "1c864166-5a98-4b66-8dd4-3aadfa236783"}, { new Terminator(DelimiterType.LeftParenthesis), "1d68382c-7171-4b2b-be37-e904cbe63bff"}, { Terminator.IdentifierTerminator, "d8d12ab8-c78b-4258-921c-d19ad4d59688"}, { new Terminator(KeywordType.Not), "1945f333-698c-484f-affd-a1b5f144b742"}, { new Terminator(OperatorType.Minus), "0d1c80c4-3fc0-43e4-b249-9a1d66dbfc13"},}, new Dictionary<Terminator, ReduceInformation>{ }, "d4c5b403-de15-4e78-88e0-591c0adc9c3a") },
		{ "8fdee538-6af7-4ee3-9dff-debce1b8d1dd", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Term), "612bc107-83fa-45a9-a671-b3b00b04fe71"}, { new NonTerminator(NonTerminatorType.Factor), "e8c6655e-9ea9-40f9-99d2-7d55a1cdf5dd"}, { Terminator.NumberTerminator, "a3cb4c23-4b02-4dd3-a2f3-1c50b7a1a951"}, { new NonTerminator(NonTerminatorType.Variable), "138c0386-8ea0-455a-a824-c6b28e3537c9"}, { new Terminator(DelimiterType.LeftParenthesis), "5b8d9c0d-8dbc-4eb2-bd3f-537c3984b48e"}, { Terminator.IdentifierTerminator, "8e45f22e-454f-47f0-8ef0-2b3c0fa1878b"}, { new Terminator(KeywordType.Not), "14e584b7-b902-49cf-95c1-16d6d0e40ac1"}, { new Terminator(OperatorType.Minus), "b723344c-19d8-4ecf-a0aa-eb8cc3782aff"},}, new Dictionary<Terminator, ReduceInformation>{ }, "8fdee538-6af7-4ee3-9dff-debce1b8d1dd") },
		{ "89fb02c8-5527-435b-a3fa-14b3751e1700", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "e8330dcd-539c-4abe-a475-b5eb84e38450"}, { Terminator.NumberTerminator, "a3cb4c23-4b02-4dd3-a2f3-1c50b7a1a951"}, { new NonTerminator(NonTerminatorType.Variable), "138c0386-8ea0-455a-a824-c6b28e3537c9"}, { new Terminator(DelimiterType.LeftParenthesis), "5b8d9c0d-8dbc-4eb2-bd3f-537c3984b48e"}, { Terminator.IdentifierTerminator, "8e45f22e-454f-47f0-8ef0-2b3c0fa1878b"}, { new Terminator(KeywordType.Not), "14e584b7-b902-49cf-95c1-16d6d0e40ac1"}, { new Terminator(OperatorType.Minus), "b723344c-19d8-4ecf-a0aa-eb8cc3782aff"},}, new Dictionary<Terminator, ReduceInformation>{ }, "89fb02c8-5527-435b-a3fa-14b3751e1700") },
		{ "7755b452-3925-4c65-a989-e15df6f8f663", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "98cce2c1-b5f2-429e-a264-162f0d3b18e9"},}, new Dictionary<Terminator, ReduceInformation>{ }, "7755b452-3925-4c65-a989-e15df6f8f663") },
		{ "1d9aa238-31cf-4e2f-a613-73680ee94717", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "60e0dc2d-2aec-4075-9c2e-e3a2405fd701"}, { new NonTerminator(NonTerminatorType.Expression), "a582ab7f-9aa1-4130-9df3-2ba9dd0fe6f3"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "c3098ddd-d7d7-4d8b-b9ab-1ebedac5467f"}, { new NonTerminator(NonTerminatorType.Term), "c83de3f5-4130-4b2e-8715-e72b2e35275e"}, { new NonTerminator(NonTerminatorType.Factor), "9c184da0-a0b4-4ae3-9b1e-93b0fe09fbf1"}, { Terminator.NumberTerminator, "eaf4d875-7318-4f6e-b25b-829d30a888b8"}, { new NonTerminator(NonTerminatorType.Variable), "eb827526-1aa2-47ab-8b01-b1b0792e4297"}, { new Terminator(DelimiterType.LeftParenthesis), "2894d0f9-3faf-4be5-b2b6-c499611fc58d"}, { Terminator.IdentifierTerminator, "8efce527-7ba0-435f-a58f-b048a7a0e505"}, { new Terminator(KeywordType.Not), "dd22d779-6929-400f-832a-6d4e7eda69e6"}, { new Terminator(OperatorType.Minus), "28fb603c-e1e9-4bfa-ab9b-f8887e1996c0"},}, new Dictionary<Terminator, ReduceInformation>{ }, "1d9aa238-31cf-4e2f-a613-73680ee94717") },
		{ "b126e8cd-fd9c-4fcd-9efb-0d577e651425", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, }, "b126e8cd-fd9c-4fcd-9efb-0d577e651425") },
		{ "49a0716d-d548-4ba7-bdfe-68c7fc3a5830", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "532c33b5-c2a7-469a-8d7b-6078f9d174d9"}, { new NonTerminator(NonTerminatorType.Expression), "10204ac0-c523-4b6c-890d-d508a8e83f48"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "72d6e812-1720-408a-ac1c-5ac20fa267d1"}, { new NonTerminator(NonTerminatorType.Term), "0b3b44d2-8458-48d1-b7c4-2329d479b56c"}, { new NonTerminator(NonTerminatorType.Factor), "2f595e0e-c29e-494d-a5bb-a570db3254fd"}, { Terminator.NumberTerminator, "c86f637c-4e17-4d3d-bfd7-f1fc5abdf7f4"}, { new NonTerminator(NonTerminatorType.Variable), "fa555a17-89d0-4cf7-989a-0b0775ade6d4"}, { new Terminator(DelimiterType.LeftParenthesis), "8fab798a-03a8-45b7-8380-7b7eaee651fa"}, { Terminator.IdentifierTerminator, "513b9c84-fc68-4334-976d-7547b1ec521f"}, { new Terminator(KeywordType.Not), "928a980c-85a8-4cf5-a5d1-117516baef4e"}, { new Terminator(OperatorType.Minus), "e2c085de-e99b-4f3a-b869-c3ca5f5dcf75"},}, new Dictionary<Terminator, ReduceInformation>{ }, "49a0716d-d548-4ba7-bdfe-68c7fc3a5830") },
		{ "f1113cac-5a3d-4575-b4f9-e07883c85a0d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "f1113cac-5a3d-4575-b4f9-e07883c85a0d") },
		{ "c31fed74-c9a5-4008-a871-1830d016ab4d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "c31fed74-c9a5-4008-a871-1830d016ab4d") },
		{ "8dbceb0c-4b60-4cdd-bb30-e3a72e0d61b8", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(KeywordType.End), "29415693-948d-465c-86a1-27911f0f9121"}, { new Terminator(DelimiterType.Semicolon), "f83f97d0-e0c9-462f-b6fa-0bc1226d63fe"},}, new Dictionary<Terminator, ReduceInformation>{ }, "8dbceb0c-4b60-4cdd-bb30-e3a72e0d61b8") },
		{ "06d47b85-52f7-4599-bdb2-57773137db6f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.IdentifierList), "7b79d969-3543-466c-8c4f-2403ae51406c"}, { Terminator.IdentifierTerminator, "3ed14f62-8e9f-4c1a-8e1e-5d522192093b"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Begin), new ReduceInformation(3, new NonTerminator(NonTerminatorType.VarDeclarations))}, }, "06d47b85-52f7-4599-bdb2-57773137db6f") },
		{ "09dc4e35-539c-448a-a588-f8628761e4dd", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.ParameterList))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.ParameterList))}, }, "09dc4e35-539c-448a-a588-f8628761e4dd") },
		{ "0d6b6db7-375e-459a-a4f5-8d900ae56699", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.ValueParameter))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.ValueParameter))}, }, "0d6b6db7-375e-459a-a4f5-8d900ae56699") },
		{ "7c1ea2ce-ab74-4a53-9781-b5f292c9774d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.BasicType))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.BasicType))}, }, "7c1ea2ce-ab74-4a53-9781-b5f292c9774d") },
		{ "319df1bd-d7b8-480e-8ab0-434607cd08e9", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.BasicType))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.BasicType))}, }, "319df1bd-d7b8-480e-8ab0-434607cd08e9") },
		{ "c19a1a5c-46f4-4c72-935c-acd21cf32ca7", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.BasicType))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.BasicType))}, }, "c19a1a5c-46f4-4c72-935c-acd21cf32ca7") },
		{ "0fedfaad-a12e-4064-a450-831c5fb93533", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.BasicType))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.BasicType))}, }, "0fedfaad-a12e-4064-a450-831c5fb93533") },
		{ "dbebe90b-0d37-43df-a8b8-477db9a7d9dc", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.BasicType), "ac94bab2-b166-4b45-bcd8-e4e7e98e265a"}, { new Terminator(KeywordType.Integer), "e3c9fab2-9cd8-44ad-a33c-54482b5323d5"}, { new Terminator(KeywordType.Real), "3bb38fcd-bf5c-401a-a2e4-f6f4d91bc650"}, { new Terminator(KeywordType.Boolean), "01a76185-0283-464f-b7d6-9f5bb8517edd"}, { new Terminator(KeywordType.Character), "ee488773-f67c-418c-a1a5-ddd934d4253b"},}, new Dictionary<Terminator, ReduceInformation>{ }, "dbebe90b-0d37-43df-a8b8-477db9a7d9dc") },
		{ "f7a93f8c-d074-4d4d-a5ad-ebae9f2b1a32", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.DoubleDots), "daa7fd64-65f5-4123-b762-eaab9c41f961"},}, new Dictionary<Terminator, ReduceInformation>{ }, "f7a93f8c-d074-4d4d-a5ad-ebae9f2b1a32") },
		{ "c6ab978e-3d10-4f6a-a6a7-568eaee5ba44", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Period))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Period))}, }, "c6ab978e-3d10-4f6a-a6a7-568eaee5ba44") },
		{ "7770ad47-2dbd-48a1-8fc8-65f5f7d27956", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Term), "30447df9-d167-4a40-b94d-3ba35cf1986c"}, { new NonTerminator(NonTerminatorType.Factor), "19cfe0eb-0189-4bfa-83ef-442a36cd1138"}, { Terminator.NumberTerminator, "ff4e43b4-9264-41ed-8fd4-fe02b4673621"}, { new NonTerminator(NonTerminatorType.Variable), "70dc3750-b84d-4199-808f-50c3ab8858e0"}, { new Terminator(DelimiterType.LeftParenthesis), "be22425a-a8f1-4ac2-b180-9c579b063d86"}, { Terminator.IdentifierTerminator, "75199901-29ad-4fde-8e16-2a9ce7439b9d"}, { new Terminator(KeywordType.Not), "ffcfe34e-f488-4c19-b03c-f14a0c709964"}, { new Terminator(OperatorType.Minus), "be7ba5b3-c18b-4687-82d8-69e621837916"},}, new Dictionary<Terminator, ReduceInformation>{ }, "7770ad47-2dbd-48a1-8fc8-65f5f7d27956") },
		{ "2cd1bf85-bbe1-47c2-ac10-a540522bdbce", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "69d21347-4e11-40ef-90e3-4f2e977dfcfc"}, { Terminator.NumberTerminator, "ff4e43b4-9264-41ed-8fd4-fe02b4673621"}, { new NonTerminator(NonTerminatorType.Variable), "70dc3750-b84d-4199-808f-50c3ab8858e0"}, { new Terminator(DelimiterType.LeftParenthesis), "be22425a-a8f1-4ac2-b180-9c579b063d86"}, { Terminator.IdentifierTerminator, "75199901-29ad-4fde-8e16-2a9ce7439b9d"}, { new Terminator(KeywordType.Not), "ffcfe34e-f488-4c19-b03c-f14a0c709964"}, { new Terminator(OperatorType.Minus), "be7ba5b3-c18b-4687-82d8-69e621837916"},}, new Dictionary<Terminator, ReduceInformation>{ }, "2cd1bf85-bbe1-47c2-ac10-a540522bdbce") },
		{ "1d626d30-a2b4-4eaf-8272-f008a108d644", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "d105faca-ad26-4f11-9c7f-abaad0977c83"},}, new Dictionary<Terminator, ReduceInformation>{ }, "1d626d30-a2b4-4eaf-8272-f008a108d644") },
		{ "df220402-c4e0-4ed1-b602-25a79ac551f4", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "68d70314-f043-4283-b542-73e3bd19f2b4"}, { new NonTerminator(NonTerminatorType.Expression), "a582ab7f-9aa1-4130-9df3-2ba9dd0fe6f3"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "c3098ddd-d7d7-4d8b-b9ab-1ebedac5467f"}, { new NonTerminator(NonTerminatorType.Term), "c83de3f5-4130-4b2e-8715-e72b2e35275e"}, { new NonTerminator(NonTerminatorType.Factor), "9c184da0-a0b4-4ae3-9b1e-93b0fe09fbf1"}, { Terminator.NumberTerminator, "eaf4d875-7318-4f6e-b25b-829d30a888b8"}, { new NonTerminator(NonTerminatorType.Variable), "eb827526-1aa2-47ab-8b01-b1b0792e4297"}, { new Terminator(DelimiterType.LeftParenthesis), "2894d0f9-3faf-4be5-b2b6-c499611fc58d"}, { Terminator.IdentifierTerminator, "8efce527-7ba0-435f-a58f-b048a7a0e505"}, { new Terminator(KeywordType.Not), "dd22d779-6929-400f-832a-6d4e7eda69e6"}, { new Terminator(OperatorType.Minus), "28fb603c-e1e9-4bfa-ab9b-f8887e1996c0"},}, new Dictionary<Terminator, ReduceInformation>{ }, "df220402-c4e0-4ed1-b602-25a79ac551f4") },
		{ "c387a2bb-8d76-420f-b6ce-1a2ba965b2ed", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, }, "c387a2bb-8d76-420f-b6ce-1a2ba965b2ed") },
		{ "fe415909-e195-4f24-b56a-40b7fad976fa", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "f591e305-ac84-4acb-88f5-2834100db7b0"}, { new NonTerminator(NonTerminatorType.Expression), "10204ac0-c523-4b6c-890d-d508a8e83f48"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "72d6e812-1720-408a-ac1c-5ac20fa267d1"}, { new NonTerminator(NonTerminatorType.Term), "0b3b44d2-8458-48d1-b7c4-2329d479b56c"}, { new NonTerminator(NonTerminatorType.Factor), "2f595e0e-c29e-494d-a5bb-a570db3254fd"}, { Terminator.NumberTerminator, "c86f637c-4e17-4d3d-bfd7-f1fc5abdf7f4"}, { new NonTerminator(NonTerminatorType.Variable), "fa555a17-89d0-4cf7-989a-0b0775ade6d4"}, { new Terminator(DelimiterType.LeftParenthesis), "8fab798a-03a8-45b7-8380-7b7eaee651fa"}, { Terminator.IdentifierTerminator, "513b9c84-fc68-4334-976d-7547b1ec521f"}, { new Terminator(KeywordType.Not), "928a980c-85a8-4cf5-a5d1-117516baef4e"}, { new Terminator(OperatorType.Minus), "e2c085de-e99b-4f3a-b869-c3ca5f5dcf75"},}, new Dictionary<Terminator, ReduceInformation>{ }, "fe415909-e195-4f24-b56a-40b7fad976fa") },
		{ "494e772e-87f2-41ec-a1ea-bde8b1aa79e3", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "494e772e-87f2-41ec-a1ea-bde8b1aa79e3") },
		{ "69490ee7-ab21-4ae8-929e-b13c51bf25d3", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "69490ee7-ab21-4ae8-929e-b13c51bf25d3") },
		{ "7fbabe78-2047-47c6-bf30-503e7d86fd9b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, }, "7fbabe78-2047-47c6-bf30-503e7d86fd9b") },
		{ "1aeba5d6-3fe4-449d-9e7a-8b55f2e37f41", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "1aeba5d6-3fe4-449d-9e7a-8b55f2e37f41") },
		{ "34927101-5f58-4602-8f84-22bf65fa02a3", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Term), "fb7bfd65-3028-4090-99f7-d7db91475e36"}, { new NonTerminator(NonTerminatorType.Factor), "13f24f9d-eafa-49d3-981d-e9708deefa70"}, { Terminator.NumberTerminator, "c36a5a82-356a-4f27-9507-4853647330f8"}, { new NonTerminator(NonTerminatorType.Variable), "25d3aea4-fc9e-41ac-a0ba-db8ef11cee57"}, { new Terminator(DelimiterType.LeftParenthesis), "801568bc-0c2a-423f-9fd4-7f216d41b5c8"}, { Terminator.IdentifierTerminator, "62d908d5-bc3f-41cd-9d71-fff26a1343c1"}, { new Terminator(KeywordType.Not), "2269c88f-87d2-4c54-9270-380d70d9ec35"}, { new Terminator(OperatorType.Minus), "3c697023-0a28-43a8-be5b-ef957bbf16a6"},}, new Dictionary<Terminator, ReduceInformation>{ }, "34927101-5f58-4602-8f84-22bf65fa02a3") },
		{ "1fcec038-a6e7-4e28-833e-6bb9424f7ca9", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "59068e86-8067-49a2-bf02-7229303f6e08"}, { Terminator.NumberTerminator, "c36a5a82-356a-4f27-9507-4853647330f8"}, { new NonTerminator(NonTerminatorType.Variable), "25d3aea4-fc9e-41ac-a0ba-db8ef11cee57"}, { new Terminator(DelimiterType.LeftParenthesis), "801568bc-0c2a-423f-9fd4-7f216d41b5c8"}, { Terminator.IdentifierTerminator, "62d908d5-bc3f-41cd-9d71-fff26a1343c1"}, { new Terminator(KeywordType.Not), "2269c88f-87d2-4c54-9270-380d70d9ec35"}, { new Terminator(OperatorType.Minus), "3c697023-0a28-43a8-be5b-ef957bbf16a6"},}, new Dictionary<Terminator, ReduceInformation>{ }, "1fcec038-a6e7-4e28-833e-6bb9424f7ca9") },
		{ "0d8e7b5f-ce90-4222-900e-26a12528e9d6", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "e4d06a61-8f33-4483-a9dc-89bb1fcd93b1"},}, new Dictionary<Terminator, ReduceInformation>{ }, "0d8e7b5f-ce90-4222-900e-26a12528e9d6") },
		{ "2535e225-0b06-4a60-93e1-85292149e8ad", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "618b4374-d213-4519-8245-1890c0d2aff4"}, { new NonTerminator(NonTerminatorType.Expression), "a582ab7f-9aa1-4130-9df3-2ba9dd0fe6f3"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "c3098ddd-d7d7-4d8b-b9ab-1ebedac5467f"}, { new NonTerminator(NonTerminatorType.Term), "c83de3f5-4130-4b2e-8715-e72b2e35275e"}, { new NonTerminator(NonTerminatorType.Factor), "9c184da0-a0b4-4ae3-9b1e-93b0fe09fbf1"}, { Terminator.NumberTerminator, "eaf4d875-7318-4f6e-b25b-829d30a888b8"}, { new NonTerminator(NonTerminatorType.Variable), "eb827526-1aa2-47ab-8b01-b1b0792e4297"}, { new Terminator(DelimiterType.LeftParenthesis), "2894d0f9-3faf-4be5-b2b6-c499611fc58d"}, { Terminator.IdentifierTerminator, "8efce527-7ba0-435f-a58f-b048a7a0e505"}, { new Terminator(KeywordType.Not), "dd22d779-6929-400f-832a-6d4e7eda69e6"}, { new Terminator(OperatorType.Minus), "28fb603c-e1e9-4bfa-ab9b-f8887e1996c0"},}, new Dictionary<Terminator, ReduceInformation>{ }, "2535e225-0b06-4a60-93e1-85292149e8ad") },
		{ "fb68fd49-5145-4ef1-acf5-8ce1675a1b8c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, }, "fb68fd49-5145-4ef1-acf5-8ce1675a1b8c") },
		{ "53f9b5b9-3450-4e3b-b74c-a5e432c28c25", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "a7d8221d-33d2-4435-81b3-de1de6acbeeb"}, { new NonTerminator(NonTerminatorType.Expression), "10204ac0-c523-4b6c-890d-d508a8e83f48"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "72d6e812-1720-408a-ac1c-5ac20fa267d1"}, { new NonTerminator(NonTerminatorType.Term), "0b3b44d2-8458-48d1-b7c4-2329d479b56c"}, { new NonTerminator(NonTerminatorType.Factor), "2f595e0e-c29e-494d-a5bb-a570db3254fd"}, { Terminator.NumberTerminator, "c86f637c-4e17-4d3d-bfd7-f1fc5abdf7f4"}, { new NonTerminator(NonTerminatorType.Variable), "fa555a17-89d0-4cf7-989a-0b0775ade6d4"}, { new Terminator(DelimiterType.LeftParenthesis), "8fab798a-03a8-45b7-8380-7b7eaee651fa"}, { Terminator.IdentifierTerminator, "513b9c84-fc68-4334-976d-7547b1ec521f"}, { new Terminator(KeywordType.Not), "928a980c-85a8-4cf5-a5d1-117516baef4e"}, { new Terminator(OperatorType.Minus), "e2c085de-e99b-4f3a-b869-c3ca5f5dcf75"},}, new Dictionary<Terminator, ReduceInformation>{ }, "53f9b5b9-3450-4e3b-b74c-a5e432c28c25") },
		{ "c26cd85c-b279-40bf-b747-cdcb11b5a046", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "c26cd85c-b279-40bf-b747-cdcb11b5a046") },
		{ "931e83e9-1736-4ae4-885d-edbfb0a4277f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "931e83e9-1736-4ae4-885d-edbfb0a4277f") },
		{ "6d1d3243-8773-4a86-ba5d-0157c8af19b9", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, }, "6d1d3243-8773-4a86-ba5d-0157c8af19b9") },
		{ "5d6ffae2-17f9-4a7a-b8fa-4a2f394dfab2", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "5d6ffae2-17f9-4a7a-b8fa-4a2f394dfab2") },
		{ "31793d6b-e65f-48d3-b3b3-42f95c8eaa5f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Term), "f8a846bc-f12d-497e-b272-eba79a788765"}, { new NonTerminator(NonTerminatorType.Factor), "a6ee60f9-e7a6-4693-a4e8-cd41934b1727"}, { Terminator.NumberTerminator, "a7df07cd-c0da-4e90-8871-a13944fae98c"}, { new NonTerminator(NonTerminatorType.Variable), "265ce997-0847-47ff-84ad-380765fb1439"}, { new Terminator(DelimiterType.LeftParenthesis), "1e10eeed-1b88-45a5-a914-bbcf404cda86"}, { Terminator.IdentifierTerminator, "55db5714-9404-40ae-a5cc-d7cd7968f2be"}, { new Terminator(KeywordType.Not), "cba1e33d-813e-4f0e-9bda-ba7f1020c426"}, { new Terminator(OperatorType.Minus), "4689d95f-7bbb-41de-9f96-ba34da344607"},}, new Dictionary<Terminator, ReduceInformation>{ }, "31793d6b-e65f-48d3-b3b3-42f95c8eaa5f") },
		{ "5cf7b429-f628-4955-90cd-8b9540a53fe4", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "c7a65262-29a7-419c-9803-8552d96b9ad0"}, { Terminator.NumberTerminator, "a7df07cd-c0da-4e90-8871-a13944fae98c"}, { new NonTerminator(NonTerminatorType.Variable), "265ce997-0847-47ff-84ad-380765fb1439"}, { new Terminator(DelimiterType.LeftParenthesis), "1e10eeed-1b88-45a5-a914-bbcf404cda86"}, { Terminator.IdentifierTerminator, "55db5714-9404-40ae-a5cc-d7cd7968f2be"}, { new Terminator(KeywordType.Not), "cba1e33d-813e-4f0e-9bda-ba7f1020c426"}, { new Terminator(OperatorType.Minus), "4689d95f-7bbb-41de-9f96-ba34da344607"},}, new Dictionary<Terminator, ReduceInformation>{ }, "5cf7b429-f628-4955-90cd-8b9540a53fe4") },
		{ "024c3624-c4cf-4acd-a3e3-890352daf1e9", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "414267ba-7e99-45d0-b258-b7c073db3181"},}, new Dictionary<Terminator, ReduceInformation>{ }, "024c3624-c4cf-4acd-a3e3-890352daf1e9") },
		{ "e141d1ca-269a-4f77-a953-63106ee7ac34", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "a1a4df8f-c531-43b6-b550-295b6fb04987"}, { new NonTerminator(NonTerminatorType.Expression), "a582ab7f-9aa1-4130-9df3-2ba9dd0fe6f3"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "c3098ddd-d7d7-4d8b-b9ab-1ebedac5467f"}, { new NonTerminator(NonTerminatorType.Term), "c83de3f5-4130-4b2e-8715-e72b2e35275e"}, { new NonTerminator(NonTerminatorType.Factor), "9c184da0-a0b4-4ae3-9b1e-93b0fe09fbf1"}, { Terminator.NumberTerminator, "eaf4d875-7318-4f6e-b25b-829d30a888b8"}, { new NonTerminator(NonTerminatorType.Variable), "eb827526-1aa2-47ab-8b01-b1b0792e4297"}, { new Terminator(DelimiterType.LeftParenthesis), "2894d0f9-3faf-4be5-b2b6-c499611fc58d"}, { Terminator.IdentifierTerminator, "8efce527-7ba0-435f-a58f-b048a7a0e505"}, { new Terminator(KeywordType.Not), "dd22d779-6929-400f-832a-6d4e7eda69e6"}, { new Terminator(OperatorType.Minus), "28fb603c-e1e9-4bfa-ab9b-f8887e1996c0"},}, new Dictionary<Terminator, ReduceInformation>{ }, "e141d1ca-269a-4f77-a953-63106ee7ac34") },
		{ "c1e2fed5-d017-4fd1-8c5e-d077a3b6d161", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, }, "c1e2fed5-d017-4fd1-8c5e-d077a3b6d161") },
		{ "34299268-f9b2-435f-b210-7395b25c72b8", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "acae3253-6722-469a-9f08-63fe0e2420e6"}, { new NonTerminator(NonTerminatorType.Expression), "10204ac0-c523-4b6c-890d-d508a8e83f48"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "72d6e812-1720-408a-ac1c-5ac20fa267d1"}, { new NonTerminator(NonTerminatorType.Term), "0b3b44d2-8458-48d1-b7c4-2329d479b56c"}, { new NonTerminator(NonTerminatorType.Factor), "2f595e0e-c29e-494d-a5bb-a570db3254fd"}, { Terminator.NumberTerminator, "c86f637c-4e17-4d3d-bfd7-f1fc5abdf7f4"}, { new NonTerminator(NonTerminatorType.Variable), "fa555a17-89d0-4cf7-989a-0b0775ade6d4"}, { new Terminator(DelimiterType.LeftParenthesis), "8fab798a-03a8-45b7-8380-7b7eaee651fa"}, { Terminator.IdentifierTerminator, "513b9c84-fc68-4334-976d-7547b1ec521f"}, { new Terminator(KeywordType.Not), "928a980c-85a8-4cf5-a5d1-117516baef4e"}, { new Terminator(OperatorType.Minus), "e2c085de-e99b-4f3a-b869-c3ca5f5dcf75"},}, new Dictionary<Terminator, ReduceInformation>{ }, "34299268-f9b2-435f-b210-7395b25c72b8") },
		{ "e9db6c03-fe83-436f-beef-6379906c15e0", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "e9db6c03-fe83-436f-beef-6379906c15e0") },
		{ "88b33a61-8b80-4e73-b675-c9027cbafac2", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "88b33a61-8b80-4e73-b675-c9027cbafac2") },
		{ "2b0118c3-48cd-489e-a7a6-623a05d1c646", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, }, "2b0118c3-48cd-489e-a7a6-623a05d1c646") },
		{ "b925d90f-939b-4806-bbc7-e9aca4c3d1ab", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "b925d90f-939b-4806-bbc7-e9aca4c3d1ab") },
		{ "d6366716-ff5a-4a53-89fa-d83fdf929ec9", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(2, new NonTerminator(NonTerminatorType.ElsePart))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.ElsePart))}, }, "d6366716-ff5a-4a53-89fa-d83fdf929ec9") },
		{ "50923814-6ff6-49a1-8a51-b0f1d6d43539", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(KeywordType.Else), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Statement))}, }, "50923814-6ff6-49a1-8a51-b0f1d6d43539") },
		{ "d4f48672-6fe9-4839-a022-03ad51a13b21", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.RelationOperator), "a6adcf5d-6051-451c-9ada-9752d0e76df6"}, { new Terminator(OperatorType.Equal), "6112dd54-ce34-4dd2-be4c-b06dd725bbd8"}, { new Terminator(OperatorType.NotEqual), "1338afe1-da9e-4380-8a28-7f2d82ef2d68"}, { new Terminator(OperatorType.Less), "fc9b446b-c4b5-4720-bc46-49344508202f"}, { new Terminator(OperatorType.LessEqual), "39ac36f9-efe7-4a1b-ba37-a9729fbf341a"}, { new Terminator(OperatorType.Greater), "fc85ac1b-5a0f-405f-a8f0-a1e20d44351d"}, { new Terminator(OperatorType.GreaterEqual), "f0598e66-3db3-44bd-8b9d-cd6c038df056"}, { new NonTerminator(NonTerminatorType.AddOperator), "1add5942-f5b1-44ac-beac-fa083456e9c7"}, { new Terminator(OperatorType.Plus), "493d6d76-ed65-4308-aee5-65c5fe502546"}, { new Terminator(OperatorType.Minus), "c41b579b-0a72-411a-b9ed-f3ca30f5a615"}, { new Terminator(KeywordType.Or), "22878828-eed7-470f-a07f-595bb2eb7e2a"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Expression))}, { new Terminator(KeywordType.Else), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Expression))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Expression))}, }, "d4f48672-6fe9-4839-a022-03ad51a13b21") },
		{ "cffb0ebf-158b-40bc-a554-a29574d03375", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "0bd458b2-525c-4ef6-9304-1ea198e8e266"}, { new Terminator(OperatorType.Multiply), "fd20e81d-52b7-4920-adab-e2a1e49a6b6a"}, { new Terminator(OperatorType.Divide), "ef7280a1-85cf-4590-ab87-68bdd867d9e5"}, { new Terminator(KeywordType.Divide), "9192ebb2-aa8a-414d-ac9e-7206883933e6"}, { new Terminator(KeywordType.Mod), "ae32e061-59db-49fd-addd-2d5c3e9014a3"}, { new Terminator(KeywordType.And), "5c4b371b-c686-4545-b641-4c196ca6b25f"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Else), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "cffb0ebf-158b-40bc-a554-a29574d03375") },
		{ "a697a4fa-5d3d-45c4-a1da-f2783855186e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Else), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, }, "a697a4fa-5d3d-45c4-a1da-f2783855186e") },
		{ "d0368978-0f30-41f5-a0cc-b837ed474739", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Else), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "d0368978-0f30-41f5-a0cc-b837ed474739") },
		{ "72921583-cb29-44a3-a883-d86b10c630e4", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Else), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "72921583-cb29-44a3-a883-d86b10c630e4") },
		{ "df70bbe0-47c7-4421-80d2-ad8e2c4dac96", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "ea00e798-b896-41f9-bc20-62e39055b355"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "38eec331-d4ae-4b06-a659-98475fdb5aa0"}, { new NonTerminator(NonTerminatorType.Term), "5a5a9a53-cfe3-4e81-8fcf-2e6d11b16b0e"}, { new NonTerminator(NonTerminatorType.Factor), "175e095a-0963-472f-b84a-ece518bd2cf2"}, { Terminator.NumberTerminator, "3861f05c-791c-4f64-b291-d985540c7651"}, { new NonTerminator(NonTerminatorType.Variable), "c693eb95-cdcd-4c7d-8717-3ead45ef8bfc"}, { new Terminator(DelimiterType.LeftParenthesis), "efc9be5f-b54d-4b96-8aec-f3540a407dc8"}, { Terminator.IdentifierTerminator, "a8ce470f-1537-428f-88c2-9270bfed81b6"}, { new Terminator(KeywordType.Not), "addb6420-8367-48a9-9f28-327cf2a88028"}, { new Terminator(OperatorType.Minus), "75101aaf-bd06-4db5-809c-9e8b4533dc5d"},}, new Dictionary<Terminator, ReduceInformation>{ }, "df70bbe0-47c7-4421-80d2-ad8e2c4dac96") },
		{ "04758ff4-52e2-493b-a02d-951bd58ede9b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.LeftParenthesis), "9925852d-39bb-4046-b44f-875af1075f24"}, { new NonTerminator(NonTerminatorType.IdVarPart), "9b7f2818-a929-4c62-a5c0-975c33244627"}, { new Terminator(DelimiterType.LeftSquareBracket), "3af5804f-cf1e-44c4-b12a-c2bf9f7163bb"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Equal), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Less), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Greater), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Else), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "04758ff4-52e2-493b-a02d-951bd58ede9b") },
		{ "a50cf991-5964-4e91-a4d2-bdfaebd4d461", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "dab188a3-9b20-4cd6-80d2-db2a11f96e25"}, { Terminator.NumberTerminator, "d0368978-0f30-41f5-a0cc-b837ed474739"}, { new NonTerminator(NonTerminatorType.Variable), "72921583-cb29-44a3-a883-d86b10c630e4"}, { new Terminator(DelimiterType.LeftParenthesis), "df70bbe0-47c7-4421-80d2-ad8e2c4dac96"}, { Terminator.IdentifierTerminator, "04758ff4-52e2-493b-a02d-951bd58ede9b"}, { new Terminator(KeywordType.Not), "a50cf991-5964-4e91-a4d2-bdfaebd4d461"}, { new Terminator(OperatorType.Minus), "c31ada0c-d881-4599-a8a5-a079e69b52dd"},}, new Dictionary<Terminator, ReduceInformation>{ }, "a50cf991-5964-4e91-a4d2-bdfaebd4d461") },
		{ "c31ada0c-d881-4599-a8a5-a079e69b52dd", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "49288d3e-6f3e-4bfc-9f9f-43383b389e9f"}, { Terminator.NumberTerminator, "d0368978-0f30-41f5-a0cc-b837ed474739"}, { new NonTerminator(NonTerminatorType.Variable), "72921583-cb29-44a3-a883-d86b10c630e4"}, { new Terminator(DelimiterType.LeftParenthesis), "df70bbe0-47c7-4421-80d2-ad8e2c4dac96"}, { Terminator.IdentifierTerminator, "04758ff4-52e2-493b-a02d-951bd58ede9b"}, { new Terminator(KeywordType.Not), "a50cf991-5964-4e91-a4d2-bdfaebd4d461"}, { new Terminator(OperatorType.Minus), "c31ada0c-d881-4599-a8a5-a079e69b52dd"},}, new Dictionary<Terminator, ReduceInformation>{ }, "c31ada0c-d881-4599-a8a5-a079e69b52dd") },
		{ "998f3c7b-1c09-443d-9b98-406c382b7329", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(KeywordType.Else), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Statement))}, }, "998f3c7b-1c09-443d-9b98-406c382b7329") },
		{ "43b8d4ad-d76d-4791-8fb1-9393ffb6d7dc", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "de46e3c3-3646-4245-99fc-72ace4bfa93a"}, { new Terminator(DelimiterType.Comma), "dd4993c2-1db9-4e5a-a3f2-c4660ec4e365"},}, new Dictionary<Terminator, ReduceInformation>{ }, "43b8d4ad-d76d-4791-8fb1-9393ffb6d7dc") },
		{ "481574c9-ea10-4832-afb6-16ef86a76e6e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Statement), "8b510cf2-2937-4d1a-875a-680521b03a9d"}, { new NonTerminator(NonTerminatorType.Variable), "c5d258c5-490b-4c90-8b5b-8242a6dbfc6d"}, { Terminator.IdentifierTerminator, "9a2b8a88-a48e-46e2-af69-f3561ad39bb7"}, { new NonTerminator(NonTerminatorType.ProcedureCall), "e9f86400-0193-40f4-a076-09e7b9b028e9"}, { new NonTerminator(NonTerminatorType.CompoundStatement), "48fe99c9-db82-494e-b751-8bda645590d0"}, { new Terminator(KeywordType.If), "79db3815-bc73-4868-ad20-1fc22d7453b9"}, { new Terminator(KeywordType.For), "ec4898d0-5ba7-47f1-885e-07725f2e43dc"}, { new Terminator(KeywordType.Begin), "65fb1933-38c9-42db-9907-36fe3c05573a"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(KeywordType.Else), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, }, "481574c9-ea10-4832-afb6-16ef86a76e6e") },
		{ "0a1fdbf2-0439-42f4-9035-5dd6f25dd036", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "56f110c6-1e8e-4db7-8564-8dc0fc7022ab"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "cd7ff125-4a37-4908-ae74-cd22185c30cb"}, { new NonTerminator(NonTerminatorType.Term), "1b085cbe-da51-490b-a30b-b52e6f8a3897"}, { new NonTerminator(NonTerminatorType.Factor), "e8c6655e-9ea9-40f9-99d2-7d55a1cdf5dd"}, { Terminator.NumberTerminator, "a3cb4c23-4b02-4dd3-a2f3-1c50b7a1a951"}, { new NonTerminator(NonTerminatorType.Variable), "138c0386-8ea0-455a-a824-c6b28e3537c9"}, { new Terminator(DelimiterType.LeftParenthesis), "5b8d9c0d-8dbc-4eb2-bd3f-537c3984b48e"}, { Terminator.IdentifierTerminator, "8e45f22e-454f-47f0-8ef0-2b3c0fa1878b"}, { new Terminator(KeywordType.Not), "14e584b7-b902-49cf-95c1-16d6d0e40ac1"}, { new Terminator(OperatorType.Minus), "b723344c-19d8-4ecf-a0aa-eb8cc3782aff"},}, new Dictionary<Terminator, ReduceInformation>{ }, "0a1fdbf2-0439-42f4-9035-5dd6f25dd036") },
		{ "7c27c7c7-ae11-4790-89df-1ce0959cb860", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.CompoundStatement))}, { new Terminator(KeywordType.Else), new ReduceInformation(3, new NonTerminator(NonTerminatorType.CompoundStatement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.CompoundStatement))}, }, "7c27c7c7-ae11-4790-89df-1ce0959cb860") },
		{ "3f10d9a1-f758-442e-be9d-821751ba8f4d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "ce757e88-c309-4c8f-b512-fde449379b78"}, { new Terminator(OperatorType.Multiply), "fd20e81d-52b7-4920-adab-e2a1e49a6b6a"}, { new Terminator(OperatorType.Divide), "ef7280a1-85cf-4590-ab87-68bdd867d9e5"}, { new Terminator(KeywordType.Divide), "9192ebb2-aa8a-414d-ac9e-7206883933e6"}, { new Terminator(KeywordType.Mod), "ae32e061-59db-49fd-addd-2d5c3e9014a3"}, { new Terminator(KeywordType.And), "5c4b371b-c686-4545-b641-4c196ca6b25f"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "3f10d9a1-f758-442e-be9d-821751ba8f4d") },
		{ "a40d046b-a706-4a39-84f2-3a28a038e558", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, }, "a40d046b-a706-4a39-84f2-3a28a038e558") },
		{ "759bbbce-3ad6-4cde-912d-3eb3d0efa050", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "759bbbce-3ad6-4cde-912d-3eb3d0efa050") },
		{ "52b527f4-8de6-4d44-924a-4a1b7847187a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "a6046e3c-f247-4f18-b9a0-bc7caed88cef"}, { new Terminator(DelimiterType.Comma), "dd4993c2-1db9-4e5a-a3f2-c4660ec4e365"},}, new Dictionary<Terminator, ReduceInformation>{ }, "52b527f4-8de6-4d44-924a-4a1b7847187a") },
		{ "9ebd33e8-8901-4932-873d-e225a974d882", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightSquareBracket), "0cd31d31-3e96-4a8e-9087-a42f503dd13e"}, { new Terminator(DelimiterType.Comma), "ecf84275-ae5c-4a59-96fb-d2ef13a595a8"},}, new Dictionary<Terminator, ReduceInformation>{ }, "9ebd33e8-8901-4932-873d-e225a974d882") },
		{ "d6624c6b-a187-4803-84fe-c3a8f6e434a2", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Term), "62d7d2f7-c815-4d03-b2e5-b6cb7d812709"}, { new NonTerminator(NonTerminatorType.Factor), "42bb7db9-5be9-4ac8-89ed-01578a50d2fc"}, { Terminator.NumberTerminator, "2a5a634a-a406-4e54-a148-6378d751f2c4"}, { new NonTerminator(NonTerminatorType.Variable), "bfc1ac1f-9a68-43c1-a75d-612f181a96d2"}, { new Terminator(DelimiterType.LeftParenthesis), "f6d9e32c-49ee-436c-ba97-fcdd527d9fdb"}, { Terminator.IdentifierTerminator, "77dacef1-16f6-4ede-93f5-3f7c1a45da85"}, { new Terminator(KeywordType.Not), "8763ccfb-4769-4111-8f72-0408293e631a"}, { new Terminator(OperatorType.Minus), "6abdc924-448f-4e57-8b91-afb7c5e8026c"},}, new Dictionary<Terminator, ReduceInformation>{ }, "d6624c6b-a187-4803-84fe-c3a8f6e434a2") },
		{ "19067c18-27f1-4ad5-90c4-194d67b7315e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "9b2e3c03-55f8-4f54-982a-c194b56fc520"}, { Terminator.NumberTerminator, "2a5a634a-a406-4e54-a148-6378d751f2c4"}, { new NonTerminator(NonTerminatorType.Variable), "bfc1ac1f-9a68-43c1-a75d-612f181a96d2"}, { new Terminator(DelimiterType.LeftParenthesis), "f6d9e32c-49ee-436c-ba97-fcdd527d9fdb"}, { Terminator.IdentifierTerminator, "77dacef1-16f6-4ede-93f5-3f7c1a45da85"}, { new Terminator(KeywordType.Not), "8763ccfb-4769-4111-8f72-0408293e631a"}, { new Terminator(OperatorType.Minus), "6abdc924-448f-4e57-8b91-afb7c5e8026c"},}, new Dictionary<Terminator, ReduceInformation>{ }, "19067c18-27f1-4ad5-90c4-194d67b7315e") },
		{ "dfaee9e5-df35-4f62-b3c7-f89c72915cbf", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "0e0b0cb6-1264-4626-9ab2-005bb8467729"},}, new Dictionary<Terminator, ReduceInformation>{ }, "dfaee9e5-df35-4f62-b3c7-f89c72915cbf") },
		{ "da7faa6b-7d3d-468d-af84-308d59ef8647", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "050d2b73-fc59-43e9-b125-db50144f7fdb"}, { new NonTerminator(NonTerminatorType.Expression), "a582ab7f-9aa1-4130-9df3-2ba9dd0fe6f3"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "c3098ddd-d7d7-4d8b-b9ab-1ebedac5467f"}, { new NonTerminator(NonTerminatorType.Term), "c83de3f5-4130-4b2e-8715-e72b2e35275e"}, { new NonTerminator(NonTerminatorType.Factor), "9c184da0-a0b4-4ae3-9b1e-93b0fe09fbf1"}, { Terminator.NumberTerminator, "eaf4d875-7318-4f6e-b25b-829d30a888b8"}, { new NonTerminator(NonTerminatorType.Variable), "eb827526-1aa2-47ab-8b01-b1b0792e4297"}, { new Terminator(DelimiterType.LeftParenthesis), "2894d0f9-3faf-4be5-b2b6-c499611fc58d"}, { Terminator.IdentifierTerminator, "8efce527-7ba0-435f-a58f-b048a7a0e505"}, { new Terminator(KeywordType.Not), "dd22d779-6929-400f-832a-6d4e7eda69e6"}, { new Terminator(OperatorType.Minus), "28fb603c-e1e9-4bfa-ab9b-f8887e1996c0"},}, new Dictionary<Terminator, ReduceInformation>{ }, "da7faa6b-7d3d-468d-af84-308d59ef8647") },
		{ "e2457739-4ca7-49ec-b8d8-8632118b69c3", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, }, "e2457739-4ca7-49ec-b8d8-8632118b69c3") },
		{ "fc61abfc-0581-472d-9982-39b508a944be", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "7eadc03c-4e1f-45cd-92d0-0744895f10c0"}, { new NonTerminator(NonTerminatorType.Expression), "10204ac0-c523-4b6c-890d-d508a8e83f48"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "72d6e812-1720-408a-ac1c-5ac20fa267d1"}, { new NonTerminator(NonTerminatorType.Term), "0b3b44d2-8458-48d1-b7c4-2329d479b56c"}, { new NonTerminator(NonTerminatorType.Factor), "2f595e0e-c29e-494d-a5bb-a570db3254fd"}, { Terminator.NumberTerminator, "c86f637c-4e17-4d3d-bfd7-f1fc5abdf7f4"}, { new NonTerminator(NonTerminatorType.Variable), "fa555a17-89d0-4cf7-989a-0b0775ade6d4"}, { new Terminator(DelimiterType.LeftParenthesis), "8fab798a-03a8-45b7-8380-7b7eaee651fa"}, { Terminator.IdentifierTerminator, "513b9c84-fc68-4334-976d-7547b1ec521f"}, { new Terminator(KeywordType.Not), "928a980c-85a8-4cf5-a5d1-117516baef4e"}, { new Terminator(OperatorType.Minus), "e2c085de-e99b-4f3a-b869-c3ca5f5dcf75"},}, new Dictionary<Terminator, ReduceInformation>{ }, "fc61abfc-0581-472d-9982-39b508a944be") },
		{ "de72cbd1-d6eb-4f5e-92d1-14feb980cc14", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "de72cbd1-d6eb-4f5e-92d1-14feb980cc14") },
		{ "2fa813ed-0199-4b84-81ce-c5ff2a0e42db", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "2fa813ed-0199-4b84-81ce-c5ff2a0e42db") },
		{ "102faf96-c009-4341-84ed-b1dd5106bdaf", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, }, "102faf96-c009-4341-84ed-b1dd5106bdaf") },
		{ "efebc401-8ed3-4811-b223-54872f5fde26", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "efebc401-8ed3-4811-b223-54872f5fde26") },
		{ "216a7454-691e-41bb-9366-39c9eab4bfb9", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(KeywordType.Do), "01eb6188-12ee-4124-a204-d92ecfb9e4aa"},}, new Dictionary<Terminator, ReduceInformation>{ }, "216a7454-691e-41bb-9366-39c9eab4bfb9") },
		{ "7326cbb1-45a6-48f7-9156-959b6c65b1ed", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.RelationOperator), "bc7f7ece-a178-4b2b-9729-c99f2713fce9"}, { new Terminator(OperatorType.Equal), "6112dd54-ce34-4dd2-be4c-b06dd725bbd8"}, { new Terminator(OperatorType.NotEqual), "1338afe1-da9e-4380-8a28-7f2d82ef2d68"}, { new Terminator(OperatorType.Less), "fc9b446b-c4b5-4720-bc46-49344508202f"}, { new Terminator(OperatorType.LessEqual), "39ac36f9-efe7-4a1b-ba37-a9729fbf341a"}, { new Terminator(OperatorType.Greater), "fc85ac1b-5a0f-405f-a8f0-a1e20d44351d"}, { new Terminator(OperatorType.GreaterEqual), "f0598e66-3db3-44bd-8b9d-cd6c038df056"}, { new NonTerminator(NonTerminatorType.AddOperator), "731915c5-cedc-4584-900a-f6cb24f258e8"}, { new Terminator(OperatorType.Plus), "493d6d76-ed65-4308-aee5-65c5fe502546"}, { new Terminator(OperatorType.Minus), "c41b579b-0a72-411a-b9ed-f3ca30f5a615"}, { new Terminator(KeywordType.Or), "22878828-eed7-470f-a07f-595bb2eb7e2a"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Expression))}, }, "7326cbb1-45a6-48f7-9156-959b6c65b1ed") },
		{ "e9a83490-e674-4a70-a544-af4b50adfbb5", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "c325fa55-e1c2-4ada-bcc9-b8798a951c4e"}, { new Terminator(OperatorType.Multiply), "fd20e81d-52b7-4920-adab-e2a1e49a6b6a"}, { new Terminator(OperatorType.Divide), "ef7280a1-85cf-4590-ab87-68bdd867d9e5"}, { new Terminator(KeywordType.Divide), "9192ebb2-aa8a-414d-ac9e-7206883933e6"}, { new Terminator(KeywordType.Mod), "ae32e061-59db-49fd-addd-2d5c3e9014a3"}, { new Terminator(KeywordType.And), "5c4b371b-c686-4545-b641-4c196ca6b25f"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "e9a83490-e674-4a70-a544-af4b50adfbb5") },
		{ "dcaea83e-fb38-455b-b282-8c83dfe32e9c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, }, "dcaea83e-fb38-455b-b282-8c83dfe32e9c") },
		{ "57ef29e2-0bea-430d-9c00-5c6811158257", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "57ef29e2-0bea-430d-9c00-5c6811158257") },
		{ "6cc38689-9575-499b-8718-f71a97c1c393", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "6cc38689-9575-499b-8718-f71a97c1c393") },
		{ "99cef7cc-a6db-4888-bd5c-3276c85ed9e0", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "d29e287c-48d9-44a7-be47-ea7424c0cb27"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "38eec331-d4ae-4b06-a659-98475fdb5aa0"}, { new NonTerminator(NonTerminatorType.Term), "5a5a9a53-cfe3-4e81-8fcf-2e6d11b16b0e"}, { new NonTerminator(NonTerminatorType.Factor), "175e095a-0963-472f-b84a-ece518bd2cf2"}, { Terminator.NumberTerminator, "3861f05c-791c-4f64-b291-d985540c7651"}, { new NonTerminator(NonTerminatorType.Variable), "c693eb95-cdcd-4c7d-8717-3ead45ef8bfc"}, { new Terminator(DelimiterType.LeftParenthesis), "efc9be5f-b54d-4b96-8aec-f3540a407dc8"}, { Terminator.IdentifierTerminator, "a8ce470f-1537-428f-88c2-9270bfed81b6"}, { new Terminator(KeywordType.Not), "addb6420-8367-48a9-9f28-327cf2a88028"}, { new Terminator(OperatorType.Minus), "75101aaf-bd06-4db5-809c-9e8b4533dc5d"},}, new Dictionary<Terminator, ReduceInformation>{ }, "99cef7cc-a6db-4888-bd5c-3276c85ed9e0") },
		{ "eb0a685b-db40-4c1f-b8d2-c41f7cf7103f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.LeftParenthesis), "c5b9749a-715a-45a7-b2bf-c060d54cf75b"}, { new NonTerminator(NonTerminatorType.IdVarPart), "55788a9d-437f-4a4d-b7d0-7c05bb1c5639"}, { new Terminator(DelimiterType.LeftSquareBracket), "c48678ac-5722-4fc2-aec2-77eb767ef6c4"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Equal), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Less), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Greater), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "eb0a685b-db40-4c1f-b8d2-c41f7cf7103f") },
		{ "fd0bf9cf-2e29-4301-82ea-99b47903069c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "2bc5b1e2-ae10-414a-87f4-8125b6956984"}, { Terminator.NumberTerminator, "57ef29e2-0bea-430d-9c00-5c6811158257"}, { new NonTerminator(NonTerminatorType.Variable), "6cc38689-9575-499b-8718-f71a97c1c393"}, { new Terminator(DelimiterType.LeftParenthesis), "99cef7cc-a6db-4888-bd5c-3276c85ed9e0"}, { Terminator.IdentifierTerminator, "eb0a685b-db40-4c1f-b8d2-c41f7cf7103f"}, { new Terminator(KeywordType.Not), "fd0bf9cf-2e29-4301-82ea-99b47903069c"}, { new Terminator(OperatorType.Minus), "cc266588-7a02-4ee7-9409-719f30519c17"},}, new Dictionary<Terminator, ReduceInformation>{ }, "fd0bf9cf-2e29-4301-82ea-99b47903069c") },
		{ "cc266588-7a02-4ee7-9409-719f30519c17", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "5e28ad03-1a4b-472d-9d9a-5578a5cfa0ea"}, { Terminator.NumberTerminator, "57ef29e2-0bea-430d-9c00-5c6811158257"}, { new NonTerminator(NonTerminatorType.Variable), "6cc38689-9575-499b-8718-f71a97c1c393"}, { new Terminator(DelimiterType.LeftParenthesis), "99cef7cc-a6db-4888-bd5c-3276c85ed9e0"}, { Terminator.IdentifierTerminator, "eb0a685b-db40-4c1f-b8d2-c41f7cf7103f"}, { new Terminator(KeywordType.Not), "fd0bf9cf-2e29-4301-82ea-99b47903069c"}, { new Terminator(OperatorType.Minus), "cc266588-7a02-4ee7-9409-719f30519c17"},}, new Dictionary<Terminator, ReduceInformation>{ }, "cc266588-7a02-4ee7-9409-719f30519c17") },
		{ "3a0f9bf3-2011-42ce-82a2-31701fcd65dc", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.AddOperator), "d7ad8074-ab1f-4b10-80eb-41abf3e42ccd"}, { new Terminator(OperatorType.Plus), "493d6d76-ed65-4308-aee5-65c5fe502546"}, { new Terminator(OperatorType.Minus), "c41b579b-0a72-411a-b9ed-f3ca30f5a615"}, { new Terminator(KeywordType.Or), "22878828-eed7-470f-a07f-595bb2eb7e2a"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Expression))}, }, "3a0f9bf3-2011-42ce-82a2-31701fcd65dc") },
		{ "597d7e8b-4d0f-49c4-a707-7e0b7a4bf952", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "0a3d4a4c-d682-4f28-be2a-4ad8950d86c4"}, { new Terminator(OperatorType.Multiply), "fd20e81d-52b7-4920-adab-e2a1e49a6b6a"}, { new Terminator(OperatorType.Divide), "ef7280a1-85cf-4590-ab87-68bdd867d9e5"}, { new Terminator(KeywordType.Divide), "9192ebb2-aa8a-414d-ac9e-7206883933e6"}, { new Terminator(KeywordType.Mod), "ae32e061-59db-49fd-addd-2d5c3e9014a3"}, { new Terminator(KeywordType.And), "5c4b371b-c686-4545-b641-4c196ca6b25f"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "597d7e8b-4d0f-49c4-a707-7e0b7a4bf952") },
		{ "11b69276-2a19-4474-87e1-22741146fc72", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, }, "11b69276-2a19-4474-87e1-22741146fc72") },
		{ "219b97c4-d730-4e29-8186-27753ab58d87", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "219b97c4-d730-4e29-8186-27753ab58d87") },
		{ "1c864166-5a98-4b66-8dd4-3aadfa236783", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "1c864166-5a98-4b66-8dd4-3aadfa236783") },
		{ "1d68382c-7171-4b2b-be37-e904cbe63bff", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "44fdaf11-bd64-4dda-9d18-155fd80b568e"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "38eec331-d4ae-4b06-a659-98475fdb5aa0"}, { new NonTerminator(NonTerminatorType.Term), "5a5a9a53-cfe3-4e81-8fcf-2e6d11b16b0e"}, { new NonTerminator(NonTerminatorType.Factor), "175e095a-0963-472f-b84a-ece518bd2cf2"}, { Terminator.NumberTerminator, "3861f05c-791c-4f64-b291-d985540c7651"}, { new NonTerminator(NonTerminatorType.Variable), "c693eb95-cdcd-4c7d-8717-3ead45ef8bfc"}, { new Terminator(DelimiterType.LeftParenthesis), "efc9be5f-b54d-4b96-8aec-f3540a407dc8"}, { Terminator.IdentifierTerminator, "a8ce470f-1537-428f-88c2-9270bfed81b6"}, { new Terminator(KeywordType.Not), "addb6420-8367-48a9-9f28-327cf2a88028"}, { new Terminator(OperatorType.Minus), "75101aaf-bd06-4db5-809c-9e8b4533dc5d"},}, new Dictionary<Terminator, ReduceInformation>{ }, "1d68382c-7171-4b2b-be37-e904cbe63bff") },
		{ "d8d12ab8-c78b-4258-921c-d19ad4d59688", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.LeftParenthesis), "754761e1-996f-4a5d-98ce-668544af0d38"}, { new NonTerminator(NonTerminatorType.IdVarPart), "057e727e-44b1-465a-977a-13baba7acf0a"}, { new Terminator(DelimiterType.LeftSquareBracket), "279a2d19-a118-4885-a08a-c1c313fd9b9a"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "d8d12ab8-c78b-4258-921c-d19ad4d59688") },
		{ "1945f333-698c-484f-affd-a1b5f144b742", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "cb262415-063b-4fd0-8d44-bc2f726032a2"}, { Terminator.NumberTerminator, "219b97c4-d730-4e29-8186-27753ab58d87"}, { new NonTerminator(NonTerminatorType.Variable), "1c864166-5a98-4b66-8dd4-3aadfa236783"}, { new Terminator(DelimiterType.LeftParenthesis), "1d68382c-7171-4b2b-be37-e904cbe63bff"}, { Terminator.IdentifierTerminator, "d8d12ab8-c78b-4258-921c-d19ad4d59688"}, { new Terminator(KeywordType.Not), "1945f333-698c-484f-affd-a1b5f144b742"}, { new Terminator(OperatorType.Minus), "0d1c80c4-3fc0-43e4-b249-9a1d66dbfc13"},}, new Dictionary<Terminator, ReduceInformation>{ }, "1945f333-698c-484f-affd-a1b5f144b742") },
		{ "0d1c80c4-3fc0-43e4-b249-9a1d66dbfc13", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "b06939c7-ef13-4ceb-a64f-b5d4599254c1"}, { Terminator.NumberTerminator, "219b97c4-d730-4e29-8186-27753ab58d87"}, { new NonTerminator(NonTerminatorType.Variable), "1c864166-5a98-4b66-8dd4-3aadfa236783"}, { new Terminator(DelimiterType.LeftParenthesis), "1d68382c-7171-4b2b-be37-e904cbe63bff"}, { Terminator.IdentifierTerminator, "d8d12ab8-c78b-4258-921c-d19ad4d59688"}, { new Terminator(KeywordType.Not), "1945f333-698c-484f-affd-a1b5f144b742"}, { new Terminator(OperatorType.Minus), "0d1c80c4-3fc0-43e4-b249-9a1d66dbfc13"},}, new Dictionary<Terminator, ReduceInformation>{ }, "0d1c80c4-3fc0-43e4-b249-9a1d66dbfc13") },
		{ "612bc107-83fa-45a9-a671-b3b00b04fe71", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "89fb02c8-5527-435b-a3fa-14b3751e1700"}, { new Terminator(OperatorType.Multiply), "fd20e81d-52b7-4920-adab-e2a1e49a6b6a"}, { new Terminator(OperatorType.Divide), "ef7280a1-85cf-4590-ab87-68bdd867d9e5"}, { new Terminator(KeywordType.Divide), "9192ebb2-aa8a-414d-ac9e-7206883933e6"}, { new Terminator(KeywordType.Mod), "ae32e061-59db-49fd-addd-2d5c3e9014a3"}, { new Terminator(KeywordType.And), "5c4b371b-c686-4545-b641-4c196ca6b25f"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "612bc107-83fa-45a9-a671-b3b00b04fe71") },
		{ "e8330dcd-539c-4abe-a475-b5eb84e38450", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, }, "e8330dcd-539c-4abe-a475-b5eb84e38450") },
		{ "98cce2c1-b5f2-429e-a264-162f0d3b18e9", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "98cce2c1-b5f2-429e-a264-162f0d3b18e9") },
		{ "60e0dc2d-2aec-4075-9c2e-e3a2405fd701", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "060bbca0-391f-4d82-8419-918c938490ef"}, { new Terminator(DelimiterType.Comma), "dd4993c2-1db9-4e5a-a3f2-c4660ec4e365"},}, new Dictionary<Terminator, ReduceInformation>{ }, "60e0dc2d-2aec-4075-9c2e-e3a2405fd701") },
		{ "532c33b5-c2a7-469a-8d7b-6078f9d174d9", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightSquareBracket), "0fdb2eae-5e94-49a8-aa00-9391755dde64"}, { new Terminator(DelimiterType.Comma), "ecf84275-ae5c-4a59-96fb-d2ef13a595a8"},}, new Dictionary<Terminator, ReduceInformation>{ }, "532c33b5-c2a7-469a-8d7b-6078f9d174d9") },
		{ "29415693-948d-465c-86a1-27911f0f9121", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.CompoundStatement))}, }, "29415693-948d-465c-86a1-27911f0f9121") },
		{ "ac94bab2-b166-4b45-bcd8-e4e7e98e265a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(6, new NonTerminator(NonTerminatorType.Type))}, }, "ac94bab2-b166-4b45-bcd8-e4e7e98e265a") },
		{ "daa7fd64-65f5-4123-b762-eaab9c41f961", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { Terminator.NumberTerminator, "c9413d00-e9c4-4f88-b52e-ac7cae9303eb"},}, new Dictionary<Terminator, ReduceInformation>{ }, "daa7fd64-65f5-4123-b762-eaab9c41f961") },
		{ "30447df9-d167-4a40-b94d-3ba35cf1986c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "2cd1bf85-bbe1-47c2-ac10-a540522bdbce"}, { new Terminator(OperatorType.Multiply), "fd20e81d-52b7-4920-adab-e2a1e49a6b6a"}, { new Terminator(OperatorType.Divide), "ef7280a1-85cf-4590-ab87-68bdd867d9e5"}, { new Terminator(KeywordType.Divide), "9192ebb2-aa8a-414d-ac9e-7206883933e6"}, { new Terminator(KeywordType.Mod), "ae32e061-59db-49fd-addd-2d5c3e9014a3"}, { new Terminator(KeywordType.And), "5c4b371b-c686-4545-b641-4c196ca6b25f"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "30447df9-d167-4a40-b94d-3ba35cf1986c") },
		{ "69d21347-4e11-40ef-90e3-4f2e977dfcfc", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, }, "69d21347-4e11-40ef-90e3-4f2e977dfcfc") },
		{ "d105faca-ad26-4f11-9c7f-abaad0977c83", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "d105faca-ad26-4f11-9c7f-abaad0977c83") },
		{ "68d70314-f043-4283-b542-73e3bd19f2b4", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "946dc850-da03-48a1-a44c-c5658760a86c"}, { new Terminator(DelimiterType.Comma), "dd4993c2-1db9-4e5a-a3f2-c4660ec4e365"},}, new Dictionary<Terminator, ReduceInformation>{ }, "68d70314-f043-4283-b542-73e3bd19f2b4") },
		{ "f591e305-ac84-4acb-88f5-2834100db7b0", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightSquareBracket), "af65cc95-cd99-40e4-8f7a-41f699a96b43"}, { new Terminator(DelimiterType.Comma), "ecf84275-ae5c-4a59-96fb-d2ef13a595a8"},}, new Dictionary<Terminator, ReduceInformation>{ }, "f591e305-ac84-4acb-88f5-2834100db7b0") },
		{ "fb7bfd65-3028-4090-99f7-d7db91475e36", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "1fcec038-a6e7-4e28-833e-6bb9424f7ca9"}, { new Terminator(OperatorType.Multiply), "fd20e81d-52b7-4920-adab-e2a1e49a6b6a"}, { new Terminator(OperatorType.Divide), "ef7280a1-85cf-4590-ab87-68bdd867d9e5"}, { new Terminator(KeywordType.Divide), "9192ebb2-aa8a-414d-ac9e-7206883933e6"}, { new Terminator(KeywordType.Mod), "ae32e061-59db-49fd-addd-2d5c3e9014a3"}, { new Terminator(KeywordType.And), "5c4b371b-c686-4545-b641-4c196ca6b25f"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "fb7bfd65-3028-4090-99f7-d7db91475e36") },
		{ "59068e86-8067-49a2-bf02-7229303f6e08", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, }, "59068e86-8067-49a2-bf02-7229303f6e08") },
		{ "e4d06a61-8f33-4483-a9dc-89bb1fcd93b1", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "e4d06a61-8f33-4483-a9dc-89bb1fcd93b1") },
		{ "618b4374-d213-4519-8245-1890c0d2aff4", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "aa1233cc-5ce3-4cbc-bde9-2857d465d2c9"}, { new Terminator(DelimiterType.Comma), "dd4993c2-1db9-4e5a-a3f2-c4660ec4e365"},}, new Dictionary<Terminator, ReduceInformation>{ }, "618b4374-d213-4519-8245-1890c0d2aff4") },
		{ "a7d8221d-33d2-4435-81b3-de1de6acbeeb", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightSquareBracket), "d846d75d-beb7-4388-a36a-3cda1e203777"}, { new Terminator(DelimiterType.Comma), "ecf84275-ae5c-4a59-96fb-d2ef13a595a8"},}, new Dictionary<Terminator, ReduceInformation>{ }, "a7d8221d-33d2-4435-81b3-de1de6acbeeb") },
		{ "f8a846bc-f12d-497e-b272-eba79a788765", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "5cf7b429-f628-4955-90cd-8b9540a53fe4"}, { new Terminator(OperatorType.Multiply), "fd20e81d-52b7-4920-adab-e2a1e49a6b6a"}, { new Terminator(OperatorType.Divide), "ef7280a1-85cf-4590-ab87-68bdd867d9e5"}, { new Terminator(KeywordType.Divide), "9192ebb2-aa8a-414d-ac9e-7206883933e6"}, { new Terminator(KeywordType.Mod), "ae32e061-59db-49fd-addd-2d5c3e9014a3"}, { new Terminator(KeywordType.And), "5c4b371b-c686-4545-b641-4c196ca6b25f"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "f8a846bc-f12d-497e-b272-eba79a788765") },
		{ "c7a65262-29a7-419c-9803-8552d96b9ad0", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, }, "c7a65262-29a7-419c-9803-8552d96b9ad0") },
		{ "414267ba-7e99-45d0-b258-b7c073db3181", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "414267ba-7e99-45d0-b258-b7c073db3181") },
		{ "a1a4df8f-c531-43b6-b550-295b6fb04987", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "3923120a-2aae-4b28-a78f-038fe4e89352"}, { new Terminator(DelimiterType.Comma), "dd4993c2-1db9-4e5a-a3f2-c4660ec4e365"},}, new Dictionary<Terminator, ReduceInformation>{ }, "a1a4df8f-c531-43b6-b550-295b6fb04987") },
		{ "acae3253-6722-469a-9f08-63fe0e2420e6", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightSquareBracket), "eb2c5d8b-46d7-4933-947e-3982ef06902d"}, { new Terminator(DelimiterType.Comma), "ecf84275-ae5c-4a59-96fb-d2ef13a595a8"},}, new Dictionary<Terminator, ReduceInformation>{ }, "acae3253-6722-469a-9f08-63fe0e2420e6") },
		{ "a6adcf5d-6051-451c-9ada-9752d0e76df6", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.SimpleExpression), "72d68187-e090-4632-833e-5962aa92bae4"}, { new NonTerminator(NonTerminatorType.Term), "40cb45fd-6066-497b-a6dc-88b858cfaf78"}, { new NonTerminator(NonTerminatorType.Factor), "5a887a3c-ea4a-4238-8c7b-3e1fa025b193"}, { Terminator.NumberTerminator, "a8e74e04-a7a8-44d3-adde-153d5ec2d981"}, { new NonTerminator(NonTerminatorType.Variable), "af5f6adb-be4a-4da9-95c8-1570da9b9d62"}, { new Terminator(DelimiterType.LeftParenthesis), "9ef9b415-195e-4576-8550-6d8a16424535"}, { Terminator.IdentifierTerminator, "1e4e3cbb-9a31-4ba9-a65f-c77cc827d8b1"}, { new Terminator(KeywordType.Not), "b0304fd8-7e63-4513-ae51-b5830b419e7a"}, { new Terminator(OperatorType.Minus), "fbc6568e-40af-4b10-a5f8-3d935bd46367"},}, new Dictionary<Terminator, ReduceInformation>{ }, "a6adcf5d-6051-451c-9ada-9752d0e76df6") },
		{ "1add5942-f5b1-44ac-beac-fa083456e9c7", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Term), "94e70186-30f0-4aa4-8efb-bf349ed9ea1e"}, { new NonTerminator(NonTerminatorType.Factor), "a697a4fa-5d3d-45c4-a1da-f2783855186e"}, { Terminator.NumberTerminator, "d0368978-0f30-41f5-a0cc-b837ed474739"}, { new NonTerminator(NonTerminatorType.Variable), "72921583-cb29-44a3-a883-d86b10c630e4"}, { new Terminator(DelimiterType.LeftParenthesis), "df70bbe0-47c7-4421-80d2-ad8e2c4dac96"}, { Terminator.IdentifierTerminator, "04758ff4-52e2-493b-a02d-951bd58ede9b"}, { new Terminator(KeywordType.Not), "a50cf991-5964-4e91-a4d2-bdfaebd4d461"}, { new Terminator(OperatorType.Minus), "c31ada0c-d881-4599-a8a5-a079e69b52dd"},}, new Dictionary<Terminator, ReduceInformation>{ }, "1add5942-f5b1-44ac-beac-fa083456e9c7") },
		{ "0bd458b2-525c-4ef6-9304-1ea198e8e266", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "8a3d1676-c2ed-49cb-a248-84c2fee66a24"}, { Terminator.NumberTerminator, "d0368978-0f30-41f5-a0cc-b837ed474739"}, { new NonTerminator(NonTerminatorType.Variable), "72921583-cb29-44a3-a883-d86b10c630e4"}, { new Terminator(DelimiterType.LeftParenthesis), "df70bbe0-47c7-4421-80d2-ad8e2c4dac96"}, { Terminator.IdentifierTerminator, "04758ff4-52e2-493b-a02d-951bd58ede9b"}, { new Terminator(KeywordType.Not), "a50cf991-5964-4e91-a4d2-bdfaebd4d461"}, { new Terminator(OperatorType.Minus), "c31ada0c-d881-4599-a8a5-a079e69b52dd"},}, new Dictionary<Terminator, ReduceInformation>{ }, "0bd458b2-525c-4ef6-9304-1ea198e8e266") },
		{ "ea00e798-b896-41f9-bc20-62e39055b355", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "dc8ac6aa-42be-4411-ad6a-d0aa555ded53"},}, new Dictionary<Terminator, ReduceInformation>{ }, "ea00e798-b896-41f9-bc20-62e39055b355") },
		{ "9925852d-39bb-4046-b44f-875af1075f24", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "f95fdfdc-7526-4d1a-bc1f-cb6da306f25e"}, { new NonTerminator(NonTerminatorType.Expression), "a582ab7f-9aa1-4130-9df3-2ba9dd0fe6f3"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "c3098ddd-d7d7-4d8b-b9ab-1ebedac5467f"}, { new NonTerminator(NonTerminatorType.Term), "c83de3f5-4130-4b2e-8715-e72b2e35275e"}, { new NonTerminator(NonTerminatorType.Factor), "9c184da0-a0b4-4ae3-9b1e-93b0fe09fbf1"}, { Terminator.NumberTerminator, "eaf4d875-7318-4f6e-b25b-829d30a888b8"}, { new NonTerminator(NonTerminatorType.Variable), "eb827526-1aa2-47ab-8b01-b1b0792e4297"}, { new Terminator(DelimiterType.LeftParenthesis), "2894d0f9-3faf-4be5-b2b6-c499611fc58d"}, { Terminator.IdentifierTerminator, "8efce527-7ba0-435f-a58f-b048a7a0e505"}, { new Terminator(KeywordType.Not), "dd22d779-6929-400f-832a-6d4e7eda69e6"}, { new Terminator(OperatorType.Minus), "28fb603c-e1e9-4bfa-ab9b-f8887e1996c0"},}, new Dictionary<Terminator, ReduceInformation>{ }, "9925852d-39bb-4046-b44f-875af1075f24") },
		{ "9b7f2818-a929-4c62-a5c0-975c33244627", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Else), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, }, "9b7f2818-a929-4c62-a5c0-975c33244627") },
		{ "3af5804f-cf1e-44c4-b12a-c2bf9f7163bb", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "4de8cdb6-fdfc-423a-b02d-44958bd1dd4e"}, { new NonTerminator(NonTerminatorType.Expression), "10204ac0-c523-4b6c-890d-d508a8e83f48"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "72d6e812-1720-408a-ac1c-5ac20fa267d1"}, { new NonTerminator(NonTerminatorType.Term), "0b3b44d2-8458-48d1-b7c4-2329d479b56c"}, { new NonTerminator(NonTerminatorType.Factor), "2f595e0e-c29e-494d-a5bb-a570db3254fd"}, { Terminator.NumberTerminator, "c86f637c-4e17-4d3d-bfd7-f1fc5abdf7f4"}, { new NonTerminator(NonTerminatorType.Variable), "fa555a17-89d0-4cf7-989a-0b0775ade6d4"}, { new Terminator(DelimiterType.LeftParenthesis), "8fab798a-03a8-45b7-8380-7b7eaee651fa"}, { Terminator.IdentifierTerminator, "513b9c84-fc68-4334-976d-7547b1ec521f"}, { new Terminator(KeywordType.Not), "928a980c-85a8-4cf5-a5d1-117516baef4e"}, { new Terminator(OperatorType.Minus), "e2c085de-e99b-4f3a-b869-c3ca5f5dcf75"},}, new Dictionary<Terminator, ReduceInformation>{ }, "3af5804f-cf1e-44c4-b12a-c2bf9f7163bb") },
		{ "dab188a3-9b20-4cd6-80d2-db2a11f96e25", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Else), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "dab188a3-9b20-4cd6-80d2-db2a11f96e25") },
		{ "49288d3e-6f3e-4bfc-9f9f-43383b389e9f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Else), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "49288d3e-6f3e-4bfc-9f9f-43383b389e9f") },
		{ "de46e3c3-3646-4245-99fc-72ace4bfa93a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(4, new NonTerminator(NonTerminatorType.ProcedureCall))}, { new Terminator(KeywordType.Else), new ReduceInformation(4, new NonTerminator(NonTerminatorType.ProcedureCall))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(4, new NonTerminator(NonTerminatorType.ProcedureCall))}, }, "de46e3c3-3646-4245-99fc-72ace4bfa93a") },
		{ "8b510cf2-2937-4d1a-875a-680521b03a9d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ElsePart), "851fc634-ebb9-4890-8c2d-dc75458d2947"}, { new Terminator(KeywordType.Else), "8c62f67f-f513-4b2b-b37f-8d9fb2a961eb"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(0, new NonTerminator(NonTerminatorType.ElsePart))}, { new Terminator(KeywordType.Else), new ReduceInformation(0, new NonTerminator(NonTerminatorType.ElsePart))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(0, new NonTerminator(NonTerminatorType.ElsePart))}, }, "8b510cf2-2937-4d1a-875a-680521b03a9d") },
		{ "56f110c6-1e8e-4db7-8564-8dc0fc7022ab", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(KeywordType.To), "6f9bf1f9-e24f-424c-8925-c5ca9b85eb61"},}, new Dictionary<Terminator, ReduceInformation>{ }, "56f110c6-1e8e-4db7-8564-8dc0fc7022ab") },
		{ "a6046e3c-f247-4f18-b9a0-bc7caed88cef", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, }, "a6046e3c-f247-4f18-b9a0-bc7caed88cef") },
		{ "0cd31d31-3e96-4a8e-9087-a42f503dd13e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "0cd31d31-3e96-4a8e-9087-a42f503dd13e") },
		{ "62d7d2f7-c815-4d03-b2e5-b6cb7d812709", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "19067c18-27f1-4ad5-90c4-194d67b7315e"}, { new Terminator(OperatorType.Multiply), "fd20e81d-52b7-4920-adab-e2a1e49a6b6a"}, { new Terminator(OperatorType.Divide), "ef7280a1-85cf-4590-ab87-68bdd867d9e5"}, { new Terminator(KeywordType.Divide), "9192ebb2-aa8a-414d-ac9e-7206883933e6"}, { new Terminator(KeywordType.Mod), "ae32e061-59db-49fd-addd-2d5c3e9014a3"}, { new Terminator(KeywordType.And), "5c4b371b-c686-4545-b641-4c196ca6b25f"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "62d7d2f7-c815-4d03-b2e5-b6cb7d812709") },
		{ "9b2e3c03-55f8-4f54-982a-c194b56fc520", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, }, "9b2e3c03-55f8-4f54-982a-c194b56fc520") },
		{ "0e0b0cb6-1264-4626-9ab2-005bb8467729", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "0e0b0cb6-1264-4626-9ab2-005bb8467729") },
		{ "050d2b73-fc59-43e9-b125-db50144f7fdb", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "16b4dc24-c6b7-4e02-89b3-9e22c0389d60"}, { new Terminator(DelimiterType.Comma), "dd4993c2-1db9-4e5a-a3f2-c4660ec4e365"},}, new Dictionary<Terminator, ReduceInformation>{ }, "050d2b73-fc59-43e9-b125-db50144f7fdb") },
		{ "7eadc03c-4e1f-45cd-92d0-0744895f10c0", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightSquareBracket), "ccf4dead-7d04-49df-8a6f-1b9fe8a69894"}, { new Terminator(DelimiterType.Comma), "ecf84275-ae5c-4a59-96fb-d2ef13a595a8"},}, new Dictionary<Terminator, ReduceInformation>{ }, "7eadc03c-4e1f-45cd-92d0-0744895f10c0") },
		{ "01eb6188-12ee-4124-a204-d92ecfb9e4aa", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Statement), "d2267cff-d4a1-4050-aad8-98ae11f9086a"}, { new NonTerminator(NonTerminatorType.Variable), "ccb9ddb8-4ca9-4498-906e-81b654135662"}, { Terminator.IdentifierTerminator, "764301e0-1e52-4941-80fc-f768fabb848a"}, { new NonTerminator(NonTerminatorType.ProcedureCall), "d19e5b10-7ef3-4981-8c0b-deca4bfff866"}, { new NonTerminator(NonTerminatorType.CompoundStatement), "67b6f07f-8024-4a8f-8f06-3e29ee8141d8"}, { new Terminator(KeywordType.If), "1139d083-a185-4a24-a86e-2dfa3d0514e2"}, { new Terminator(KeywordType.For), "8337e624-bbce-4b9a-8231-b6b264eefd7f"}, { new Terminator(KeywordType.Begin), "5c23381d-aee0-4773-ab73-20aa2a344931"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, }, "01eb6188-12ee-4124-a204-d92ecfb9e4aa") },
		{ "bc7f7ece-a178-4b2b-9729-c99f2713fce9", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.SimpleExpression), "0302a68b-1aa0-4311-908e-50c01716524a"}, { new NonTerminator(NonTerminatorType.Term), "31a0c898-94b2-4a1f-9057-dd33c9e8b348"}, { new NonTerminator(NonTerminatorType.Factor), "37bd1654-8a38-4f2b-8226-1d53978d984f"}, { Terminator.NumberTerminator, "d67e3da2-14db-461f-adf6-2a759c54c2a4"}, { new NonTerminator(NonTerminatorType.Variable), "3dd275fe-d8e6-4845-a991-b88b0815f1a3"}, { new Terminator(DelimiterType.LeftParenthesis), "a3bd2d83-bb0a-4e75-a1f8-832a87446417"}, { Terminator.IdentifierTerminator, "e0faf7c3-b2f6-4bb4-aaa0-6c9848949555"}, { new Terminator(KeywordType.Not), "1cd0f7bc-43f3-4895-9214-37758c95de66"}, { new Terminator(OperatorType.Minus), "88502125-f1fb-44ad-994f-9d7283ecda6d"},}, new Dictionary<Terminator, ReduceInformation>{ }, "bc7f7ece-a178-4b2b-9729-c99f2713fce9") },
		{ "731915c5-cedc-4584-900a-f6cb24f258e8", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Term), "2cb4b9b5-d2bf-487c-9458-1e8fb410d5bb"}, { new NonTerminator(NonTerminatorType.Factor), "dcaea83e-fb38-455b-b282-8c83dfe32e9c"}, { Terminator.NumberTerminator, "57ef29e2-0bea-430d-9c00-5c6811158257"}, { new NonTerminator(NonTerminatorType.Variable), "6cc38689-9575-499b-8718-f71a97c1c393"}, { new Terminator(DelimiterType.LeftParenthesis), "99cef7cc-a6db-4888-bd5c-3276c85ed9e0"}, { Terminator.IdentifierTerminator, "eb0a685b-db40-4c1f-b8d2-c41f7cf7103f"}, { new Terminator(KeywordType.Not), "fd0bf9cf-2e29-4301-82ea-99b47903069c"}, { new Terminator(OperatorType.Minus), "cc266588-7a02-4ee7-9409-719f30519c17"},}, new Dictionary<Terminator, ReduceInformation>{ }, "731915c5-cedc-4584-900a-f6cb24f258e8") },
		{ "c325fa55-e1c2-4ada-bcc9-b8798a951c4e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "c294bbf4-bc04-4650-b1c5-62faf1a08986"}, { Terminator.NumberTerminator, "57ef29e2-0bea-430d-9c00-5c6811158257"}, { new NonTerminator(NonTerminatorType.Variable), "6cc38689-9575-499b-8718-f71a97c1c393"}, { new Terminator(DelimiterType.LeftParenthesis), "99cef7cc-a6db-4888-bd5c-3276c85ed9e0"}, { Terminator.IdentifierTerminator, "eb0a685b-db40-4c1f-b8d2-c41f7cf7103f"}, { new Terminator(KeywordType.Not), "fd0bf9cf-2e29-4301-82ea-99b47903069c"}, { new Terminator(OperatorType.Minus), "cc266588-7a02-4ee7-9409-719f30519c17"},}, new Dictionary<Terminator, ReduceInformation>{ }, "c325fa55-e1c2-4ada-bcc9-b8798a951c4e") },
		{ "d29e287c-48d9-44a7-be47-ea7424c0cb27", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "a15bd7c9-8d1a-45cf-8097-05aec14a13c4"},}, new Dictionary<Terminator, ReduceInformation>{ }, "d29e287c-48d9-44a7-be47-ea7424c0cb27") },
		{ "c5b9749a-715a-45a7-b2bf-c060d54cf75b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "202f0d08-a208-4a7c-b64f-4e51d7796c2f"}, { new NonTerminator(NonTerminatorType.Expression), "a582ab7f-9aa1-4130-9df3-2ba9dd0fe6f3"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "c3098ddd-d7d7-4d8b-b9ab-1ebedac5467f"}, { new NonTerminator(NonTerminatorType.Term), "c83de3f5-4130-4b2e-8715-e72b2e35275e"}, { new NonTerminator(NonTerminatorType.Factor), "9c184da0-a0b4-4ae3-9b1e-93b0fe09fbf1"}, { Terminator.NumberTerminator, "eaf4d875-7318-4f6e-b25b-829d30a888b8"}, { new NonTerminator(NonTerminatorType.Variable), "eb827526-1aa2-47ab-8b01-b1b0792e4297"}, { new Terminator(DelimiterType.LeftParenthesis), "2894d0f9-3faf-4be5-b2b6-c499611fc58d"}, { Terminator.IdentifierTerminator, "8efce527-7ba0-435f-a58f-b048a7a0e505"}, { new Terminator(KeywordType.Not), "dd22d779-6929-400f-832a-6d4e7eda69e6"}, { new Terminator(OperatorType.Minus), "28fb603c-e1e9-4bfa-ab9b-f8887e1996c0"},}, new Dictionary<Terminator, ReduceInformation>{ }, "c5b9749a-715a-45a7-b2bf-c060d54cf75b") },
		{ "55788a9d-437f-4a4d-b7d0-7c05bb1c5639", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, }, "55788a9d-437f-4a4d-b7d0-7c05bb1c5639") },
		{ "c48678ac-5722-4fc2-aec2-77eb767ef6c4", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "0fa6a9d5-6685-4839-949d-68c9fedb645b"}, { new NonTerminator(NonTerminatorType.Expression), "10204ac0-c523-4b6c-890d-d508a8e83f48"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "72d6e812-1720-408a-ac1c-5ac20fa267d1"}, { new NonTerminator(NonTerminatorType.Term), "0b3b44d2-8458-48d1-b7c4-2329d479b56c"}, { new NonTerminator(NonTerminatorType.Factor), "2f595e0e-c29e-494d-a5bb-a570db3254fd"}, { Terminator.NumberTerminator, "c86f637c-4e17-4d3d-bfd7-f1fc5abdf7f4"}, { new NonTerminator(NonTerminatorType.Variable), "fa555a17-89d0-4cf7-989a-0b0775ade6d4"}, { new Terminator(DelimiterType.LeftParenthesis), "8fab798a-03a8-45b7-8380-7b7eaee651fa"}, { Terminator.IdentifierTerminator, "513b9c84-fc68-4334-976d-7547b1ec521f"}, { new Terminator(KeywordType.Not), "928a980c-85a8-4cf5-a5d1-117516baef4e"}, { new Terminator(OperatorType.Minus), "e2c085de-e99b-4f3a-b869-c3ca5f5dcf75"},}, new Dictionary<Terminator, ReduceInformation>{ }, "c48678ac-5722-4fc2-aec2-77eb767ef6c4") },
		{ "2bc5b1e2-ae10-414a-87f4-8125b6956984", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "2bc5b1e2-ae10-414a-87f4-8125b6956984") },
		{ "5e28ad03-1a4b-472d-9d9a-5578a5cfa0ea", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "5e28ad03-1a4b-472d-9d9a-5578a5cfa0ea") },
		{ "d7ad8074-ab1f-4b10-80eb-41abf3e42ccd", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Term), "e19d4828-b223-4a5c-a847-2f87ecf5bfb8"}, { new NonTerminator(NonTerminatorType.Factor), "11b69276-2a19-4474-87e1-22741146fc72"}, { Terminator.NumberTerminator, "219b97c4-d730-4e29-8186-27753ab58d87"}, { new NonTerminator(NonTerminatorType.Variable), "1c864166-5a98-4b66-8dd4-3aadfa236783"}, { new Terminator(DelimiterType.LeftParenthesis), "1d68382c-7171-4b2b-be37-e904cbe63bff"}, { Terminator.IdentifierTerminator, "d8d12ab8-c78b-4258-921c-d19ad4d59688"}, { new Terminator(KeywordType.Not), "1945f333-698c-484f-affd-a1b5f144b742"}, { new Terminator(OperatorType.Minus), "0d1c80c4-3fc0-43e4-b249-9a1d66dbfc13"},}, new Dictionary<Terminator, ReduceInformation>{ }, "d7ad8074-ab1f-4b10-80eb-41abf3e42ccd") },
		{ "0a3d4a4c-d682-4f28-be2a-4ad8950d86c4", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "1b0b11fb-f2fe-4d37-ad02-326e20e03255"}, { Terminator.NumberTerminator, "219b97c4-d730-4e29-8186-27753ab58d87"}, { new NonTerminator(NonTerminatorType.Variable), "1c864166-5a98-4b66-8dd4-3aadfa236783"}, { new Terminator(DelimiterType.LeftParenthesis), "1d68382c-7171-4b2b-be37-e904cbe63bff"}, { Terminator.IdentifierTerminator, "d8d12ab8-c78b-4258-921c-d19ad4d59688"}, { new Terminator(KeywordType.Not), "1945f333-698c-484f-affd-a1b5f144b742"}, { new Terminator(OperatorType.Minus), "0d1c80c4-3fc0-43e4-b249-9a1d66dbfc13"},}, new Dictionary<Terminator, ReduceInformation>{ }, "0a3d4a4c-d682-4f28-be2a-4ad8950d86c4") },
		{ "44fdaf11-bd64-4dda-9d18-155fd80b568e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "6c3a6d1e-36ea-4bfc-a177-6dac9cc89dd0"},}, new Dictionary<Terminator, ReduceInformation>{ }, "44fdaf11-bd64-4dda-9d18-155fd80b568e") },
		{ "754761e1-996f-4a5d-98ce-668544af0d38", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "9f5a51ae-380d-4699-ab42-c4d62a0650c8"}, { new NonTerminator(NonTerminatorType.Expression), "a582ab7f-9aa1-4130-9df3-2ba9dd0fe6f3"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "c3098ddd-d7d7-4d8b-b9ab-1ebedac5467f"}, { new NonTerminator(NonTerminatorType.Term), "c83de3f5-4130-4b2e-8715-e72b2e35275e"}, { new NonTerminator(NonTerminatorType.Factor), "9c184da0-a0b4-4ae3-9b1e-93b0fe09fbf1"}, { Terminator.NumberTerminator, "eaf4d875-7318-4f6e-b25b-829d30a888b8"}, { new NonTerminator(NonTerminatorType.Variable), "eb827526-1aa2-47ab-8b01-b1b0792e4297"}, { new Terminator(DelimiterType.LeftParenthesis), "2894d0f9-3faf-4be5-b2b6-c499611fc58d"}, { Terminator.IdentifierTerminator, "8efce527-7ba0-435f-a58f-b048a7a0e505"}, { new Terminator(KeywordType.Not), "dd22d779-6929-400f-832a-6d4e7eda69e6"}, { new Terminator(OperatorType.Minus), "28fb603c-e1e9-4bfa-ab9b-f8887e1996c0"},}, new Dictionary<Terminator, ReduceInformation>{ }, "754761e1-996f-4a5d-98ce-668544af0d38") },
		{ "057e727e-44b1-465a-977a-13baba7acf0a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, }, "057e727e-44b1-465a-977a-13baba7acf0a") },
		{ "279a2d19-a118-4885-a08a-c1c313fd9b9a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "3c267022-04d1-42de-afeb-f57f1b396035"}, { new NonTerminator(NonTerminatorType.Expression), "10204ac0-c523-4b6c-890d-d508a8e83f48"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "72d6e812-1720-408a-ac1c-5ac20fa267d1"}, { new NonTerminator(NonTerminatorType.Term), "0b3b44d2-8458-48d1-b7c4-2329d479b56c"}, { new NonTerminator(NonTerminatorType.Factor), "2f595e0e-c29e-494d-a5bb-a570db3254fd"}, { Terminator.NumberTerminator, "c86f637c-4e17-4d3d-bfd7-f1fc5abdf7f4"}, { new NonTerminator(NonTerminatorType.Variable), "fa555a17-89d0-4cf7-989a-0b0775ade6d4"}, { new Terminator(DelimiterType.LeftParenthesis), "8fab798a-03a8-45b7-8380-7b7eaee651fa"}, { Terminator.IdentifierTerminator, "513b9c84-fc68-4334-976d-7547b1ec521f"}, { new Terminator(KeywordType.Not), "928a980c-85a8-4cf5-a5d1-117516baef4e"}, { new Terminator(OperatorType.Minus), "e2c085de-e99b-4f3a-b869-c3ca5f5dcf75"},}, new Dictionary<Terminator, ReduceInformation>{ }, "279a2d19-a118-4885-a08a-c1c313fd9b9a") },
		{ "cb262415-063b-4fd0-8d44-bc2f726032a2", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "cb262415-063b-4fd0-8d44-bc2f726032a2") },
		{ "b06939c7-ef13-4ceb-a64f-b5d4599254c1", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "b06939c7-ef13-4ceb-a64f-b5d4599254c1") },
		{ "060bbca0-391f-4d82-8419-918c938490ef", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, }, "060bbca0-391f-4d82-8419-918c938490ef") },
		{ "0fdb2eae-5e94-49a8-aa00-9391755dde64", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "0fdb2eae-5e94-49a8-aa00-9391755dde64") },
		{ "c9413d00-e9c4-4f88-b52e-ac7cae9303eb", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(5, new NonTerminator(NonTerminatorType.Period))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(5, new NonTerminator(NonTerminatorType.Period))}, }, "c9413d00-e9c4-4f88-b52e-ac7cae9303eb") },
		{ "946dc850-da03-48a1-a44c-c5658760a86c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, }, "946dc850-da03-48a1-a44c-c5658760a86c") },
		{ "af65cc95-cd99-40e4-8f7a-41f699a96b43", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "af65cc95-cd99-40e4-8f7a-41f699a96b43") },
		{ "aa1233cc-5ce3-4cbc-bde9-2857d465d2c9", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, }, "aa1233cc-5ce3-4cbc-bde9-2857d465d2c9") },
		{ "d846d75d-beb7-4388-a36a-3cda1e203777", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "d846d75d-beb7-4388-a36a-3cda1e203777") },
		{ "3923120a-2aae-4b28-a78f-038fe4e89352", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, }, "3923120a-2aae-4b28-a78f-038fe4e89352") },
		{ "eb2c5d8b-46d7-4933-947e-3982ef06902d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "eb2c5d8b-46d7-4933-947e-3982ef06902d") },
		{ "72d68187-e090-4632-833e-5962aa92bae4", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.AddOperator), "24658671-f22e-4187-b25c-49ad002cb327"}, { new Terminator(OperatorType.Plus), "493d6d76-ed65-4308-aee5-65c5fe502546"}, { new Terminator(OperatorType.Minus), "c41b579b-0a72-411a-b9ed-f3ca30f5a615"}, { new Terminator(KeywordType.Or), "22878828-eed7-470f-a07f-595bb2eb7e2a"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Expression))}, { new Terminator(KeywordType.Else), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Expression))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Expression))}, }, "72d68187-e090-4632-833e-5962aa92bae4") },
		{ "40cb45fd-6066-497b-a6dc-88b858cfaf78", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "2f86a129-1871-4334-8940-3da8f159d4b9"}, { new Terminator(OperatorType.Multiply), "fd20e81d-52b7-4920-adab-e2a1e49a6b6a"}, { new Terminator(OperatorType.Divide), "ef7280a1-85cf-4590-ab87-68bdd867d9e5"}, { new Terminator(KeywordType.Divide), "9192ebb2-aa8a-414d-ac9e-7206883933e6"}, { new Terminator(KeywordType.Mod), "ae32e061-59db-49fd-addd-2d5c3e9014a3"}, { new Terminator(KeywordType.And), "5c4b371b-c686-4545-b641-4c196ca6b25f"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Else), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "40cb45fd-6066-497b-a6dc-88b858cfaf78") },
		{ "5a887a3c-ea4a-4238-8c7b-3e1fa025b193", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Else), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, }, "5a887a3c-ea4a-4238-8c7b-3e1fa025b193") },
		{ "a8e74e04-a7a8-44d3-adde-153d5ec2d981", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Else), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "a8e74e04-a7a8-44d3-adde-153d5ec2d981") },
		{ "af5f6adb-be4a-4da9-95c8-1570da9b9d62", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Else), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "af5f6adb-be4a-4da9-95c8-1570da9b9d62") },
		{ "9ef9b415-195e-4576-8550-6d8a16424535", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "8e2e319c-9284-4b6a-a2cb-dd7d60b311bb"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "38eec331-d4ae-4b06-a659-98475fdb5aa0"}, { new NonTerminator(NonTerminatorType.Term), "5a5a9a53-cfe3-4e81-8fcf-2e6d11b16b0e"}, { new NonTerminator(NonTerminatorType.Factor), "175e095a-0963-472f-b84a-ece518bd2cf2"}, { Terminator.NumberTerminator, "3861f05c-791c-4f64-b291-d985540c7651"}, { new NonTerminator(NonTerminatorType.Variable), "c693eb95-cdcd-4c7d-8717-3ead45ef8bfc"}, { new Terminator(DelimiterType.LeftParenthesis), "efc9be5f-b54d-4b96-8aec-f3540a407dc8"}, { Terminator.IdentifierTerminator, "a8ce470f-1537-428f-88c2-9270bfed81b6"}, { new Terminator(KeywordType.Not), "addb6420-8367-48a9-9f28-327cf2a88028"}, { new Terminator(OperatorType.Minus), "75101aaf-bd06-4db5-809c-9e8b4533dc5d"},}, new Dictionary<Terminator, ReduceInformation>{ }, "9ef9b415-195e-4576-8550-6d8a16424535") },
		{ "1e4e3cbb-9a31-4ba9-a65f-c77cc827d8b1", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.LeftParenthesis), "45de687d-e138-4481-88ca-ddcb43bb4a7b"}, { new NonTerminator(NonTerminatorType.IdVarPart), "a94c9301-52a1-4fc5-b4f1-85013f457118"}, { new Terminator(DelimiterType.LeftSquareBracket), "96a56a50-685f-4ee2-a15c-0246a192968f"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Else), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "1e4e3cbb-9a31-4ba9-a65f-c77cc827d8b1") },
		{ "b0304fd8-7e63-4513-ae51-b5830b419e7a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "f7212a1f-121f-44ff-9b3a-8784208708c6"}, { Terminator.NumberTerminator, "a8e74e04-a7a8-44d3-adde-153d5ec2d981"}, { new NonTerminator(NonTerminatorType.Variable), "af5f6adb-be4a-4da9-95c8-1570da9b9d62"}, { new Terminator(DelimiterType.LeftParenthesis), "9ef9b415-195e-4576-8550-6d8a16424535"}, { Terminator.IdentifierTerminator, "1e4e3cbb-9a31-4ba9-a65f-c77cc827d8b1"}, { new Terminator(KeywordType.Not), "b0304fd8-7e63-4513-ae51-b5830b419e7a"}, { new Terminator(OperatorType.Minus), "fbc6568e-40af-4b10-a5f8-3d935bd46367"},}, new Dictionary<Terminator, ReduceInformation>{ }, "b0304fd8-7e63-4513-ae51-b5830b419e7a") },
		{ "fbc6568e-40af-4b10-a5f8-3d935bd46367", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "c25b430d-af68-4f3a-93c6-97f4be6fd6fa"}, { Terminator.NumberTerminator, "a8e74e04-a7a8-44d3-adde-153d5ec2d981"}, { new NonTerminator(NonTerminatorType.Variable), "af5f6adb-be4a-4da9-95c8-1570da9b9d62"}, { new Terminator(DelimiterType.LeftParenthesis), "9ef9b415-195e-4576-8550-6d8a16424535"}, { Terminator.IdentifierTerminator, "1e4e3cbb-9a31-4ba9-a65f-c77cc827d8b1"}, { new Terminator(KeywordType.Not), "b0304fd8-7e63-4513-ae51-b5830b419e7a"}, { new Terminator(OperatorType.Minus), "fbc6568e-40af-4b10-a5f8-3d935bd46367"},}, new Dictionary<Terminator, ReduceInformation>{ }, "fbc6568e-40af-4b10-a5f8-3d935bd46367") },
		{ "94e70186-30f0-4aa4-8efb-bf349ed9ea1e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "0bd458b2-525c-4ef6-9304-1ea198e8e266"}, { new Terminator(OperatorType.Multiply), "fd20e81d-52b7-4920-adab-e2a1e49a6b6a"}, { new Terminator(OperatorType.Divide), "ef7280a1-85cf-4590-ab87-68bdd867d9e5"}, { new Terminator(KeywordType.Divide), "9192ebb2-aa8a-414d-ac9e-7206883933e6"}, { new Terminator(KeywordType.Mod), "ae32e061-59db-49fd-addd-2d5c3e9014a3"}, { new Terminator(KeywordType.And), "5c4b371b-c686-4545-b641-4c196ca6b25f"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Else), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "94e70186-30f0-4aa4-8efb-bf349ed9ea1e") },
		{ "8a3d1676-c2ed-49cb-a248-84c2fee66a24", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Else), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, }, "8a3d1676-c2ed-49cb-a248-84c2fee66a24") },
		{ "dc8ac6aa-42be-4411-ad6a-d0aa555ded53", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Else), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "dc8ac6aa-42be-4411-ad6a-d0aa555ded53") },
		{ "f95fdfdc-7526-4d1a-bc1f-cb6da306f25e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "02a4c650-94e3-411c-8fc1-7193c2459346"}, { new Terminator(DelimiterType.Comma), "dd4993c2-1db9-4e5a-a3f2-c4660ec4e365"},}, new Dictionary<Terminator, ReduceInformation>{ }, "f95fdfdc-7526-4d1a-bc1f-cb6da306f25e") },
		{ "4de8cdb6-fdfc-423a-b02d-44958bd1dd4e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightSquareBracket), "7c2ba50c-06bd-454a-abd5-f78ee0b8ceb5"}, { new Terminator(DelimiterType.Comma), "ecf84275-ae5c-4a59-96fb-d2ef13a595a8"},}, new Dictionary<Terminator, ReduceInformation>{ }, "4de8cdb6-fdfc-423a-b02d-44958bd1dd4e") },
		{ "851fc634-ebb9-4890-8c2d-dc75458d2947", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(5, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(KeywordType.Else), new ReduceInformation(5, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(5, new NonTerminator(NonTerminatorType.Statement))}, }, "851fc634-ebb9-4890-8c2d-dc75458d2947") },
		{ "8c62f67f-f513-4b2b-b37f-8d9fb2a961eb", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Statement), "0296cd89-e186-4c9d-8b7e-1d843177fdc7"}, { new NonTerminator(NonTerminatorType.Variable), "c5d258c5-490b-4c90-8b5b-8242a6dbfc6d"}, { Terminator.IdentifierTerminator, "9a2b8a88-a48e-46e2-af69-f3561ad39bb7"}, { new NonTerminator(NonTerminatorType.ProcedureCall), "e9f86400-0193-40f4-a076-09e7b9b028e9"}, { new NonTerminator(NonTerminatorType.CompoundStatement), "48fe99c9-db82-494e-b751-8bda645590d0"}, { new Terminator(KeywordType.If), "79db3815-bc73-4868-ad20-1fc22d7453b9"}, { new Terminator(KeywordType.For), "ec4898d0-5ba7-47f1-885e-07725f2e43dc"}, { new Terminator(KeywordType.Begin), "65fb1933-38c9-42db-9907-36fe3c05573a"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(KeywordType.Else), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, }, "8c62f67f-f513-4b2b-b37f-8d9fb2a961eb") },
		{ "6f9bf1f9-e24f-424c-8925-c5ca9b85eb61", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "c6e730de-5667-4775-ab15-2f863fec9e1a"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "7326cbb1-45a6-48f7-9156-959b6c65b1ed"}, { new NonTerminator(NonTerminatorType.Term), "e9a83490-e674-4a70-a544-af4b50adfbb5"}, { new NonTerminator(NonTerminatorType.Factor), "dcaea83e-fb38-455b-b282-8c83dfe32e9c"}, { Terminator.NumberTerminator, "57ef29e2-0bea-430d-9c00-5c6811158257"}, { new NonTerminator(NonTerminatorType.Variable), "6cc38689-9575-499b-8718-f71a97c1c393"}, { new Terminator(DelimiterType.LeftParenthesis), "99cef7cc-a6db-4888-bd5c-3276c85ed9e0"}, { Terminator.IdentifierTerminator, "eb0a685b-db40-4c1f-b8d2-c41f7cf7103f"}, { new Terminator(KeywordType.Not), "fd0bf9cf-2e29-4301-82ea-99b47903069c"}, { new Terminator(OperatorType.Minus), "cc266588-7a02-4ee7-9409-719f30519c17"},}, new Dictionary<Terminator, ReduceInformation>{ }, "6f9bf1f9-e24f-424c-8925-c5ca9b85eb61") },
		{ "16b4dc24-c6b7-4e02-89b3-9e22c0389d60", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, }, "16b4dc24-c6b7-4e02-89b3-9e22c0389d60") },
		{ "ccf4dead-7d04-49df-8a6f-1b9fe8a69894", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "ccf4dead-7d04-49df-8a6f-1b9fe8a69894") },
		{ "d2267cff-d4a1-4050-aad8-98ae11f9086a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(8, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(8, new NonTerminator(NonTerminatorType.Statement))}, }, "d2267cff-d4a1-4050-aad8-98ae11f9086a") },
		{ "0302a68b-1aa0-4311-908e-50c01716524a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.AddOperator), "3d7f2f8e-b029-42e3-ad84-d559138bfa13"}, { new Terminator(OperatorType.Plus), "493d6d76-ed65-4308-aee5-65c5fe502546"}, { new Terminator(OperatorType.Minus), "c41b579b-0a72-411a-b9ed-f3ca30f5a615"}, { new Terminator(KeywordType.Or), "22878828-eed7-470f-a07f-595bb2eb7e2a"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Expression))}, }, "0302a68b-1aa0-4311-908e-50c01716524a") },
		{ "31a0c898-94b2-4a1f-9057-dd33c9e8b348", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "f1a01b65-0d45-4b82-b73d-05b38553d94f"}, { new Terminator(OperatorType.Multiply), "fd20e81d-52b7-4920-adab-e2a1e49a6b6a"}, { new Terminator(OperatorType.Divide), "ef7280a1-85cf-4590-ab87-68bdd867d9e5"}, { new Terminator(KeywordType.Divide), "9192ebb2-aa8a-414d-ac9e-7206883933e6"}, { new Terminator(KeywordType.Mod), "ae32e061-59db-49fd-addd-2d5c3e9014a3"}, { new Terminator(KeywordType.And), "5c4b371b-c686-4545-b641-4c196ca6b25f"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "31a0c898-94b2-4a1f-9057-dd33c9e8b348") },
		{ "37bd1654-8a38-4f2b-8226-1d53978d984f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, }, "37bd1654-8a38-4f2b-8226-1d53978d984f") },
		{ "d67e3da2-14db-461f-adf6-2a759c54c2a4", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "d67e3da2-14db-461f-adf6-2a759c54c2a4") },
		{ "3dd275fe-d8e6-4845-a991-b88b0815f1a3", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "3dd275fe-d8e6-4845-a991-b88b0815f1a3") },
		{ "a3bd2d83-bb0a-4e75-a1f8-832a87446417", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "f17107e8-6867-4d9c-8a47-6f724092d214"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "38eec331-d4ae-4b06-a659-98475fdb5aa0"}, { new NonTerminator(NonTerminatorType.Term), "5a5a9a53-cfe3-4e81-8fcf-2e6d11b16b0e"}, { new NonTerminator(NonTerminatorType.Factor), "175e095a-0963-472f-b84a-ece518bd2cf2"}, { Terminator.NumberTerminator, "3861f05c-791c-4f64-b291-d985540c7651"}, { new NonTerminator(NonTerminatorType.Variable), "c693eb95-cdcd-4c7d-8717-3ead45ef8bfc"}, { new Terminator(DelimiterType.LeftParenthesis), "efc9be5f-b54d-4b96-8aec-f3540a407dc8"}, { Terminator.IdentifierTerminator, "a8ce470f-1537-428f-88c2-9270bfed81b6"}, { new Terminator(KeywordType.Not), "addb6420-8367-48a9-9f28-327cf2a88028"}, { new Terminator(OperatorType.Minus), "75101aaf-bd06-4db5-809c-9e8b4533dc5d"},}, new Dictionary<Terminator, ReduceInformation>{ }, "a3bd2d83-bb0a-4e75-a1f8-832a87446417") },
		{ "e0faf7c3-b2f6-4bb4-aaa0-6c9848949555", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.LeftParenthesis), "87a88465-fae6-4eeb-bce5-4124a4da8dca"}, { new NonTerminator(NonTerminatorType.IdVarPart), "d03129cd-2bca-4342-a171-bb2c9cc5f8ca"}, { new Terminator(DelimiterType.LeftSquareBracket), "e5ff139e-e61b-4571-8efc-30a2c0c37c74"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "e0faf7c3-b2f6-4bb4-aaa0-6c9848949555") },
		{ "1cd0f7bc-43f3-4895-9214-37758c95de66", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "8f08beb2-d5cb-430b-95a7-5fb0770e5909"}, { Terminator.NumberTerminator, "d67e3da2-14db-461f-adf6-2a759c54c2a4"}, { new NonTerminator(NonTerminatorType.Variable), "3dd275fe-d8e6-4845-a991-b88b0815f1a3"}, { new Terminator(DelimiterType.LeftParenthesis), "a3bd2d83-bb0a-4e75-a1f8-832a87446417"}, { Terminator.IdentifierTerminator, "e0faf7c3-b2f6-4bb4-aaa0-6c9848949555"}, { new Terminator(KeywordType.Not), "1cd0f7bc-43f3-4895-9214-37758c95de66"}, { new Terminator(OperatorType.Minus), "88502125-f1fb-44ad-994f-9d7283ecda6d"},}, new Dictionary<Terminator, ReduceInformation>{ }, "1cd0f7bc-43f3-4895-9214-37758c95de66") },
		{ "88502125-f1fb-44ad-994f-9d7283ecda6d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "f2c394a9-4994-4129-970d-22bd172092f5"}, { Terminator.NumberTerminator, "d67e3da2-14db-461f-adf6-2a759c54c2a4"}, { new NonTerminator(NonTerminatorType.Variable), "3dd275fe-d8e6-4845-a991-b88b0815f1a3"}, { new Terminator(DelimiterType.LeftParenthesis), "a3bd2d83-bb0a-4e75-a1f8-832a87446417"}, { Terminator.IdentifierTerminator, "e0faf7c3-b2f6-4bb4-aaa0-6c9848949555"}, { new Terminator(KeywordType.Not), "1cd0f7bc-43f3-4895-9214-37758c95de66"}, { new Terminator(OperatorType.Minus), "88502125-f1fb-44ad-994f-9d7283ecda6d"},}, new Dictionary<Terminator, ReduceInformation>{ }, "88502125-f1fb-44ad-994f-9d7283ecda6d") },
		{ "2cb4b9b5-d2bf-487c-9458-1e8fb410d5bb", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "c325fa55-e1c2-4ada-bcc9-b8798a951c4e"}, { new Terminator(OperatorType.Multiply), "fd20e81d-52b7-4920-adab-e2a1e49a6b6a"}, { new Terminator(OperatorType.Divide), "ef7280a1-85cf-4590-ab87-68bdd867d9e5"}, { new Terminator(KeywordType.Divide), "9192ebb2-aa8a-414d-ac9e-7206883933e6"}, { new Terminator(KeywordType.Mod), "ae32e061-59db-49fd-addd-2d5c3e9014a3"}, { new Terminator(KeywordType.And), "5c4b371b-c686-4545-b641-4c196ca6b25f"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "2cb4b9b5-d2bf-487c-9458-1e8fb410d5bb") },
		{ "c294bbf4-bc04-4650-b1c5-62faf1a08986", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, }, "c294bbf4-bc04-4650-b1c5-62faf1a08986") },
		{ "a15bd7c9-8d1a-45cf-8097-05aec14a13c4", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "a15bd7c9-8d1a-45cf-8097-05aec14a13c4") },
		{ "202f0d08-a208-4a7c-b64f-4e51d7796c2f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "d484e87f-98cd-4163-81d3-ca26ad405a37"}, { new Terminator(DelimiterType.Comma), "dd4993c2-1db9-4e5a-a3f2-c4660ec4e365"},}, new Dictionary<Terminator, ReduceInformation>{ }, "202f0d08-a208-4a7c-b64f-4e51d7796c2f") },
		{ "0fa6a9d5-6685-4839-949d-68c9fedb645b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightSquareBracket), "9d7e18b2-f146-4bf3-8f6c-b773391d5bc5"}, { new Terminator(DelimiterType.Comma), "ecf84275-ae5c-4a59-96fb-d2ef13a595a8"},}, new Dictionary<Terminator, ReduceInformation>{ }, "0fa6a9d5-6685-4839-949d-68c9fedb645b") },
		{ "e19d4828-b223-4a5c-a847-2f87ecf5bfb8", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "0a3d4a4c-d682-4f28-be2a-4ad8950d86c4"}, { new Terminator(OperatorType.Multiply), "fd20e81d-52b7-4920-adab-e2a1e49a6b6a"}, { new Terminator(OperatorType.Divide), "ef7280a1-85cf-4590-ab87-68bdd867d9e5"}, { new Terminator(KeywordType.Divide), "9192ebb2-aa8a-414d-ac9e-7206883933e6"}, { new Terminator(KeywordType.Mod), "ae32e061-59db-49fd-addd-2d5c3e9014a3"}, { new Terminator(KeywordType.And), "5c4b371b-c686-4545-b641-4c196ca6b25f"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "e19d4828-b223-4a5c-a847-2f87ecf5bfb8") },
		{ "1b0b11fb-f2fe-4d37-ad02-326e20e03255", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, }, "1b0b11fb-f2fe-4d37-ad02-326e20e03255") },
		{ "6c3a6d1e-36ea-4bfc-a177-6dac9cc89dd0", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "6c3a6d1e-36ea-4bfc-a177-6dac9cc89dd0") },
		{ "9f5a51ae-380d-4699-ab42-c4d62a0650c8", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "7b48ff10-7621-438e-8883-11146ef07997"}, { new Terminator(DelimiterType.Comma), "dd4993c2-1db9-4e5a-a3f2-c4660ec4e365"},}, new Dictionary<Terminator, ReduceInformation>{ }, "9f5a51ae-380d-4699-ab42-c4d62a0650c8") },
		{ "3c267022-04d1-42de-afeb-f57f1b396035", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightSquareBracket), "1e6b291c-de7d-4494-a2a7-569ddc984d86"}, { new Terminator(DelimiterType.Comma), "ecf84275-ae5c-4a59-96fb-d2ef13a595a8"},}, new Dictionary<Terminator, ReduceInformation>{ }, "3c267022-04d1-42de-afeb-f57f1b396035") },
		{ "24658671-f22e-4187-b25c-49ad002cb327", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Term), "509f6d3f-9a85-4d41-96f7-f9b45467598e"}, { new NonTerminator(NonTerminatorType.Factor), "5a887a3c-ea4a-4238-8c7b-3e1fa025b193"}, { Terminator.NumberTerminator, "a8e74e04-a7a8-44d3-adde-153d5ec2d981"}, { new NonTerminator(NonTerminatorType.Variable), "af5f6adb-be4a-4da9-95c8-1570da9b9d62"}, { new Terminator(DelimiterType.LeftParenthesis), "9ef9b415-195e-4576-8550-6d8a16424535"}, { Terminator.IdentifierTerminator, "1e4e3cbb-9a31-4ba9-a65f-c77cc827d8b1"}, { new Terminator(KeywordType.Not), "b0304fd8-7e63-4513-ae51-b5830b419e7a"}, { new Terminator(OperatorType.Minus), "fbc6568e-40af-4b10-a5f8-3d935bd46367"},}, new Dictionary<Terminator, ReduceInformation>{ }, "24658671-f22e-4187-b25c-49ad002cb327") },
		{ "2f86a129-1871-4334-8940-3da8f159d4b9", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "563c7fab-ed60-49a6-bd96-763ac22e12d5"}, { Terminator.NumberTerminator, "a8e74e04-a7a8-44d3-adde-153d5ec2d981"}, { new NonTerminator(NonTerminatorType.Variable), "af5f6adb-be4a-4da9-95c8-1570da9b9d62"}, { new Terminator(DelimiterType.LeftParenthesis), "9ef9b415-195e-4576-8550-6d8a16424535"}, { Terminator.IdentifierTerminator, "1e4e3cbb-9a31-4ba9-a65f-c77cc827d8b1"}, { new Terminator(KeywordType.Not), "b0304fd8-7e63-4513-ae51-b5830b419e7a"}, { new Terminator(OperatorType.Minus), "fbc6568e-40af-4b10-a5f8-3d935bd46367"},}, new Dictionary<Terminator, ReduceInformation>{ }, "2f86a129-1871-4334-8940-3da8f159d4b9") },
		{ "8e2e319c-9284-4b6a-a2cb-dd7d60b311bb", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "79b2f934-f2c7-4006-b3ca-a96dac34640f"},}, new Dictionary<Terminator, ReduceInformation>{ }, "8e2e319c-9284-4b6a-a2cb-dd7d60b311bb") },
		{ "45de687d-e138-4481-88ca-ddcb43bb4a7b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "4cb799fe-5e00-424f-9654-5c6c1759fc1e"}, { new NonTerminator(NonTerminatorType.Expression), "a582ab7f-9aa1-4130-9df3-2ba9dd0fe6f3"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "c3098ddd-d7d7-4d8b-b9ab-1ebedac5467f"}, { new NonTerminator(NonTerminatorType.Term), "c83de3f5-4130-4b2e-8715-e72b2e35275e"}, { new NonTerminator(NonTerminatorType.Factor), "9c184da0-a0b4-4ae3-9b1e-93b0fe09fbf1"}, { Terminator.NumberTerminator, "eaf4d875-7318-4f6e-b25b-829d30a888b8"}, { new NonTerminator(NonTerminatorType.Variable), "eb827526-1aa2-47ab-8b01-b1b0792e4297"}, { new Terminator(DelimiterType.LeftParenthesis), "2894d0f9-3faf-4be5-b2b6-c499611fc58d"}, { Terminator.IdentifierTerminator, "8efce527-7ba0-435f-a58f-b048a7a0e505"}, { new Terminator(KeywordType.Not), "dd22d779-6929-400f-832a-6d4e7eda69e6"}, { new Terminator(OperatorType.Minus), "28fb603c-e1e9-4bfa-ab9b-f8887e1996c0"},}, new Dictionary<Terminator, ReduceInformation>{ }, "45de687d-e138-4481-88ca-ddcb43bb4a7b") },
		{ "a94c9301-52a1-4fc5-b4f1-85013f457118", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Else), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, }, "a94c9301-52a1-4fc5-b4f1-85013f457118") },
		{ "96a56a50-685f-4ee2-a15c-0246a192968f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "cec4bffe-a1da-47ec-b752-83552c0a0949"}, { new NonTerminator(NonTerminatorType.Expression), "10204ac0-c523-4b6c-890d-d508a8e83f48"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "72d6e812-1720-408a-ac1c-5ac20fa267d1"}, { new NonTerminator(NonTerminatorType.Term), "0b3b44d2-8458-48d1-b7c4-2329d479b56c"}, { new NonTerminator(NonTerminatorType.Factor), "2f595e0e-c29e-494d-a5bb-a570db3254fd"}, { Terminator.NumberTerminator, "c86f637c-4e17-4d3d-bfd7-f1fc5abdf7f4"}, { new NonTerminator(NonTerminatorType.Variable), "fa555a17-89d0-4cf7-989a-0b0775ade6d4"}, { new Terminator(DelimiterType.LeftParenthesis), "8fab798a-03a8-45b7-8380-7b7eaee651fa"}, { Terminator.IdentifierTerminator, "513b9c84-fc68-4334-976d-7547b1ec521f"}, { new Terminator(KeywordType.Not), "928a980c-85a8-4cf5-a5d1-117516baef4e"}, { new Terminator(OperatorType.Minus), "e2c085de-e99b-4f3a-b869-c3ca5f5dcf75"},}, new Dictionary<Terminator, ReduceInformation>{ }, "96a56a50-685f-4ee2-a15c-0246a192968f") },
		{ "f7212a1f-121f-44ff-9b3a-8784208708c6", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Else), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "f7212a1f-121f-44ff-9b3a-8784208708c6") },
		{ "c25b430d-af68-4f3a-93c6-97f4be6fd6fa", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Else), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "c25b430d-af68-4f3a-93c6-97f4be6fd6fa") },
		{ "02a4c650-94e3-411c-8fc1-7193c2459346", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Else), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, }, "02a4c650-94e3-411c-8fc1-7193c2459346") },
		{ "7c2ba50c-06bd-454a-abd5-f78ee0b8ceb5", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Else), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "7c2ba50c-06bd-454a-abd5-f78ee0b8ceb5") },
		{ "0296cd89-e186-4c9d-8b7e-1d843177fdc7", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(2, new NonTerminator(NonTerminatorType.ElsePart))}, { new Terminator(KeywordType.Else), new ReduceInformation(2, new NonTerminator(NonTerminatorType.ElsePart))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.ElsePart))}, }, "0296cd89-e186-4c9d-8b7e-1d843177fdc7") },
		{ "c6e730de-5667-4775-ab15-2f863fec9e1a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(KeywordType.Do), "6b17a47d-9025-45c7-be51-d04f70a7003b"},}, new Dictionary<Terminator, ReduceInformation>{ }, "c6e730de-5667-4775-ab15-2f863fec9e1a") },
		{ "3d7f2f8e-b029-42e3-ad84-d559138bfa13", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Term), "97990957-fb27-470c-87e6-26dfe1369ca7"}, { new NonTerminator(NonTerminatorType.Factor), "37bd1654-8a38-4f2b-8226-1d53978d984f"}, { Terminator.NumberTerminator, "d67e3da2-14db-461f-adf6-2a759c54c2a4"}, { new NonTerminator(NonTerminatorType.Variable), "3dd275fe-d8e6-4845-a991-b88b0815f1a3"}, { new Terminator(DelimiterType.LeftParenthesis), "a3bd2d83-bb0a-4e75-a1f8-832a87446417"}, { Terminator.IdentifierTerminator, "e0faf7c3-b2f6-4bb4-aaa0-6c9848949555"}, { new Terminator(KeywordType.Not), "1cd0f7bc-43f3-4895-9214-37758c95de66"}, { new Terminator(OperatorType.Minus), "88502125-f1fb-44ad-994f-9d7283ecda6d"},}, new Dictionary<Terminator, ReduceInformation>{ }, "3d7f2f8e-b029-42e3-ad84-d559138bfa13") },
		{ "f1a01b65-0d45-4b82-b73d-05b38553d94f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "85ec30ab-6db6-44a2-8543-e9d36a08bda5"}, { Terminator.NumberTerminator, "d67e3da2-14db-461f-adf6-2a759c54c2a4"}, { new NonTerminator(NonTerminatorType.Variable), "3dd275fe-d8e6-4845-a991-b88b0815f1a3"}, { new Terminator(DelimiterType.LeftParenthesis), "a3bd2d83-bb0a-4e75-a1f8-832a87446417"}, { Terminator.IdentifierTerminator, "e0faf7c3-b2f6-4bb4-aaa0-6c9848949555"}, { new Terminator(KeywordType.Not), "1cd0f7bc-43f3-4895-9214-37758c95de66"}, { new Terminator(OperatorType.Minus), "88502125-f1fb-44ad-994f-9d7283ecda6d"},}, new Dictionary<Terminator, ReduceInformation>{ }, "f1a01b65-0d45-4b82-b73d-05b38553d94f") },
		{ "f17107e8-6867-4d9c-8a47-6f724092d214", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "181480dc-cfad-433b-a9a2-381090bd7139"},}, new Dictionary<Terminator, ReduceInformation>{ }, "f17107e8-6867-4d9c-8a47-6f724092d214") },
		{ "87a88465-fae6-4eeb-bce5-4124a4da8dca", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "ae315d82-37bf-42ad-9d17-90ec0bf55dbf"}, { new NonTerminator(NonTerminatorType.Expression), "a582ab7f-9aa1-4130-9df3-2ba9dd0fe6f3"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "c3098ddd-d7d7-4d8b-b9ab-1ebedac5467f"}, { new NonTerminator(NonTerminatorType.Term), "c83de3f5-4130-4b2e-8715-e72b2e35275e"}, { new NonTerminator(NonTerminatorType.Factor), "9c184da0-a0b4-4ae3-9b1e-93b0fe09fbf1"}, { Terminator.NumberTerminator, "eaf4d875-7318-4f6e-b25b-829d30a888b8"}, { new NonTerminator(NonTerminatorType.Variable), "eb827526-1aa2-47ab-8b01-b1b0792e4297"}, { new Terminator(DelimiterType.LeftParenthesis), "2894d0f9-3faf-4be5-b2b6-c499611fc58d"}, { Terminator.IdentifierTerminator, "8efce527-7ba0-435f-a58f-b048a7a0e505"}, { new Terminator(KeywordType.Not), "dd22d779-6929-400f-832a-6d4e7eda69e6"}, { new Terminator(OperatorType.Minus), "28fb603c-e1e9-4bfa-ab9b-f8887e1996c0"},}, new Dictionary<Terminator, ReduceInformation>{ }, "87a88465-fae6-4eeb-bce5-4124a4da8dca") },
		{ "d03129cd-2bca-4342-a171-bb2c9cc5f8ca", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, }, "d03129cd-2bca-4342-a171-bb2c9cc5f8ca") },
		{ "e5ff139e-e61b-4571-8efc-30a2c0c37c74", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "bd560555-17b5-413d-afd4-f2d42829120a"}, { new NonTerminator(NonTerminatorType.Expression), "10204ac0-c523-4b6c-890d-d508a8e83f48"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "72d6e812-1720-408a-ac1c-5ac20fa267d1"}, { new NonTerminator(NonTerminatorType.Term), "0b3b44d2-8458-48d1-b7c4-2329d479b56c"}, { new NonTerminator(NonTerminatorType.Factor), "2f595e0e-c29e-494d-a5bb-a570db3254fd"}, { Terminator.NumberTerminator, "c86f637c-4e17-4d3d-bfd7-f1fc5abdf7f4"}, { new NonTerminator(NonTerminatorType.Variable), "fa555a17-89d0-4cf7-989a-0b0775ade6d4"}, { new Terminator(DelimiterType.LeftParenthesis), "8fab798a-03a8-45b7-8380-7b7eaee651fa"}, { Terminator.IdentifierTerminator, "513b9c84-fc68-4334-976d-7547b1ec521f"}, { new Terminator(KeywordType.Not), "928a980c-85a8-4cf5-a5d1-117516baef4e"}, { new Terminator(OperatorType.Minus), "e2c085de-e99b-4f3a-b869-c3ca5f5dcf75"},}, new Dictionary<Terminator, ReduceInformation>{ }, "e5ff139e-e61b-4571-8efc-30a2c0c37c74") },
		{ "8f08beb2-d5cb-430b-95a7-5fb0770e5909", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "8f08beb2-d5cb-430b-95a7-5fb0770e5909") },
		{ "f2c394a9-4994-4129-970d-22bd172092f5", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "f2c394a9-4994-4129-970d-22bd172092f5") },
		{ "d484e87f-98cd-4163-81d3-ca26ad405a37", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, }, "d484e87f-98cd-4163-81d3-ca26ad405a37") },
		{ "9d7e18b2-f146-4bf3-8f6c-b773391d5bc5", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "9d7e18b2-f146-4bf3-8f6c-b773391d5bc5") },
		{ "7b48ff10-7621-438e-8883-11146ef07997", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, }, "7b48ff10-7621-438e-8883-11146ef07997") },
		{ "1e6b291c-de7d-4494-a2a7-569ddc984d86", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "1e6b291c-de7d-4494-a2a7-569ddc984d86") },
		{ "509f6d3f-9a85-4d41-96f7-f9b45467598e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "2f86a129-1871-4334-8940-3da8f159d4b9"}, { new Terminator(OperatorType.Multiply), "fd20e81d-52b7-4920-adab-e2a1e49a6b6a"}, { new Terminator(OperatorType.Divide), "ef7280a1-85cf-4590-ab87-68bdd867d9e5"}, { new Terminator(KeywordType.Divide), "9192ebb2-aa8a-414d-ac9e-7206883933e6"}, { new Terminator(KeywordType.Mod), "ae32e061-59db-49fd-addd-2d5c3e9014a3"}, { new Terminator(KeywordType.And), "5c4b371b-c686-4545-b641-4c196ca6b25f"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Else), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "509f6d3f-9a85-4d41-96f7-f9b45467598e") },
		{ "563c7fab-ed60-49a6-bd96-763ac22e12d5", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Else), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, }, "563c7fab-ed60-49a6-bd96-763ac22e12d5") },
		{ "79b2f934-f2c7-4006-b3ca-a96dac34640f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Else), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "79b2f934-f2c7-4006-b3ca-a96dac34640f") },
		{ "4cb799fe-5e00-424f-9654-5c6c1759fc1e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "6de0b275-cc63-4dac-a70b-abb8dde92bae"}, { new Terminator(DelimiterType.Comma), "dd4993c2-1db9-4e5a-a3f2-c4660ec4e365"},}, new Dictionary<Terminator, ReduceInformation>{ }, "4cb799fe-5e00-424f-9654-5c6c1759fc1e") },
		{ "cec4bffe-a1da-47ec-b752-83552c0a0949", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightSquareBracket), "5893707d-82d4-4412-946f-2a9bca0ec850"}, { new Terminator(DelimiterType.Comma), "ecf84275-ae5c-4a59-96fb-d2ef13a595a8"},}, new Dictionary<Terminator, ReduceInformation>{ }, "cec4bffe-a1da-47ec-b752-83552c0a0949") },
		{ "6b17a47d-9025-45c7-be51-d04f70a7003b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Statement), "7919d550-f96b-4032-99d6-1d8795ae45dc"}, { new NonTerminator(NonTerminatorType.Variable), "c5d258c5-490b-4c90-8b5b-8242a6dbfc6d"}, { Terminator.IdentifierTerminator, "9a2b8a88-a48e-46e2-af69-f3561ad39bb7"}, { new NonTerminator(NonTerminatorType.ProcedureCall), "e9f86400-0193-40f4-a076-09e7b9b028e9"}, { new NonTerminator(NonTerminatorType.CompoundStatement), "48fe99c9-db82-494e-b751-8bda645590d0"}, { new Terminator(KeywordType.If), "79db3815-bc73-4868-ad20-1fc22d7453b9"}, { new Terminator(KeywordType.For), "ec4898d0-5ba7-47f1-885e-07725f2e43dc"}, { new Terminator(KeywordType.Begin), "65fb1933-38c9-42db-9907-36fe3c05573a"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(KeywordType.Else), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, }, "6b17a47d-9025-45c7-be51-d04f70a7003b") },
		{ "97990957-fb27-470c-87e6-26dfe1369ca7", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "f1a01b65-0d45-4b82-b73d-05b38553d94f"}, { new Terminator(OperatorType.Multiply), "fd20e81d-52b7-4920-adab-e2a1e49a6b6a"}, { new Terminator(OperatorType.Divide), "ef7280a1-85cf-4590-ab87-68bdd867d9e5"}, { new Terminator(KeywordType.Divide), "9192ebb2-aa8a-414d-ac9e-7206883933e6"}, { new Terminator(KeywordType.Mod), "ae32e061-59db-49fd-addd-2d5c3e9014a3"}, { new Terminator(KeywordType.And), "5c4b371b-c686-4545-b641-4c196ca6b25f"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "97990957-fb27-470c-87e6-26dfe1369ca7") },
		{ "85ec30ab-6db6-44a2-8543-e9d36a08bda5", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, }, "85ec30ab-6db6-44a2-8543-e9d36a08bda5") },
		{ "181480dc-cfad-433b-a9a2-381090bd7139", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "181480dc-cfad-433b-a9a2-381090bd7139") },
		{ "ae315d82-37bf-42ad-9d17-90ec0bf55dbf", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "80f31d0e-45f4-4dd3-8e12-c4a3d1bfb091"}, { new Terminator(DelimiterType.Comma), "dd4993c2-1db9-4e5a-a3f2-c4660ec4e365"},}, new Dictionary<Terminator, ReduceInformation>{ }, "ae315d82-37bf-42ad-9d17-90ec0bf55dbf") },
		{ "bd560555-17b5-413d-afd4-f2d42829120a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightSquareBracket), "71d5e2a7-60c9-47c0-9cf3-e2fa46e3a52e"}, { new Terminator(DelimiterType.Comma), "ecf84275-ae5c-4a59-96fb-d2ef13a595a8"},}, new Dictionary<Terminator, ReduceInformation>{ }, "bd560555-17b5-413d-afd4-f2d42829120a") },
		{ "6de0b275-cc63-4dac-a70b-abb8dde92bae", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Else), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, }, "6de0b275-cc63-4dac-a70b-abb8dde92bae") },
		{ "5893707d-82d4-4412-946f-2a9bca0ec850", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Else), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "5893707d-82d4-4412-946f-2a9bca0ec850") },
		{ "7919d550-f96b-4032-99d6-1d8795ae45dc", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(8, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(KeywordType.Else), new ReduceInformation(8, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(8, new NonTerminator(NonTerminatorType.Statement))}, }, "7919d550-f96b-4032-99d6-1d8795ae45dc") },
		{ "80f31d0e-45f4-4dd3-8e12-c4a3d1bfb091", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, }, "80f31d0e-45f4-4dd3-8e12-c4a3d1bfb091") },
		{ "71d5e2a7-60c9-47c0-9cf3-e2fa46e3a52e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "71d5e2a7-60c9-47c0-9cf3-e2fa46e3a52e") },
	};

    private GeneratedGrammarParser()
    {
        foreach(GeneratedTransformer transformer in s_transformers.Values)
        {
            transformer.ConstructShiftTable(s_transformers);
        }
    }

    private static GeneratedGrammarParser s_instance = new GeneratedGrammarParser();

    public static GeneratedGrammarParser Instance => s_instance;

	public ITransformer BeginTransformer => s_transformers["1943566a-342f-47b2-b2fe-f9cd93e7301b"];
	public NonTerminator Begin => new NonTerminator(NonTerminatorType.StartNonTerminator);
}
